function [u_mon_p_f,varargout] = acoustic_p_gfs_direct_f(freq,c,srx,rxs,dim,srctype,varargin)

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% This code is distributed as part of the source-code package 
%                   acoustic_modelling_foldy_v.4.1_seg 
% that accompanies Galetti et al. (2013). The package can be downloaded 
% from the Geophysics source-code archive at 
%                   http://software.seg.org/2013/0003
% Use of this code is subject to acceptance of the terms and conditions
% that can be found at http://software.seg.org/disclaimer.txt 
% Copyright 2013 by The Society of Exploration Geophysicists (SEG)
% Reference:
% Galetti, E., Halliday, D., Curtis, A. (2013) A simple and exact acoustic 
% wavefield modelling code for data processing, imaging and interferometry 
% applications. Geophysics.
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% [u_mon_p_f,varargout] = acoustic_p_gfs_direct_f(freq,c,srx,rxs,ndim,srctype,varargin)
%
% Models the direct Green's functions in a homogeneous acoustic medium
% due to monopole sources and dipole (derivative) sources.
%
% Results are given in the frequency domain and no source wavelet is applied.
%
% Input:
%   freq        frequencies to model
%   c           velocity of the medium (in m/s)
%   srx         location of sources (matrix with x-coords in 1st column, y-coords in 2nd column, z-coords in 3rd column)
%   rxs         location of receivers (matrix with x-coords in 1st column, y-coords in 2nd column, z-coords in 3rd column)
%   ndim        number of dimensions
%   srctype     type of source
%                   'volinjpos'     positive volume injection ["Modified Green's function" section of Wapenaar & Fokkema (2006)]
%                   'volinjneg'     negative volume injection [van Manen et al. (2005)]
%                   'volinjratepos' positive volume injection rate [Wapenaar & Fokkema (2006)]
%                   'volinjrateneg' not implemented in interferometry (yet)
%   diploc      compute derivative of the Green's function...
%                   'srcdip'        ...with respect to sources (as in inter-receiver interferometry)
%                   'recdip'        ...with respect to receivers (as in inter-source interferometry)
%
% Output:
%   u_mon_p_f       monopole - pressure (G)
%   u_dip_p_f       dipole - pressure (x, y, z derivatives of G)
%
%
% References:
%       Snieder, R. (2009) A Guided Tour of Mathematical Methods for the
%   Physical Sciences, 2nd ed., Cambridge University Press.
%       van Manen, D.-J., Robertsson, J.O.A., Curtis, A. (2005) Modeling of
%   Wave Propagation in Inhomogeneous Media. Physical Review Letters, 94:
%   164301.
%       Wapenaar, K., Fokkema, J. (2006) Green's function representations
%   for seismic interferometry, Geophysics, 71: SI33-SI46.
%
%
% Script written by Erica Galetti (EG) based on similar scripts received
% from David Halliday (DH) between July 2011 and January 2012.
%
%
% Erica Galetti, July 2012
% Contact: erica.galetti@ed.ac.uk


% Get input from varargin
if nargin == 7 && nargout == 2
    diploc = varargin{1};
    if ~strcmp(diploc,'srcdip') && ~strcmp(diploc,'recdip')
        error('Invalid input for derivative location. Choose between ''srcdip'' and ''recdip''.')
    end
elseif nargin ~= 7 && nargout == 2
    error('Need to specify derivative location if derivatives are to be evaluated.')
elseif nargin == 7 && nargout ~= 2
    error('Need to specify derivative output if derivatives are to be evaluated.')
end


% Get total number of sources and receivers
nsrx = length(srx(:,1));
nrxs = length(rxs(:,1));


% Get number of dimensions
if strcmp(dim,'1d'); ndim = 1;
elseif strcmp(dim,'2d') || strcmp(dim,'2d_far'); ndim = 2;
elseif strcmp(dim,'3d'); ndim = 3;
end


% Determine the offsets and direction cosines for each source-receiver pair
% The direction cosines give the cos and sin of the angle between each ray
% (source-receiver direction) and the x axis
offs = zeros(nsrx,nrxs);
dircos = zeros(nsrx,nrxs,ndim);
for ss = 1:nsrx
    for rr = 1:nrxs
        if ndim == 1; offs(ss,rr) = sqrt((srx(ss,1)-rxs(rr,1))^2);
        elseif ndim == 2; offs(ss,rr) = sqrt((srx(ss,1)-rxs(rr,1))^2+(srx(ss,2)-rxs(rr,2))^2);
        elseif ndim == 3; offs(ss,rr) = sqrt((srx(ss,1)-rxs(rr,1))^2+(srx(ss,2)-rxs(rr,2))^2+(srx(ss,3)-rxs(rr,3))^2);
        end
        if exist('diploc','var')
            if offs(ss,rr) ~= 0;
                if strcmp(diploc,'srcdip')      % Receiver at the origin - derivative with respect to source (as in inter-receiver interferometry, with boundary of sources)
                    if ndim == 1; dircos(ss,rr,:) = -(rxs(rr,1)-srx(ss,1))./offs(ss,rr);
                    elseif ndim == 2; dircos(ss,rr,:) = -[(rxs(rr,1)-srx(ss,1))./offs(ss,rr),(rxs(rr,2)-srx(ss,2))./offs(ss,rr)];
                    elseif ndim == 3; dircos(ss,rr,:) = -[(rxs(rr,1)-srx(ss,1))./offs(ss,rr),(rxs(rr,2)-srx(ss,2))./offs(ss,rr),(rxs(rr,3)-srx(ss,3))./offs(ss,rr)];
                    end
                elseif exist('diploc','var') && strcmp(diploc,'recdip')  % Source at the origin - derivative with respect to receiver (as in inter-source interferometry, with boundary of receivers)
                    if ndim == 1; dircos(ss,rr,:) = (rxs(rr,1)-srx(ss,1))./offs(ss,rr);
                    elseif ndim == 2; dircos(ss,rr,:) = [(rxs(rr,1)-srx(ss,1))./offs(ss,rr),(rxs(rr,2)-srx(ss,2))./offs(ss,rr)];
                    elseif ndim == 3; dircos(ss,rr,:) = [(rxs(rr,1)-srx(ss,1))./offs(ss,rr),(rxs(rr,2)-srx(ss,2))./offs(ss,rr),(rxs(rr,3)-srx(ss,3))./offs(ss,rr)];
                    end
                end
            end
        end
    end
end

nf = length(freq); % number of frequencies
freq=reshape(freq,nf,1);
om = 2*pi*freq; % angular frequency = 2*pi*frequency
kp = om./c; % wavenumber = angular frequency / velocity


% Allocate space for results
u_mon_p_f = zeros(nf,nrxs,nsrx);
if exist('diploc','var')
    dgf = zeros(nf,nrxs,1,ndim);
    u_dip_p_f = zeros(nf,nrxs,nsrx,ndim);
end


% Loop over sources - calculate responses from each source separately
for ss = 1:nsrx
    
    % Calculate propagation terms for each frequency - monopole source,
    % pressure response.
    % Formulae from Snieder (2009), p. 302, equation (19.43), with a few
    % changes:
    % - the extra minus sign inside the exponential is due to a different
    %   sign convention in the Fourier transform - Matlab uses exp(i*om*t)
    %   from frequency to time domain, rather than exp(-i*om*t)
    % - i has been taken inside the exponential in the 1D and 2D_far cases
    % - the minus sign in front of the expression indicates that the
    %   Green's function is solution to the Helmoltz equation with minus
    %   delta on the right-hand side, which is what Wapenaar & Fokkema use
    %   (rather than plus delta which is what is used in Snieder (2002) and
    %   van Manen et al. (2005))
    if ndim == 1
        % Green's function in 1 dimension
        gf = -1./(2.*(repmat(kp,[1 nrxs]))).*exp(-1i.*(repmat(kp,[1 nrxs]).*repmat(offs(ss,:),[nf 1])-pi./2));
    elseif ndim == 2
        % Green's function in 2 dimensions - uses the Hankel function of
        % the 2nd kind and order 0
        if strcmp(dim,'2d')
            gf = -1i./4.*besselh(0,2,repmat(kp,[1 nrxs]).*repmat(offs(ss,:),[nf 1]));
        elseif strcmp(dim,'2d_far')
            % Far-field approximation
            gf = -1./4.*exp(-1i.*(repmat(kp,[1 nrxs]).*repmat(offs(ss,:),[nf 1])-3.*pi./4)).*sqrt(2./(pi.*repmat(kp,[1 nrxs]).*repmat(offs(ss,:),[nf 1])));
        end
    elseif ndim == 3
        % Green's function in 3 dimensions
        gf = 1./(4.*pi.*repmat(offs(ss,:),[nf 1])).*exp(-1i.*(repmat(kp,[1 nrxs]).*repmat(offs(ss,:),[nf 1])));
    end
    
    if strcmp(srctype,'volinjpos')
        u_mon_p_f(:,:,ss) = gf;                                 % Wapenaar and Fokkema (2006) - volume injection source (see "Modified Green's function" section)
    elseif strcmp(srctype,'volinjratepos')
        u_mon_p_f(:,:,ss) = 1i.*(repmat(om,[1 nrxs])).*gf;      % Wapenaar and Fokkema (2006) - volume injection rate source
    elseif strcmp(srctype,'volinjneg')
        u_mon_p_f(:,:,ss) = -gf;                                % van Manen et al. (2005) - volume injection source
    elseif strcmp(srctype,'volinjrateneg')
        u_mon_p_f(:,:,ss) = -1i.*(repmat(om,[1 nrxs])).*gf;     % Not implemented in interferometry (yet), but follows naturally from the van Manen et al. (2005) approach
    end
    
    
    % Calculate pressure responses for dipole sources by calculating the
    % derivatives of the monopole response along the x, y and z directions
    if exist('diploc','var')
        if ndim == 1
            % Analytical derivative of the 1D formula
            u_dip_p_f(:,:,ss,1) = -1i.*repmat(kp,[1 nrxs]).*u_mon_p_f(:,:,ss).*repmat(dircos(ss,:,1),[nf 1]); % x-derivative of G - to be multiplied by x-component of boundary normal in interferometry
        elseif ndim == 2
            if strcmp(dim,'2d')
                % Derivative of the exact 2D formula - uses the Hankel
                % function of the 2nd kind and order 1
                for dd = 1:ndim; dgf(:,:,1,dd) = 1i.*(repmat(kp,[1 nrxs]))./4.*(repmat(dircos(ss,:,dd),[nf 1])).*besselh(1,2,repmat(kp,[1 nrxs]).*repmat(offs(ss,:),[nf 1])); end % dd-derivative of G - to be multiplied by dd-component of boundary normal in interferometry
                if strcmp(srctype,'volinjpos')
                    u_dip_p_f(:,:,ss,:) = dgf;                              % Wapenaar and Fokkema (2006) - volume injection source (see "Modified Green's function" section)
                elseif strcmp(srctype,'volinjratepos')
                    u_dip_p_f(:,:,ss,:) = 1i.*(repmat(om,[1 nrxs 1 ndim])).*dgf;   % Wapenaar and Fokkema (2006) - volume injection rate source
                elseif strcmp(srctype,'volinjneg')
                    u_dip_p_f(:,:,ss,:) = -dgf;                             % van Manen et al. (2005) - volume injection source
                elseif strcmp(srctype,'volinjrateneg')
                    u_dip_p_f(:,:,ss,:) = -1i.*(repmat(om,[1 nrxs 1 ndim])).*dgf;  % Not implemented in interferometry (yet), but follows naturally from the van Manen et al. (2005) approach
                end
            elseif strcmp(dim,'2d_far')
                % Analytical derivative of the 2D far-field formula
                for dd = 1:ndim; u_dip_p_f(:,:,ss,dd) = u_mon_p_f(:,:,ss).*(repmat(dircos(ss,:,dd),[nf 1])).*(-1i.*repmat(kp,[1 nrxs])-1./(2.*repmat(offs(ss,:),[nf 1]))); end % dd-derivative of G - to be multiplied by dd-component of boundary normal in interferometry
            end
        elseif ndim == 3
            % Analytical derivative of the 3D formula
            for dd = 1:ndim; u_dip_p_f(:,:,ss,dd) = u_mon_p_f(:,:,ss).*(repmat(dircos(ss,:,dd),[nf 1])).*(-1i.*repmat(kp,[1 nrxs])-1./repmat(offs(ss,:),[nf 1])); end % dd-derivative of G - to be multiplied by dd-component of boundary normal in interferometry
        end
        varargout{1} = u_dip_p_f;
    end
    
end

end
