function [varargout] = model_gfs_p_scat_acoust(srx,rxs,maxfreq,nfreq,c,dim,srctype,diploc,scatflag,wavlflag,freqflag,timeflag,sepflag,varargin)

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% This code is distributed as part of the source-code package 
%                   acoustic_modelling_foldy_v.4.1_seg 
% that accompanies Galetti et al. (2013). The package can be downloaded 
% from the Geophysics source-code archive at 
%                   http://software.seg.org/2013/0003
% Use of this code is subject to acceptance of the terms and conditions
% that can be found at http://software.seg.org/disclaimer.txt 
% Copyright 2013 by The Society of Exploration Geophysicists (SEG)
% Reference:
% Galetti, E., Halliday, D., Curtis, A. (2013) A simple and exact acoustic 
% wavefield modelling code for data processing, imaging and interferometry 
% applications. Geophysics.
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% [varargout] = model_gfs_p_scat_acoust(srx,rxs,maxfreq,nfreq,c,dim,srctype,diploc,scatflag,wavlflag,freqflag,timeflag,sepflag,varargin)
%
% Models the direct and scattered acoustic Green's functions (pressure) in
% a homogeneous acoustic medium emitted/recorded by monopole and dipole
% (derivative) sources/receivers.
% 
% Outputs can be given in either or both of the frequency and time domains,
% as total wavefield (sum of direct and scattered wavefield) or as total
% and separate direct and scattered wavefields.
% 
% The output also includes a frequency and a time vector, and, if a source
% wavelet is applied, the wavelet in either or both of frequency and time
% domains.
% 
% For other options, use the optional inputs/outputs.
% 
% Input:
%   srx         location of sources in matrix having:
%                   - 1st column: x-coords
%                   - 2nd column: y-coords (ignored by code if dim set to '1d')
%                   - 3rd column: z-coords (ignored by code if dim set to '1d' or '2d')
%   rxs         location of sources in matrix having:
%                   - 1st column: x-coords
%                   - 2nd column: y-coords (ignored by code if dim set to '1d')
%                   - 3rd column: z-coords (ignored by code if dim set to '1d' or '2d')
%   maxfreq     maximum frequency to model
%   nfreq       number of frequencies to model (the code rounds this up to 
%               the nearest power of two – go to lines 277-278 to change this)
%   c           velocity of the medium (in m/s)
%   dim         number of dimensions - important for geometrical spreading
%                   '1d'            uses the 1D Green's function formula 
%                   '2d'            uses the 2D Green's function formula
%                   '2d_far'        uses the far-field approximation for the 2D Green's function formula 
%                   '3d'            uses the 3D Green's function formula
%                 N.B. If '1d' is selected, code will ignore srx, rxs and
%                    scats coords in 2nd and 3rd column; if '2d' is
%                    selected, code will ignore srx, rxs and scats coords
%                    in 3rd column.
%   srctype     select source type:
%                   'volinjpos'     positive volume injection
%                                   ["Modified Green's function" section of 
%                                   Wapenaar & Fokkema (2006)]
%                   'volinjneg'     negative volume injection
%                                   [van Manen et al. (2005)]
%                   'volinjratepos' positive volume injection rate
%                                   [Wapenaar & Fokkema (2006)]
%                   'volinjrateneg' negative volume injection rate
%                                   [not implemented in interferometry (yet)]
%   diploc      compute the dipole (derivative) Green's function...
%                   'srcdip'        ...with respect to sources (as in inter-receiver interferometry)
%                   'recdip'        ...with respect to receivers (as in inter-source interferometry)
%   scatflag    are there any scatterers?
%                   'scatn' no
%                   'scaty' yes
%   wavlflag    apply source wavelet?
%                   'wavln'         no
%                   'wavly_pos'     yes, with Ricker wavelet at positive times
%                   'wavly_centr'   yes, with Ricker wavelet centred on zero time
%                   'wavly_neg'     yes, with Ricker wavelet at negative times
%                   'wavly_shift'   yes, with Ricker wavelet shifted by ts seconds from zero
%   freqflag    output in frequency domain?
%                   'freqn'     no frequency-domain output
%                   'freqy_1nf' output frequencies in range [1:nf]
%                   'freqy_1s'  output positive frequency spectrum [0:nf]
%                   'freqy_2s'  output double-sided frequency spectrum [-nf:-1 0 1:nf-1]
%   timeflag    output in time domain?
%                   'timen' no time-domain output
%                   'timey' output wavefields in time domain 
%   sepflag     separate direct and scattered wavefield in distinct outputs?
%                   'tot' output total (sum of direct and scattered)
%                   'dir' output total and direct
%                   'scat' output total and scattered
%                   'dirscat' output total, direct and scattered
% 
% Varargin:
%   scats       location of scatterers in matrix having:
%                   - 1st column: x-coords
%                   - 2nd column: y-coords (ignored by code if dim set to '1d')
%                   - 3rd column: z-coords (ignored by code if dim set to '1d' or '2d')
%                   - 4th column: imaginary part of scattering amplitude
%                                 if Im(A) independent of frequency
%                   - columns 4 to nfreq+3: imaginary part of scattering amplitude
%                                           if Im(A) depends on frequency -
%                                           increasing frequencies along dim2
%                                           from column 4 to column 3+nfreq,
%                                           where nfreq is the number of
%                                           frequencies modelled
%                   (if no scatterers, script will evaluate direct Green's functions only)
%   fc          central frequency of Ricker wavelet
%   ts          time-shift (in s) of peak of Ricker wavelet (only needed if wavlflag is set to 'wavly_shift';
%               gets rounded up to ceil((1/fc)/dt)*dt by the code)
% 
% Output (varargout):
%   frequencies         frequency vector matching the frequencies of the modelled wavefields
%   gf_mon_p_f          monopole source/receiver, total wavefield, frequency domain
%   gf_dip_p_f          dipole source/receiver, total wavefield, frequency domain
%   gfdir_mon_p_f       monopole source/receiver, direct wavefield, frequency domain
%   gfdir_dip_p_f       dipole source/receiver, direct wavefield, frequency domain
%   gfscat_mon_p_f      monopole source/receiver, scattered wavefield, frequency domain
%   gfscat_dip_p_f      dipole source/receiver, scattered wavefield, frequency domain
%   time                time vector matching the time domain outputs
%   gf_mon_p_t          monopole source/receiver, total wavefield, time domain
%   gf_dip_p_t          dipole source/receiver, total wavefield, time domain
%   gfdir_mon_p_t       monopole source/receiver, direct wavefield, time domain
%   gfdir_dip_p_t       dipole source/receiver, direct wavefield, time domain
%   gfscat_mon_p_t      monopole source/receiver, scattered wavefield, time domain
%   gfscat_dip_p_t      dipole source/receiver, scattered wavefield, time domain
%   wavelet_f           source wavelet in frequency domain (if wavlflag = 'wavly')
%   wavelet_t           source wavelet in time domain (if wavlflag = 'wavly')
%                       N.B. the centred wavelet has positive time in
%                       sample range [1:nf], reverse negative time in
%                       sample range [nf+1:2*nf]
% 
%
% Required:
%   acoustic_p_gfs_direct_f.m
%   rickerwavl_time.m
%   centerfreqs.m
%
% References:
%       Groenenboom, J., Snieder, R. (1995) Attenuation, dispersion, and
%   anisotropy by multiple scattering of transmitted waves through
%   distributions of scatterers. J. Acoust. Soc. Am. 98 (6): 3482-3492.
%       Snieder, R. (1999) Imaging and averaging in complex media, in
%   Diffuse waves in complex media, edited by J.P. Fouque, pp. 405-454,
%   Kluwer, Dordrecht
%
%
% Code written by Erica Galetti (EG) based on similar scripts received
% from David Halliday (DH) between July 2011 and January 2012.
%
%
% Erica Galetti, September 2012
% Contact: erica.galetti@ed.ac.uk



%% Check input/output is correct

% Error messages for invalid flags
if ~strcmp(dim,'1d') && ~strcmp(dim,'2d') && ~strcmp(dim,'2d_far') && ~strcmp(dim,'3d')
    error('Invalid input for dimension. Choose between ''1d'', ''2d'', ''2d_far'', ''3d''.')
elseif ~strcmp(srctype,'volinjpos') && ~strcmp(srctype,'volinjratepos') && ~strcmp(srctype,'volinjneg') && ~strcmp(srctype,'volinjrateneg')
    error('Invalid input for source type. Choose between ''volinjpos'', ''volinjratepos'', ''volinjneg'', ''volinjrateneg''.')
elseif ~strcmp(diploc,'srcdip') && ~strcmp(diploc,'recdip')    
    error('Invalid input for derivative location. Choose between ''srcdip'' and ''recdip''.')
elseif ~strcmp(scatflag,'scaty') && ~strcmp(scatflag,'scatn')
    error('Invalid input for scattering flag. Choose between ''scaty'' and ''scatn''.')
elseif ~strcmp(wavlflag,'wavly_pos') && ~strcmp(wavlflag,'wavly_centr') && ~strcmp(wavlflag,'wavly_neg') && ~strcmp(wavlflag,'wavly_shift') && ~strcmp(wavlflag,'wavln')
    error('Invalid input for wavelet flag. Choose between ''wavly_pos'', ''wavly_centr'', ''wavly_neg'', ''wavly_shift'' and ''wavln''.')
elseif ~strcmp(freqflag,'freqy_1nf') && ~strcmp(freqflag,'freqy_1s') && ~strcmp(freqflag,'freqy_2s') && ~strcmp(freqflag,'freqn')
    error('Invalid input for frequency output flag. ''Choose between ''freqy_1nf'', ''freqy_1s'', ''freqy_2s'' and ''freqn''.')
elseif ~strcmp(timeflag,'timen') && ~strcmp(timeflag,'timey')
    error('Invalid input for time output flag. Choose between ''timey'' and ''timen''.')
elseif ~strcmp(sepflag,'tot') && ~strcmp(sepflag,'dir') && ~strcmp(sepflag,'scat') && ~strcmp(sepflag,'dirscat')
    error('Invalid input for separate output flag. Choose between ''tot'', ''dir'', ''scat'', ''dirscat''.')
end

% Error message if trying to get separate direct and scattered output for
% no scattering
if strcmp(scatflag,'scatn') && ~strcmp(sepflag,'tot')
    error('For no scattering, the total wavefield corresponds to the direct wavefield. Set sepflag to ''tot'' if scatflag is ''scatn''.')
end

% Error message if both freqflag and timeflag are set to 'no'
if strcmp(freqflag,'freqn') && strcmp(timeflag,'timen')
    error('Both freqflag and timeflag are set to ''no'' - i.e. no output! Set either freqflag to ''freqy_'' or timeflag to ''timey'' to get some output.')
end

% Error messages if inputs and outputs don't match
if strcmp(freqflag,'freqn')
    if strcmp(wavlflag,'wavln')
        if strcmp(scatflag,'scatn')
            if nargin~=13 || nargout~=3; error('Error:inputoutput','Check inputs/outputs are correct!\nWith the current input flags, there should be 13 inputs and 3 outputs and the function call should be something like:\n  [time,gf_mon_p_t,gf_dip_p_t] = model_gfs_p_scat_acoust(srx,rxs,maxfreq,nfreq,c,''1d''/''2d''/''3d'',''volinjpos/neg''/''volinjratepos/neg'',''srcdip''/''recdip'',''scatn'',''wavln'',''freqn'',''timey'',''tot'')'); end
        elseif strcmp(scatflag,'scaty')
            if strcmp(sepflag,'tot')
                if nargin~=14 || nargout~=3; error('Error:inputoutput','Check inputs/outputs are correct!\nWith the current input flags, there should be 14 inputs and 3 outputs and the function call should be something like:\n  [time,gf_mon_p_t,gf_dip_p_t] = model_gfs_p_scat_acoust(srx,rxs,maxfreq,nfreq,c,''1d''/''2d''/''3d'',''volinjpos/neg''/''volinjratepos/neg'',''srcdip''/''recdip'',''scaty'',''wavln'',''freqn'',''timey'',''tot'',scats)'); end
            elseif strcmp(sepflag,'dir') || strcmp(sepflag,'scat')
                if nargin~=14 || nargout~=5; error('Error:inputoutput','Check inputs/outputs are correct!\nWith the current input flags, there should be 14 inputs and 5 outputs and the function call should be something like:\n  [time,gf_mon_p_t,gf_dip_p_t,gfdir_mon_p_t,gfdir_dip_p_t] = model_gfs_p_scat_acoust(srx,rxs,maxfreq,nfreq,c,''1d''/''2d''/''3d'',''volinjpos/neg''/''volinjratepos/neg'',''srcdip''/''recdip'',''scaty'',''wavln'',''freqn'',''timey'',''dir'',scats)\nor\n  [time,gf_mon_p_t,gf_dip_p_t,gfscat_mon_p_t,gfscat_dip_p_t] = model_gfs_p_scat_acoust(srx,rxs,maxfreq,nfreq,c,''1d''/''2d''/''3d'',''volinjpos/neg''/''volinjratepos/neg'',''srcdip''/''recdip'',''scaty'',''wavln'',''freqn'',''timey'',''scat'',scats)'); end
            elseif strcmp(sepflag,'dirscat')
                if nargin~=14 || nargout~=7; error('Error:inputoutput','Check inputs/outputs are correct!\nWith the current input flags, there should be 14 inputs and 7 outputs and the function call should be something like:\n  [time,gf_mon_p_t,gf_dip_p_t,gfdir_mon_p_t,gfdir_dip_p_t,gfscat_mon_p_t,gfscat_dip_p_t] = model_gfs_p_scat_acoust(srx,rxs,maxfreq,nfreq,c,''1d''/''2d''/''3d'',''volinjpos/neg''/''volinjratepos/neg'',''srcdip''/''recdip'',''scaty'',''wavln'',''freqn'',''timey'',''dirscat'',scats)'); end
            end
        end
    elseif strcmp(wavlflag,'wavly_pos') || strcmp(wavlflag,'wavly_centr') || strcmp(wavlflag,'wavly_neg') || strcmp(wavlflag,'wavly_shift')
        if strcmp(scatflag,'scatn')
            if nargin~=14 || nargout~=4; error('Error:inputoutput','Check inputs/outputs are correct!\nWith the current input flags, there should be 14 inputs and 4 outputs and the function call should be something like:\n  [time,gf_mon_p_t,gf_dip_p_t,wavelet_t] = model_gfs_p_scat_acoust(srx,rxs,maxfreq,nfreq,c,''1d''/''2d''/''3d'',''volinjpos/neg''/''volinjratepos/neg'',''srcdip''/''recdip'',''scatn'',''wavly_pos/centr/neg'',''freqn'',''timey'',''tot'',fc)'); end
        elseif strcmp(scatflag,'scaty')
            if strcmp(sepflag,'tot')
                if nargin~=15 || nargout~=4; error('Error:inputoutput','Check inputs/outputs are correct!\nWith the current input flags, there should be 15 inputs and 4 outputs and the function call should be something like:\n  [time,gf_mon_p_t,gf_dip_p_t,wavelet_t] = model_gfs_p_scat_acoust(srx,rxs,maxfreq,nfreq,c,''1d''/''2d''/''3d'',''volinjpos/neg''/''volinjratepos/neg'',''srcdip''/''recdip'',''scaty'',''wavly_pos/centr/neg'',''freqn'',''timey'',''tot'',scats,fc)'); end
            elseif strcmp(sepflag,'dir') || strcmp(sepflag,'scat')
                if nargin~=15 || nargout~=6; error('Error:inputoutput','Check inputs/outputs are correct!\nWith the current input flags, there should be 15 inputs and 6 outputs and the function call should be something like:\n  [time,gf_mon_p_t,gf_dip_p_t,gfdir_mon_p_t,gfdir_dip_p_t,wavelet_t] = model_gfs_p_scat_acoust(srx,rxs,maxfreq,nfreq,c,''1d''/''2d''/''3d'',''volinjpos/neg''/''volinjratepos/neg'',''srcdip''/''recdip'',''scaty'',''wavly_pos/centr/neg'',''freqn'',''timey'',''dir'',scats,fc)\nor\n  [time,gf_mon_p_t,gf_dip_p_t,gfscat_mon_p_t,gfscat_dip_p_t,wavelet_t] = model_gfs_p_scat_acoust(srx,rxs,maxfreq,nfreq,c,''1d''/''2d''/''3d'',''volinjpos/neg''/''volinjratepos/neg'',''srcdip''/''recdip'',''scaty'',''wavly_pos/centr/neg'',''freqn'',''timey'',''scat'',scats,fc)'); end
            elseif strcmp(sepflag,'dirscat')
                if nargin~=15 || nargout~=8; error('Error:inputoutput','Check inputs/outputs are correct!\nWith the current input flags, there should be 15 inputs and 8 outputs and the function call should be something like:\n  [time,gf_mon_p_t,gf_dip_p_t,gfdir_mon_p_t,gfdir_dip_p_t,gfscat_mon_p_t,gfscat_dip_p_t,wavelet_t] = model_gfs_p_scat_acoust(srx,rxs,maxfreq,nfreq,c,''1d''/''2d''/''3d'',''volinjpos/neg''/''volinjratepos/neg'',''srcdip''/''recdip'',''scaty'',''wavly_pos/centr/neg'',''freqn'',''timey'',''dirscat'',scats,fc)'); end
            end
        end
    end
elseif ~strcmp(freqflag,'freqn')
    if strcmp(wavlflag,'wavln')
        if strcmp(scatflag,'scatn')
            if strcmp(timeflag,'timen')
                if nargin~=13 || nargout~=3; error('Error:inputoutput','Check inputs/outputs are correct!\nWith the current input flags, there should be 13 inputs and 3 outputs and the function call should be something like:\n  [frequencies,gf_mon_p_f,gf_dip_p_f] = model_gfs_p_scat_acoust(srx,rxs,maxfreq,nfreq,c,''1d''/''2d''/''3d'',''volinjpos/neg''/''volinjratepos/neg'',''srcdip''/''recdip'',''scatn'',''wavln'',''freqy_1nf''/''freqy_1s''/''freqy_2s'',''timen'',''tot'')'); end
            elseif strcmp(timeflag,'timey')
                if nargin~=13 || nargout~=6; error('Error:inputoutput','Check inputs/outputs are correct!\nWith the current input flags, there should be 13 inputs and 6 outputs and the function call should be something like:\n  [frequencies,gf_mon_p_f,gf_dip_p_f,time,gf_mon_p_t,gf_dip_p_t] = model_gfs_p_scat_acoust(srx,rxs,maxfreq,nfreq,c,''1d''/''2d''/''3d'',''volinjpos/neg''/''volinjratepos/neg'',''srcdip''/''recdip'',''scatn'',''wavln'',''freqy_1nf''/''freqy_1s''/''freqy_2s'',''timey'',''tot'')'); end
            end
        elseif strcmp(scatflag,'scaty')
            if strcmp(timeflag,'timen')
                if strcmp(sepflag,'tot')
                    if nargin~=14 || nargout~=3; error('Error:inputoutput','Check inputs/outputs are correct!\nWith the current input flags, there should be 14 inputs and 3 outputs and the function call should be something like:\n  [frequencies,gf_mon_p_f,gf_dip_p_f] = model_gfs_p_scat_acoust(srx,rxs,maxfreq,nfreq,c,''1d''/''2d''/''3d'',''volinjpos/neg''/''volinjratepos/neg'',''srcdip''/''recdip'',''scaty'',''wavln'',''freqy_1nf''/''freqy_1s''/''freqy_2s'',''timen'',''tot'',scats)'); end
                elseif strcmp(sepflag,'dir') || strcmp(sepflag,'scat')
                    if nargin~=14 || nargout~=5; error('Error:inputoutput','Check inputs/outputs are correct!\nWith the current input flags, there should be 14 inputs and 5 outputs and the function call should be something like:\n  [frequencies,gf_mon_p_f,gf_dip_p_f,gfdir_mon_p_f,gfdir_dip_p_f] = model_gfs_p_scat_acoust(srx,rxs,maxfreq,nfreq,c,''1d''/''2d''/''3d'',''volinjpos/neg''/''volinjratepos/neg'',''srcdip''/''recdip'',''scaty'',''wavln'',''freqy_1nf''/''freqy_1s''/''freqy_2s'',''timen'',''dir'',scats)\nor\n  [frequencies,gf_mon_p_f,gf_dip_p_f,gfscat_mon_p_f,gfscat_dip_p_f] = model_gfs_p_scat_acoust(srx,rxs,maxfreq,nfreq,c,''1d''/''2d''/''3d'',''volinjpos/neg''/''volinjratepos/neg'',''srcdip''/''recdip'',''scaty'',''wavln'',''freqy_1nf''/''freqy_1s''/''freqy_2s'',''timen'',''scat'',scats)'); end
                elseif strcmp(sepflag,'dirscat')
                    if nargin~=14 || nargout~=7; error('Error:inputoutput','Check inputs/outputs are correct!\nWith the current input flags, there should be 14 inputs and 7 outputs and the function call should be something like:\n  [frequencies,gf_mon_p_f,gf_dip_p_f,gfdir_mon_p_f,gfdir_dip_p_f,gfscat_mon_p_f,gfscat_dip_p_f] = model_gfs_p_scat_acoust(srx,rxs,maxfreq,nfreq,c,''1d''/''2d''/''3d'',''volinjpos/neg''/''volinjratepos/neg'',''srcdip''/''recdip'',''scaty'',''wavln'',''freqy_1nf''/''freqy_1s''/''freqy_2s'',''timen'',''dirscat'',scats)'); end
                end
            elseif strcmp(timeflag,'timey')
                if strcmp(sepflag,'tot')
                    if nargin~=14 || nargout~=6; error('Error:inputoutput','Check inputs/outputs are correct!\nWith the current input flags, there should be 14 inputs and 6 outputs and the function call should be something like:\n  [frequencies,gf_mon_p_f,gf_dip_p_f,time,gf_mon_p_t,gf_dip_p_t] = model_gfs_p_scat_acoust(srx,rxs,maxfreq,nfreq,c,''1d''/''2d''/''3d'',''volinjpos/neg''/''volinjratepos/neg'',''srcdip''/''recdip'',''scaty'',''wavln'',''freqy_1nf''/''freqy_1s''/''freqy_2s'',''timey'',''tot'',scats)'); end
                elseif strcmp(sepflag,'dir') || strcmp(sepflag,'scat')
                    if nargin~=14 || nargout~=10; error('Error:inputoutput','Check inputs/outputs are correct!\nWith the current input flags, there should be 14 inputs and 10 outputs and the function call should be something like:\n  [frequencies,gf_mon_p_f,gf_dip_p_f,gfdir_mon_p_f,gfdir_dip_p_f,time,gf_mon_p_t,gf_dip_p_t,gfdir_mon_p_t,gfdir_dip_p_t] = model_gfs_p_scat_acoust(srx,rxs,maxfreq,nfreq,c,''1d''/''2d''/''3d'',''volinjpos/neg''/''volinjratepos/neg'',''srcdip''/''recdip'',''scaty'',''wavln'',''freqy_1nf''/''freqy_1s''/''freqy_2s'',''timey'',''dir'',scats)\nor\n  [frequencies,gf_mon_p_f,gf_dip_p_f,gfscat_mon_p_f,gfscat_dip_p_f,time,gf_mon_p_t,gf_dip_p_t,gfscat_mon_p_t,gfscat_dip_p_t] = model_gfs_p_scat_acoust(srx,rxs,maxfreq,nfreq,c,''1d''/''2d''/''3d'',''volinjpos/neg''/''volinjratepos/neg'',''srcdip''/''recdip'',''scaty'',''wavln'',''freqy_1nf''/''freqy_1s''/''freqy_2s'',''timey'',''scat'',scats)'); end
                elseif strcmp(sepflag,'dirscat')
                    if nargin~=14 || nargout~=14; error('Error:inputoutput','Check inputs/outputs are correct!\nWith the current input flags, there should be 14 inputs and 14 outputs and the function call should be something like:\n  [frequencies,gf_mon_p_f,gf_dip_p_f,gfdir_mon_p_f,gfdir_dip_p_f,gfscat_mon_p_f,gfscat_dip_p_f,time,gf_mon_p_t,gf_dip_p_t,gfdir_mon_p_t,gfdir_dip_p_t,gfscat_mon_p_t,gfscat_dip_p_t] = model_gfs_p_scat_acoust(srx,rxs,maxfreq,nfreq,c,''1d''/''2d''/''3d'',''volinjpos/neg''/''volinjratepos/neg'',''srcdip''/''recdip'',''scaty'',''wavln'',''freqy_1nf''/''freqy_1s''/''freqy_2s'',''timey'',''dirscat'',scats)'); end
                end
            end
        end
    elseif strcmp(wavlflag,'wavly_pos') || strcmp(wavlflag,'wavly_centr') || strcmp(wavlflag,'wavly_neg') || strcmp(wavlflag,'wavly_shift')
        if strcmp(scatflag,'scatn')
            if strcmp(timeflag,'timen')
                if nargin~=14 || nargout~=4; error('Error:inputoutput','Check inputs/outputs are correct!\nWith the current input flags, there should be 14 inputs and 4 outputs and the function call should be something like:\n  [frequencies,gf_mon_p_f,gf_dip_p_f,wavelet_f] = model_gfs_p_scat_acoust(srx,rxs,maxfreq,nfreq,c,''1d''/''2d''/''3d'',''volinjpos/neg''/''volinjratepos/neg'',''srcdip''/''recdip'',''scatn'',''wavly_pos/centr/neg'',''freqy_1nf''/''freqy_1s''/''freqy_2s'',''timen'',''tot'',fc)'); end
            elseif strcmp(timeflag,'timey')
                if nargin~=14 || nargout~=8; error('Error:inputoutput','Check inputs/outputs are correct!\nWith the current input flags, there should be 14 inputs and 8 outputs and the function call should be something like:\n  [frequencies,gf_mon_p_f,gf_dip_p_f,time,gf_mon_p_t,gf_dip_p_t,wavelet_f,wavelet_t] = model_gfs_p_scat_acoust(srx,rxs,maxfreq,nfreq,c,''1d''/''2d''/''3d'',''volinjpos/neg''/''volinjratepos/neg'',''srcdip''/''recdip'',''scatn'',''wavly_pos/centr/neg'',''freqy_1nf''/''freqy_1s''/''freqy_2s'',''timey'',''tot'',fc)'); end
            end
        elseif strcmp(scatflag,'scaty')
            if strcmp(timeflag,'timen')
                if strcmp(sepflag,'tot')
                    if nargin~=15 || nargout~=4; error('Error:inputoutput','Check inputs/outputs are correct!\nWith the current input flags, there should be 15 inputs and 4 outputs and the function call should be something like:\n  [frequencies,gf_mon_p_f,gf_dip_p_f,wavelet_f] = model_gfs_p_scat_acoust(srx,rxs,maxfreq,nfreq,c,''1d''/''2d''/''3d'',''volinjpos/neg''/''volinjratepos/neg'',''srcdip''/''recdip'',''scaty'',''wavly_pos/centr/neg'',''freqy_1nf''/''freqy_1s''/''freqy_2s'',''timen'',''tot'',scats,fc)'); end
                elseif strcmp(sepflag,'dir') || strcmp(sepflag,'scat')
                    if nargin~=15 || nargout~=6; error('Error:inputoutput','Check inputs/outputs are correct!\nWith the current input flags, there should be 15 inputs and 6 outputs and the function call should be something like:\n  [frequencies,gf_mon_p_f,gf_dip_p_f,gfdir_mon_p_f,gfdir_dip_p_f,wavelet_f] = model_gfs_p_scat_acoust(srx,rxs,maxfreq,nfreq,c,''1d''/''2d''/''3d'',''volinjpos/neg''/''volinjratepos/neg'',''srcdip''/''recdip'',''scaty'',''wavly_pos/centr/neg'',''freqy_1nf''/''freqy_1s''/''freqy_2s'',''timen'',''dir'',scats,fc)\nor\n  [frequencies,gf_mon_p_f,gf_dip_p_f,gfscat_mon_p_f,gfscat_dip_p_f,wavelet_f] = model_gfs_p_scat_acoust(srx,rxs,maxfreq,nfreq,c,''1d''/''2d''/''3d'',''volinjpos/neg''/''volinjratepos/neg'',''srcdip''/''recdip'',''scaty'',''wavly_pos/centr/neg'',''freqy_1nf''/''freqy_1s''/''freqy_2s'',''timen'',''scat'',scats,fc)'); end
                elseif strcmp(sepflag,'dirscat')
                    if nargin~=15 || nargout~=8; error('Error:inputoutput','Check inputs/outputs are correct!\nWith the current input flags, there should be 15 inputs and 8 outputs and the function call should be something like:\n  [frequencies,gf_mon_p_f,gf_dip_p_f,gfdir_mon_p_f,gfdir_dip_p_f,gfscat_mon_p_f,gfscat_dip_p_f,wavelet_f] = model_gfs_p_scat_acoust(srx,rxs,maxfreq,nfreq,c,''1d''/''2d''/''3d'',''volinjpos/neg''/''volinjratepos/neg'',''srcdip''/''recdip'',''scaty'',''wavly_pos/centr/neg'',''freqy_1nf''/''freqy_1s''/''freqy_2s'',''timen'',''dirscat'',scats,fc)'); end
                end
            elseif strcmp(timeflag,'timey')
                if strcmp(sepflag,'tot')
                    if nargin~=15 || nargout~=8; error('Error:inputoutput','Check inputs/outputs are correct!\nWith the current input flags, there should be 15 inputs and 8 outputs and the function call should be something like:\n  [frequencies,gf_mon_p_f,gf_dip_p_f,time,gf_mon_p_t,gf_dip_p_t,wavelet_f,wavelet_t] = model_gfs_p_scat_acoust(srx,rxs,maxfreq,nfreq,c,''1d''/''2d''/''3d'',''volinjpos/neg''/''volinjratepos/neg'',''srcdip''/''recdip'',''scaty'',''wavly_pos/centr/neg'',''freqy_1nf''/''freqy_1s''/''freqy_2s'',''timey'',''tot'',scats,fc)'); end
                elseif strcmp(sepflag,'dir') || strcmp(sepflag,'scat')
                    if nargin~=15 || nargout~=12; error('Error:inputoutput','Check inputs/outputs are correct!\nWith the current input flags, there should be 15 inputs and 12 outputs and the function call should be something like:\n  [frequencies,gf_mon_p_f,gf_dip_p_f,gfdir_mon_p_f,gfdir_dip_p_f,time,gf_mon_p_t,gf_dip_p_t,gfdir_mon_p_t,gfdir_dip_p_t,wavelet_f,wavelet_t] = model_gfs_p_scat_acoust(srx,rxs,maxfreq,nfreq,c,''1d''/''2d''/''3d'',''volinjpos/neg''/''volinjratepos/neg'',''srcdip''/''recdip'',''scaty'',''wavly_pos/centr/neg'',''freqy_1nf''/''freqy_1s''/''freqy_2s'',''timey'',''dir'',scats,fc)\nor\n  [frequencies,gf_mon_p_f,gf_dip_p_f,gfscat_mon_p_f,gfscat_dip_p_f,time,gf_mon_p_t,gf_dip_p_t,gfscat_mon_p_t,gfscat_dip_p_t,wavelet_f,wavelet_t] = model_gfs_p_scat_acoust(srx,rxs,maxfreq,nfreq,c,''1d''/''2d''/''3d'',''volinjpos/neg''/''volinjratepos/neg'',''srcdip''/''recdip'',''scaty'',''wavly_pos/centr/neg'',''freqy_1nf''/''freqy_1s''/''freqy_2s'',''timey'',''scat'',scats,fc)'); end
                elseif strcmp(sepflag,'dirscat')
                    if nargin~=15 || nargout~=16; error('Error:inputoutput','Check inputs/outputs are correct!\nWith the current input flags, there should be 15 inputs and 16 outputs and the function call should be something like:\n  [frequencies,gf_mon_p_f,gf_dip_p_f,gfdir_mon_p_f,gfdir_dip_p_f,gfscat_mon_p_f,gfscat_dip_p_f,time,gf_mon_p_t,gf_dip_p_t,gfdir_mon_p_t,gfdir_dip_p_t,gfscat_mon_p_t,gfscat_dip_p_t,wavelet_f,wavelet_t] = model_gfs_p_scat_acoust(srx,rxs,maxfreq,nfreq,c,''1d''/''2d''/''3d'',''volinjpos/neg''/''volinjratepos/neg'',''srcdip''/''recdip'',''scaty'',''wavly_pos/centr/neg'',''freqy_1nf''/''freqy_1s''/''freqy_2s'',''timey'',''dirscat'',scats,fc)'); end
                end
            end
        end
    end
end



%% Retrieve information from input

nsrx=size(srx,1); % number of sources
nrxs=size(rxs,1); % number of receivers

nf = 2^nextpow2(nfreq); % round number of frequencies to the nearest power of two - makes fft/ifft run faster
%nf = nfreq; % keep the same number of frequencies as in the input

% Calculate df and frequency range
df = maxfreq/nf;
freq = (1:nf).*df;
freq=reshape(freq,nf,1);
om = 2*pi*freq; % angular frequency = 2*pi*frequency
kp = om./c; % wavenumber = angular frequency / velocity

% Calculate dt and time range
dt = 1/(2*maxfreq); % time step
time = dt.*(0:2*nf-1);
time = reshape(time,2*nf,1);

% Get number of dimensions
if strcmp(dim,'1d'); ndim = 1;
elseif strcmp(dim,'2d') || strcmp(dim,'2d_far'); ndim = 2;
elseif strcmp(dim,'3d'); ndim = 3;
end

% Get data from varargin
if (strcmp(scatflag,'scaty') || strcmp(wavlflag,'wavly_pos') || strcmp(wavlflag,'wavly_centr') || strcmp(wavlflag,'wavly_neg') || strcmp(wavlflag,'wavly_shift')) && isempty(varargin)
    error('Missing input for scatterers and/or wavelet.')
elseif ~isempty(varargin)    
    if strcmp(scatflag,'scaty')
        if strcmp(wavlflag,'wavly_pos') || strcmp(wavlflag,'wavly_centr') || strcmp(wavlflag,'wavly_neg') || strcmp(wavlflag,'wavly_shift')
            if ~isempty(varargin{1}) && size(varargin{1}(:,:),2)>=4
                scats(:,:) = varargin{1}(:,:);
            else
                error('Invalid input for scatterers'' coordinates.')
            end
            if ~isempty(varargin{2}) && (size(varargin{2}(:,:),2)==1 || size(varargin{2}(:,:),2)==2)
                fc = varargin{2}(1,1);
                if size(varargin{2}(:,:),2)==2 && strcmp(wavlflag,'wavly_shift')
                    ts = varargin{2}(1,2);
                elseif size(varargin{2}(:,:),2)==1 && strcmp(wavlflag,'wavly_shift')
                    error('Wavelet time-shift required if wavelet flag set to ''wavly_shift''.')
                end
            else
                error('Invalid input for wavelet.')
            end
        elseif strcmp(wavlflag,'wavln')
            if ~isempty(varargin{1}) && size(varargin{1}(:,:),2)>=4
                scats(:,:) = varargin{1}(:,:);
            else
                error('Invalid input for scatterers'' coordinates.')
            end
            if exist('varargin{2}','var')
                error('Extra input argument not needed.')
            end
        end
    elseif strcmp(scatflag,'scatn')
        if strcmp(wavlflag,'wavly_pos') || strcmp(wavlflag,'wavly_centr') || strcmp(wavlflag,'wavly_neg') || strcmp(wavlflag,'wavly_shift')
            if ~isempty(varargin{1}) && (size(varargin{1}(:,:),2)==1 || size(varargin{1}(:,:),2)==2)
                fc = varargin{1}(1,1);
                if size(varargin{1}(:,:),2)==2 && strcmp(wavlflag,'wavly_shift')
                    ts = varargin{1}(1,2);
                elseif size(varargin{1}(:,:),2)==1 && strcmp(wavlflag,'wavly_shift')
                    error('Wavelet time-shift required if wavelet flag set to ''wavly_shift''.')
                end
            else
                error('Invalid input for wavelet.')
            end
        elseif strcmp(wavlflag,'wavln')
            error('Extra input argument(s) not needed.')
        end
    end
end

% Get scattarers' coordinates and calculate scattering potential
if strcmp(scatflag,'scaty') && exist('scats','var')
    nscat = size(scats,1); % number of scatterers
    % Calculate scattering potential
    if size(scats,2)==4
        % Check that the values of the imaginary part of the scattering
        % amplitude fall within the range imposed by the optical theorem
        % - see Snieder (1999), equation (16)
        if strcmp(dim,'1d') && sum((scats(:,4)<(-2*kp(1))) | (scats(:,4)>0))>0
            error('Error:scattering_amplitude',['The value of the imaginary part of the scattering amplitude of ' num2str(sum((scats(:,4)<(-2*kp(1))) | (scats(:,4)>0))) ' scatterer(s) falls outside the acceptable range.\nIn 1D and a minimum modelled frequency of ' num2str(freq(1)) ' Hz, the value of Im(A) should be between ' num2str(-2*kp(1)) ' and 0.'])
        elseif (strcmp(dim,'2d') || strcmp(dim,'2d_far')) && sum((scats(:,4)<(-4)) | (scats(:,4)>0))>0
            error('Error:scattering_amplitude',['The value of the imaginary part of the scattering amplitude of ' num2str(sum((scats(:,4)<(-4)) | (scats(:,4)>0))) ' scatterer(s) falls outside the acceptable range.\nIn 2D the value of Im(A) should be between ' num2str(-4) ' and 0.'])
        elseif strcmp(dim,'3d') && sum((scats(:,4)<(-4*pi/kp(nf))) | (scats(:,4)>0))>0
            error('Error:scattering_amplitude',['The value of the imaginary part of the scattering amplitude of ' num2str(sum((scats(:,4)<(-4*pi/kp(nf))) | (scats(:,4)>0))) ' scatterer(s) falls outside the acceptable range.\nIn 3D and a maximum modelled frequency of ' num2str(freq(nf)) ' Hz, the value of Im(A) should be between ' num2str(-4*pi/kp(nf)) ' and 0.'])
        end
        tempi = repmat(reshape(scats(:,4),1,nscat),[nf 1 1]); % get imaginary part of scattering amplitude from input
    elseif size(scats,2)==(3+nf)
        % Check that the values of the imaginary part of the scattering
        % amplitude fall within the range imposed by the optical theorem
        % - see Snieder (1999), equation (16)
        if strcmp(dim,'1d') && sum(sum((scats(:,4:3+nf)<(-2.*repmat(kp.',[nscat 1 1]))) | (scats(:,4:3+nf)>0)))>0
            wrongscat = sum((scats(:,4:3+nf)<(-2.*repmat(kp.',[nscat 1 1]))) | (scats(:,4:3+nf)>0),2);
            error('Error:scattering_amplitude',['The value of the imaginary part of the scattering amplitude of ' num2str(sum((wrongscat>0))) ' scatterer(s) falls outside the acceptable range.\nIn 1D the value of Im(A) should be between -2k and 0.'])
        elseif (strcmp(dim,'2d') || strcmp(dim,'2d_far')) && sum(sum((scats(:,4:3+nf)<(-4)) | (scats(:,4:3+nf)>0)))>0
            wrongscat = sum((scats(:,4:3+nf)<(-4)) | (scats(:,4:3+nf)>0),2);
            error('Error:scattering_amplitude',['The value of the imaginary part of the scattering amplitude of ' num2str(sum((wrongscat>0))) ' scatterer(s) falls outside the acceptable range.\nIn 2D the value of Im(A) should be between -4 and 0.'])
        elseif strcmp(dim,'3d') && sum(sum((scats(:,4:3+nf)<(-4*pi./repmat(kp.',[nscat 1 1]))) | (scats(:,4:3+nf)>0)))>0
            wrongscat = sum((scats(:,4:3+nf)<(-4*pi./repmat(kp.',[nscat 1 1]))) | (scats(:,4:3+nf)>0),2);
            error('Error:scattering_amplitude',['The value of the imaginary part of the scattering amplitude of ' num2str(sum((wrongscat>0))) ' scatterer(s) falls outside the acceptable range.\nIn 3D the value of Im(A) should be between -4*pi/k and 0.'])
        end
        tempi = scats(:,4:3+nf).'; % get imaginary part of scattering amplitude from input
    else
       error('Wrong input format for scatterer(s) coordinates and/or imaginary part of scattering potential.') 
    end
    % Determine the real part of the scattering amplitude from the optical
    % theorem - see Snieder (1999), equation (16)
    if strcmp(dim,'1d')
        tempr = sqrt(-tempi.*(2.*repmat(kp,[1 nscat 1])+tempi));
    elseif strcmp(dim,'2d') || strcmp(dim,'2d_far')
        tempr = sqrt(-tempi.*(4+tempi)); % real part determined from the imaginary part in line with the optical theorem - see Groenenboom and Snieder (1995), equation (23)
    elseif strcmp(dim,'3d')
        tempr = sqrt(-tempi.*(4*pi./repmat(kp,[1 nscat 1])+tempi));
    end
    scat_pot(1:nf,1:nscat) = complex(tempr,tempi); % combine real and imaginary part to get scattering amplitude of each scatterer
end

% Get central frequency of Ricker wavelet from input and calculate source
% wavelet
if (strcmp(wavlflag,'wavly_pos') || strcmp(wavlflag,'wavly_centr') || strcmp(wavlflag,'wavly_neg') || strcmp(wavlflag,'wavly_shift')) && exist('fc','var')
    if strcmp(wavlflag,'wavly_pos')
        wavelet_t(1:2*nf,1) = rickerwavl_time(time-ceil((1/fc)/dt).*dt,fc);
    elseif strcmp(wavlflag,'wavly_centr')
        wavelet_t(1:2*nf,1) = rickerwavl_time(dt*(-nf:nf-1),fc);
        wavelet_t(1:2*nf,1) = fftshift(wavelet_t);
    elseif strcmp(wavlflag,'wavly_neg')
        wavelet_t(2*nf:-1:1,1) = rickerwavl_time(time-ceil((1/fc)/dt).*dt,fc);
    elseif strcmp(wavlflag,'wavly_shift')
        if abs(ts) > time(end)
            fprintf('WARNING: wavelet time-shift is greater than the maximum modelled time.\n')
        end
        if ts>0
            wavelet_t(1:2*nf,1) = rickerwavl_time(time-ceil(abs(ts)/dt).*dt,fc);
        elseif ts==0
            wavelet_t(1:2*nf,1) = rickerwavl_time(dt*(-nf:nf-1),fc);
            wavelet_t(1:2*nf,1) = fftshift(wavelet_t);
        elseif ts<0
            wavelet_t(2*nf:-1:1,1) = rickerwavl_time(time-ceil(abs(ts)/dt).*dt,fc);
        end        
    end
    wavelet_f = fft(wavelet_t,2*nf)./sqrt(2*nf); % fft wavelet to frequency domain
    wavelet_f1nf(1:nf,1) = [sqrt(2).*(wavelet_f(2:nf));(wavelet_f(nf+1))]; % take only frequencies from 1 to nf
    wavelet_f2s = fftshift(wavelet_f,1); % take all frequency and ffshift to have zero in the middle
    wavlflag = 'wavly';
end




%% Allocate space for results

% Allocate space for results in frequency domain
gf_mon_p_f1nf = zeros(nf,nrxs,nsrx);
gf_dip_p_f1nf = zeros(nf,nrxs,nsrx,ndim);
% Direct wavefields
gfdir_mon_p_f1nf = zeros(nf,nrxs,nsrx);
gfdir_dip_p_f1nf = zeros(nf,nrxs,nsrx,ndim);
% Scattered wavefields
gfscat_mon_p_f1nf = zeros(nf,nrxs,nsrx);
gfscat_dip_p_f1nf = zeros(nf,nrxs,nsrx,ndim);



%% Model background wavefield    
    
% For each source-receiver pair determine the incident background wavefield
% ump0: pressure from monopole source
% udp0: pressure from dipole source
[ump0,udp0] = acoustic_p_gfs_direct_f(freq,c,srx,rxs,dim,srctype,diploc);



%% Model scattered wavefield

if strcmp(scatflag,'scaty') && exist('scats','var')
    
    % Calculate the source-scatterer and scatterer receiver wavefields.
    
    % Evaluate derivatives at the sources --> calculate source-scatterer
    % monopole and dipole fields (with dipole evaluated at the sources),
    % and scatterer-receiver monopole field
    if strcmp(diploc,'srcdip') 
        
        % For each source-scatterer pair determine the incident
        % background wavefield - monopole and dipole, source type given in
        % the function call
        [umpssc,udpssc] = acoustic_p_gfs_direct_f(freq,c,srx,scats,dim,srctype,diploc);
        
        % Organise these wavefields into a matrix
        % -- see Appendix in Groenenboom and Snieder (1995) for details --
        
        % Matrices E0m and E0d have:
        %   scatterers along dim1
        %   sources along dim2
        %   frequencies along dim3
        E0m = zeros(nscat,nsrx,nf);
        E0d = zeros(nscat,nsrx,nf,ndim);
        for ff = 1:nf
            % If a source wavelet is applied, convolve with wavelet...
            if strcmp(wavlflag,'wavly') && exist('fc','var')
                E0m(:,:,ff) = umpssc(ff,:,:).*wavelet_f1nf(ff);    % Pressure from monopole source
                for dd = 1:ndim
                    E0d(:,:,ff,dd) = udpssc(ff,:,:,dd).*wavelet_f1nf(ff); % Pressure from dipole source, dd-derivative of monopole
                end
                % ...otherwise...
            elseif strcmp(wavlflag,'wavln')
                E0m(:,:,ff) = umpssc(ff,:,:);    % Pressure from monopole source
                for dd = 1:ndim
                    E0d(:,:,ff,dd) = udpssc(ff,:,:,dd); % Pressure from dipole source, dd-derivative of monopole
                end
            end
        end
        
        % For each scatterer-receiver pair determine the incident background wavefield
        % (each scatterer acts as a monopole source of volume injection)
        [umpscr] = acoustic_p_gfs_direct_f(freq,c,scats,rxs,dim,'volinjpos');
    
        
    % Evaluate derivatives at the receivers --> calculate source-scatterer
    % monopole field, and scatterer-receiver monopole and dipole fields
    % (with dipole evaluated at the receivers)
    elseif strcmp(diploc,'recdip')
        
        % For each source-scatterer pair determine the incident
        % background wavefield - monopole only, volume injection sources
        [umpssc] = acoustic_p_gfs_direct_f(freq,c,srx,scats,dim,'volinjpos');
        
        % Organise this wavefield into a matrix
        % -- see Appendix in Groenenboom and Snieder (1995) for details --
        
        % Matrix E0m has:
        %   scatterers along dim1
        %   sources along dim2
        %   frequencies along dim3
        E0m = zeros(nscat,nsrx,nf);
        for ff = 1:nf
            % If a source wavelet is applied, convolve with wavelet...
            if strcmp(wavlflag,'wavly') && exist('fc','var')
                E0m(:,:,ff) = umpssc(ff,:,:).*wavelet_f1nf(ff);    % Pressure from monopole source
                % ...otherwise...
            elseif strcmp(wavlflag,'wavln')
                E0m(:,:,ff) = umpssc(ff,:,:);    % Pressure from monopole source
            end
        end
        
        % For each scatterer-receiver pair determine the incident
        % background wavefield - monopole and dipole, source type given in
        % the function call
        [umpscr,udpscr] = acoustic_p_gfs_direct_f(freq,c,scats,rxs,dim,srctype,diploc);
        
    end
    
    % For each pair of scatterers determine the inter-scatterer Green's
    % function - monopole only.
    % Matrix M contains the (monopole) scatterer-scatterer responses, with
    % scatterers along dim1 and dim2, and frequencies along dim3.
    
    % Pre-allocate matrix M
    M = zeros(nscat,nscat,nf);
    M(end,end,:) = -1; % Diagonal elements are equal to minus one
    
    % Fill matrix M with scatterer-scatterer responses
    if nscat > 1 % this if statement is only true when there is multiple scattering
        % Loop over all scatterers
        for kk = 1:(nscat-1)
            
            % Compute the response between each scatterer paired with a second.
            % Due to reciprocity we do not need to compute both pairs [r_i,r_j] and  [r_j,r_i].
            % (each scatterer acts as a monopole source of volume injection)
            [umpscsc] = acoustic_p_gfs_direct_f(freq,c,scats(kk,:),scats(kk+1:end,:),dim,'volinjpos');
            
            % Weight the inter-scatterer Green's functions by the scattering potential
            ttt(:,:) = repmat(scat_pot(:,kk),[1 size(umpscsc,2) 1]).*umpscsc(:,:,1); % volume injection source
            % Put responses into matrix M
            M(kk+1:end,kk,:) = ttt.'; % Allocate appropriate Green's functions to matrix M
            M(kk,kk+1:end,:) = ttt.'; % Allocate the reciprocal Green's functions to matrix M
            M(kk,kk,:) = -1; % Diagonal elements are equal to -1
            clear ttt
            
        end
    else
        M(1,1,1:end) = -1; % Diagonal elements are equal to -1
    end
    
    % Some more pre-allocation of matrices
    Em = zeros(nf,nscat,nsrx);
    if strcmp(diploc,'srcdip')
        Ed = zeros(nf,nscat,nsrx,ndim);
    end
    u_sc_mp = zeros(nf,nscat);
    u_sc_dp = zeros(nf,nscat,ndim);

end



%% Calculate pressure responses

% Calculate responses from each source separately
for ss = 1:nsrx; % For each source...
    
    % If there are scatterers calculate the total field from source ss to
    % all scatterers
    if strcmp(scatflag,'scaty') && exist('scats','var')
        for ff = 1:nf % for each frequency
            Em(ff,:,ss) = -M(:,:,ff)\E0m(:,ss,ff); % Use mldivide to solve the equation -M*Em = E0m
            %Em(ff,:,ss) = -pinv(M(:,:,ff))*E0m(:,ss,ff); % Use pinv to solve the equation -M*Em = E0m (in case the solution using mldivide is non-unique)
            if strcmp(diploc,'srcdip')
                for dd = 1:ndim
                    Ed(ff,:,ss,dd) = -M(:,:,ff)\E0d(:,ss,ff,dd); % Use mldivide to solve the equation -M*Ed = E0d
                    %Ed(ff,:,ss,dd) = -pinv(M(:,:,ff))*E0d(:,ss,ff,dd); % Use pinv to solve the equation -M*Ed = E0d (in case the solution using mldivide is non-unique)
                end
            end
        end
    end
    
    
    for rr = 1:nrxs % ...for each receiver...
        
        % If there are scatterers
        if strcmp(scatflag,'scaty') && exist('scats','var')
            for ff = 1:nf % ...for each frequency...
                for kk = 1:nscat % ...for each scatterer...
                    % ...determine the scattered wavefield (from source ss to receiver rr via scatterer kk) in the frequency domain
                    u_sc_mp(ff,kk) = Em(ff,kk,ss).*umpscr(ff,rr,kk).*scat_pot(ff,kk);
                    for dd = 1:ndim
                        if strcmp(diploc,'srcdip')
                            u_sc_dp(ff,kk,dd) = Ed(ff,kk,ss,dd).*umpscr(ff,rr,kk).*scat_pot(ff,kk);
                        elseif strcmp(diploc,'recdip')
                            u_sc_dp(ff,kk,dd) = Em(ff,kk,ss).*udpscr(ff,rr,kk,dd).*scat_pot(ff,kk);
                        end
                    end
                end
            end
            
            
            % Scattered GF wavefields in the frequency domain, frequency range 1:nf
            % The total scattered wavefield is obtained by summing over all
            % scatterers (sum over dim2 of u_sc_mp and u_sc_dp)
            gfscat_mon_p_f1nf(:,rr,ss) = sum(u_sc_mp(1:nf,:),2); % Pressure from monopole source
            for dd = 1:ndim
                gfscat_dip_p_f1nf(:,rr,ss,dd) = sum(u_sc_dp(1:nf,:,dd),2); % Pressure from dipole source, dd-derivative of monopole
            end
        end
        
        
        % Direct GF wavefields in the frequency domain, frequency range 1:nf
        % If a source wavelet is applied, convolve with wavelet...
        if strcmp(wavlflag,'wavly') && exist('fc','var')
            gfdir_mon_p_f1nf(:,rr,ss) = ump0(1:nf,rr,ss).*wavelet_f1nf; % Pressure from monopole source
            for dd = 1:ndim
                gfdir_dip_p_f1nf(:,rr,ss,dd) = udp0(1:nf,rr,ss,dd).*wavelet_f1nf; % Pressure from dipole source, dd-derivative of monopole
            end
            % ...otherwise...
        elseif strcmp(wavlflag,'wavln')
            gfdir_mon_p_f1nf(:,rr,ss) = ump0(1:nf,rr,ss); % Pressure from monopole source
            for dd = 1:ndim
                gfdir_dip_p_f1nf(:,rr,ss,dd) = udp0(1:nf,rr,ss,dd); % Pressure from dipole source, dd-derivative of monopole
            end
        end
        
        
        % Total GF wavefields in the frequency domain, frequency range 1:nf
        % If there are scatterers, sum direct and scattered wavefield
        if strcmp(scatflag,'scaty') && exist('scats','var')
            gf_mon_p_f1nf(:,rr,ss) = gfdir_mon_p_f1nf(:,rr,ss) + gfscat_mon_p_f1nf(:,rr,ss); % Pressure from monopole source
            for dd = 1:ndim
                gf_dip_p_f1nf(:,rr,ss,dd) = gfdir_dip_p_f1nf(:,rr,ss,dd) + gfscat_dip_p_f1nf(:,rr,ss,dd); % Pressure from dipole source, dd-derivative of monopole
            end
            % For no scatterers, the total wavefield is the direct wavefield
        elseif strcmp(scatflag,'scatn')
            gf_mon_p_f1nf(:,rr,ss) = gfdir_mon_p_f1nf(:,rr,ss); % Pressure from monopole source
            for dd = 1:ndim
                gf_dip_p_f1nf(:,rr,ss,dd) = gfdir_dip_p_f1nf(:,rr,ss,dd); % Pressure from dipole source, dd-derivative of monopole
            end
        end
        
    end % end of receiver loop
end % end of source loop



%% Define outputs (varagout)

% Error message if there are no scatterers and the separate output flag is
% not set to 'tot'
if strcmp(scatflag,'scatn') && ~strcmp(sepflag,'tot')
    error('If there are no scatterers the direct and total wavefield cannot be given separately - change input!')
    
    
% Varargout = TOTAL wavefield
elseif (strcmp(scatflag,'scatn') || strcmp(scatflag,'scaty')) && strcmp(sepflag,'tot')
    
    if ~strcmp(freqflag,'freqn')
        
        % Waveforms in FREQUENCY domain
        if strcmp(freqflag,'freqy_1nf')
            varargout{1} = freq;                                            % FREQUENCY vector
            varargout{2} = gf_mon_p_f1nf;                                   % TOTAL - Pressure from monopole source
            varargout{3} = gf_dip_p_f1nf;                                   % TOTAL - Pressure from dipole source (x-,y-,z-derivative of monopole at level 1,2,3 of dim4)
        elseif strcmp(freqflag,'freqy_1s')
            varargout{1} = [0;freq];                                                                                % FREQUENCY vector 
            varargout{2} = [zeros(1,nrxs,nsrx);gf_mon_p_f1nf(1:nf-1,:,:);real(gf_mon_p_f1nf(nf,:,:))];              % TOTAL - Pressure from monopole source
            varargout{3} = [zeros(1,nrxs,nsrx,ndim);gf_dip_p_f1nf(1:nf-1,:,:,:);real(gf_dip_p_f1nf(nf,:,:,:))];     % TOTAL - Pressure from dipole source (x-,y-,z-derivative of monopole at level 1,2,3 of dim4)
        elseif strcmp(freqflag,'freqy_2s')
            gf_dip_p_f = zeros(2*nf,nrxs,nsrx,ndim);
            for dd = 1:ndim
                gf_dip_p_dd = centerfreqs(gf_dip_p_f1nf(:,:,:,dd),nf);
                gf_dip_p_f(:,:,:,dd) = gf_dip_p_dd;                         
                clear gf_dip_p_dd
            end
            varargout{1} = [flipdim(-freq,1);0;freq(1:end-1)];              % FREQUENCY vector 
            varargout{2} = centerfreqs(gf_mon_p_f1nf,nf);                   % TOTAL - Pressure from monopole source
            varargout{3} = gf_dip_p_f;                                      % TOTAL - Pressure from dipole source (x-,y-,z-derivative of monopole at level 1,2,3 of dim4)
        end
        
        % Waveforms in TIME domain
        if strcmp(timeflag,'timey')
            gf_mon_p_ff = centerfreqs(gf_mon_p_f1nf,nf);
            gf_dip_p_ff = zeros(2*nf,nrxs,nsrx,ndim);
            for dd = 1:ndim
                gf_dip_p_dd = centerfreqs(gf_dip_p_f1nf(:,:,:,dd),nf);
                gf_dip_p_ff(:,:,:,dd) = gf_dip_p_dd;
                clear gf_dip_p_dd
            end
            varargout{4} = time;                                            % TIME vector
            varargout{5} = real(sqrt(2*nf).*ifft(fftshift(gf_mon_p_ff,1),[],1));  % TOTAL - Pressure from monopole source
            varargout{6} = real(sqrt(2*nf).*ifft(fftshift(gf_dip_p_ff,1),[],1));  % TOTAL - Pressure from dipole source (x-,y-,z-derivative of monopole at level 1,2,3 of dim4)
        end
        
        % Output wavelet if applied
        if strcmp(wavlflag,'wavly')
            if strcmp(timeflag,'timen')
                if strcmp(freqflag,'freqy_1nf')
                    varargout{4} = wavelet_f1nf;                            % WAVELET, frequency (1:nf)
                elseif strcmp(freqflag,'freqy_1s')
                    varargout{4} = [0;wavelet_f1nf];                        % WAVELET, frequency (0:nf)
                elseif strcmp(freqflag,'freqy_2s')
                    varargout{4} = wavelet_f2s;                             % WAVELET, frequency (-nf:-1,0,1:nf-1)
                end
            elseif strcmp(timeflag,'timey')
                if strcmp(freqflag,'freqy_1nf')
                    varargout{7} = wavelet_f1nf;                            % WAVELET, frequency (1:nf)
                    varargout{8} = wavelet_t;                               % WAVELET, time
                elseif strcmp(freqflag,'freqy_1s')
                    varargout{7} = [0;wavelet_f1nf];                        % WAVELET, frequency (0:nf)
                    varargout{8} = wavelet_t;                               % WAVELET, time
                elseif strcmp(freqflag,'freqy_2s')
                    varargout{7} = wavelet_f2s;                             % WAVELET, frequency (-nf:-1,0,1:nf-1)
                    varargout{8} = wavelet_t;                               % WAVELET, time
                end
            end
        end
        
    elseif strcmp(freqflag,'freqn')
        
        % Waveforms in TIME domain
        if strcmp(timeflag,'timey')
            gf_mon_p_ff = centerfreqs(gf_mon_p_f1nf,nf);
            gf_dip_p_ff = zeros(2*nf,nrxs,nsrx,ndim);
            for dd = 1:ndim
                gf_dip_p_dd = centerfreqs(gf_dip_p_f1nf(:,:,:,dd),nf);
                gf_dip_p_ff(:,:,:,dd) = gf_dip_p_dd;
                clear gf_dip_p_dd
            end
            varargout{1} = time;                                                    % TIME vector
            varargout{2} = real(sqrt(2*nf).*ifft(fftshift(gf_mon_p_ff,1),[],1));	% TOTAL - Pressure from monopole source
            varargout{3} = real(sqrt(2*nf).*ifft(fftshift(gf_dip_p_ff,1),[],1));	% TOTAL - Pressure from dipole source (x-,y-,z-derivative of monopole at level 1,2,3 of dim4)
            
            % Output wavelet if applied
            if strcmp(wavlflag,'wavly')
                varargout{4} = wavelet_t;                                   % WAVELET, time
            end
        end
    end
       
    
% Varargout = TOTAL + DIRECT wavefield
elseif (strcmp(scatflag,'scaty') && strcmp(sepflag,'dir'))
    
    if ~strcmp(freqflag,'freqn')
        
        % Waveforms in FREQUENCY domain
        if strcmp(freqflag,'freqy_1nf')
            varargout{1} = freq;                                            % FREQUENCY vector
            varargout{2} = gf_mon_p_f1nf;                                   % TOTAL - Pressure from monopole source
            varargout{3} = gf_dip_p_f1nf;                                   % TOTAL - Pressure from dipole source (x-,y-,z-derivative of monopole at level 1,2,3 of dim4)
            varargout{4} = gfdir_mon_p_f1nf;                                % DIRECT - Pressure from monopole source
            varargout{5} = gfdir_dip_p_f1nf;                                % DIRECT - Pressure from dipole source (x-,y-,z-derivative of monopole at level 1,2,3 of dim4)
        elseif strcmp(freqflag,'freqy_1s')
            varargout{1} = [0;freq];                                                                                    % FREQUENCY vector 
            varargout{2} = [zeros(1,nrxs,nsrx);gf_mon_p_f1nf(1:nf-1,:,:);real(gf_mon_p_f1nf(nf,:,:))];                  % TOTAL - Pressure from monopole source
            varargout{3} = [zeros(1,nrxs,nsrx,ndim);gf_dip_p_f1nf(1:nf-1,:,:,:);real(gf_dip_p_f1nf(nf,:,:,:))];         % TOTAL - Pressure from dipole source (x-,y-,z-derivative of monopole at level 1,2,3 of dim4)
            varargout{4} = [zeros(1,nrxs,nsrx);gfdir_mon_p_f1nf(1:nf-1,:,:);real(gfdir_mon_p_f1nf(nf,:,:))];            % DIRECT - Pressure from monopole source
            varargout{5} = [zeros(1,nrxs,nsrx,ndim);gfdir_dip_p_f1nf(1:nf-1,:,:,:);real(gfdir_dip_p_f1nf(nf,:,:,:))];   % DIRECT - Pressure from dipole source (x-,y-,z-derivative of monopole at level 1,2,3 of dim4)
        elseif strcmp(freqflag,'freqy_2s')
            gf_dip_p_f = zeros(2*nf,nrxs,nsrx,ndim);
            gfdir_dip_p_f2s = zeros(2*nf,nrxs,nsrx,ndim);
            for dd = 1:ndim
                gf_dip_p_dd = centerfreqs(gf_dip_p_f1nf(:,:,:,dd),nf);
                gf_dip_p_f(:,:,:,dd) = gf_dip_p_dd;                         
                clear gf_dip_p_dd
                gfdir_dip_p_dd = centerfreqs(gfdir_dip_p_f1nf(:,:,:,dd),nf);
                gfdir_dip_p_f2s(:,:,:,dd) = gfdir_dip_p_dd;
                clear gfdir_dip_p_dd
            end
            varargout{1} = [flipdim(-freq,1);0;freq(1:end-1)];              % FREQUENCY vector 
            varargout{2} = centerfreqs(gf_mon_p_f1nf,nf);                   % TOTAL - Pressure from monopole source
            varargout{3} = gf_dip_p_f;                                      % TOTAL - Pressure from dipole source (x-,y-,z-derivative of monopole at level 1,2,3 of dim4)
            varargout{4} = centerfreqs(gfdir_mon_p_f1nf,nf);                % DIRECT - Pressure from monopole source
            varargout{5} = gfdir_dip_p_f2s;                                 % DIRECT - Pressure from dipole source (x-,y-,z-derivative of monopole at level 1,2,3 of dim4)
        end
        
        % Waveforms in TIME domain
        if strcmp(timeflag,'timey')
            gf_mon_p_ff = centerfreqs(gf_mon_p_f1nf,nf);
            gfdir_mon_p_ff = centerfreqs(gfdir_mon_p_f1nf,nf);
            gf_dip_p_ff = zeros(2*nf,nrxs,nsrx,ndim);
            gfdir_dip_p_ff = zeros(2*nf,nrxs,nsrx,ndim);
            for dd = 1:ndim
                gf_dip_p_dd = centerfreqs(gf_dip_p_f1nf(:,:,:,dd),nf);
                gf_dip_p_ff(:,:,:,dd) = gf_dip_p_dd;
                clear gf_dip_p_dd
                gfdir_dip_p_dd = centerfreqs(gfdir_dip_p_f1nf(:,:,:,dd),nf);
                gfdir_dip_p_ff(:,:,:,dd) = gfdir_dip_p_dd;
                clear gfdir_dip_p_dd
            end
            varargout{6} = time;                                                        % TIME vector
            varargout{7} = real(sqrt(2*nf).*ifft(fftshift(gf_mon_p_ff,1),[],1));        % TOTAL - Pressure from monopole source
            varargout{8} = real(sqrt(2*nf).*ifft(fftshift(gf_dip_p_ff,1),[],1));        % TOTAL - Pressure from dipole source (x-,y-,z-derivative of monopole at level 1,2,3 of dim4)
            varargout{9} = real(sqrt(2*nf).*ifft(fftshift(gfdir_mon_p_ff,1),[],1));     % DIRECT - Pressure from monopole source
            varargout{10} = real(sqrt(2*nf).*ifft(fftshift(gfdir_dip_p_ff,1),[],1));	% DIRECT - Pressure from dipole source (x-,y-,z-derivative of monopole at level 1,2,3 of dim4) 
        end
        
        % Output wavelet if applied
        if strcmp(wavlflag,'wavly')
            if strcmp(timeflag,'timen')
                if strcmp(freqflag,'freqy_1nf')
                    varargout{6} = wavelet_f1nf;                            % WAVELET, frequency (1:nf)
                elseif strcmp(freqflag,'freqy_1s')
                    varargout{6} = [0;wavelet_f1nf];                        % WAVELET, frequency (0:nf)
                elseif strcmp(freqflag,'freqy_2s')
                    varargout{6} = wavelet_f2s;                             % WAVELET, frequency (-nf:-1,0,1:nf-1)
                end
            elseif strcmp(timeflag,'timey')
                if strcmp(freqflag,'freqy_1nf')
                    varargout{11} = wavelet_f1nf;                           % WAVELET, frequency (1:nf)
                    varargout{12} = wavelet_t;                              % WAVELET, time
                elseif strcmp(freqflag,'freqy_1s')
                    varargout{11} = [0;wavelet_f1nf];                       % WAVELET, frequency (0:nf)
                    varargout{12} = wavelet_t;                              % WAVELET, time
                elseif strcmp(freqflag,'freqy_2s')
                    varargout{11} = wavelet_f2s;                            % WAVELET, frequency (-nf:-1,0,1:nf-1)
                    varargout{12} = wavelet_t;                              % WAVELET, time
                end
            end
        end
        
    elseif strcmp(freqflag,'freqn')
        
        % Waveforms in TIME domain
        if strcmp(timeflag,'timey')
            gf_mon_p_ff = centerfreqs(gf_mon_p_f1nf,nf);
            gfdir_mon_p_ff = centerfreqs(gfdir_mon_p_f1nf,nf);
            gf_dip_p_ff = zeros(2*nf,nrxs,nsrx,ndim);
            gfdir_dip_p_ff = zeros(2*nf,nrxs,nsrx,ndim);
            for dd = 1:ndim
                gf_dip_p_dd = centerfreqs(gf_dip_p_f1nf(:,:,:,dd),nf);
                gf_dip_p_ff(:,:,:,dd) = gf_dip_p_dd;
                clear gf_dip_p_dd
                gfdir_dip_p_dd = centerfreqs(gfdir_dip_p_f1nf(:,:,:,dd),nf);
                gfdir_dip_p_ff(:,:,:,dd) = gfdir_dip_p_dd;
                clear gfdir_dip_p_dd
            end
            varargout{1} = time;                                                    % TIME vector
            varargout{2} = real(sqrt(2*nf).*ifft(fftshift(gf_mon_p_ff,1),[],1));    % TOTAL - Pressure from monopole source
            varargout{3} = real(sqrt(2*nf).*ifft(fftshift(gf_dip_p_ff,1),[],1));    % TOTAL - Pressure from dipole source (x-,y-,z-derivative of monopole at level 1,2,3 of dim4)
            varargout{4} = real(sqrt(2*nf).*ifft(fftshift(gfdir_mon_p_ff,1),[],1)); % DIRECT - Pressure from monopole source
            varargout{5} = real(sqrt(2*nf).*ifft(fftshift(gfdir_dip_p_ff,1),[],1));	% DIRECT - Pressure from dipole source (x-,y-,z-derivative of monopole at level 1,2,3 of dim4)
            
            % Output wavelet if applied
            if strcmp(wavlflag,'wavly')
                varargout{6} = wavelet_t;                                   % WAVELET, time
            end
        end
    end
    
       
% Varargout = TOTAL + SCATTERED wavefield
elseif (strcmp(scatflag,'scaty') && strcmp(sepflag,'scat'))
    
    if ~strcmp(freqflag,'freqn')
        
        % Waveforms in FREQUENCY domain
        if strcmp(freqflag,'freqy_1nf')
            varargout{1} = freq;                                            % FREQUENCY vector
            varargout{2} = gf_mon_p_f1nf;                                   % TOTAL - Pressure from monopole source
            varargout{3} = gf_dip_p_f1nf;                                   % TOTAL - Pressure from dipole source (x-,y-,z-derivative of monopole at level 1,2,3 of dim4)
            varargout{4} = gfscat_mon_p_f1nf;                             	% SCATTERED - Pressure from monopole source
            varargout{5} = gfscat_dip_p_f1nf;                             	% SCATTERED - Pressure from dipole source (x-,y-,z-derivative of monopole at level 1,2,3 of dim4)
        elseif strcmp(freqflag,'freqy_1s')
            varargout{1} = [0;freq];                                                                                    % FREQUENCY vector 
            varargout{2} = [zeros(1,nrxs,nsrx);gf_mon_p_f1nf(1:nf-1,:,:);real(gf_mon_p_f1nf(nf,:,:))];                  % TOTAL - Pressure from monopole source
            varargout{3} = [zeros(1,nrxs,nsrx,ndim);gf_dip_p_f1nf(1:nf-1,:,:,:);real(gf_dip_p_f1nf(nf,:,:,:))];         % TOTAL - Pressure from dipole source (x-,y-,z-derivative of monopole at level 1,2,3 of dim4)
            varargout{4} = [zeros(1,nrxs,nsrx);gfscat_mon_p_f1nf(1:nf-1,:,:);real(gfscat_mon_p_f1nf(nf,:,:))];        	% SCATTERED - Pressure from monopole source
            varargout{5} = [zeros(1,nrxs,nsrx,ndim);gfscat_dip_p_f1nf(1:nf-1,:,:,:);real(gfscat_dip_p_f1nf(nf,:,:,:))];	% SCATTERED - Pressure from dipole source (x-,y-,z-derivative of monopole at level 1,2,3 of dim4)
        elseif strcmp(freqflag,'freqy_2s')
            gf_dip_p_f = zeros(2*nf,nrxs,nsrx,ndim);
            gfscat_dip_p_f2s = zeros(2*nf,nrxs,nsrx,ndim);
            for dd = 1:ndim
                gf_dip_p_dd = centerfreqs(gf_dip_p_f1nf(:,:,:,dd),nf);
                gf_dip_p_f(:,:,:,dd) = gf_dip_p_dd;                         
                clear gf_dip_p_dd
                gfscat_dip_p_dd = centerfreqs(gfscat_dip_p_f1nf(:,:,:,dd),nf);
                gfscat_dip_p_f2s(:,:,:,dd) = gfscat_dip_p_dd;
                clear gfscat_dip_p_dd
            end
            varargout{1} = [flipdim(-freq,1);0;freq(1:end-1)];              % FREQUENCY vector 
            varargout{2} = centerfreqs(gf_mon_p_f1nf,nf);                   % TOTAL - Pressure from monopole source
            varargout{3} = gf_dip_p_f;                                      % TOTAL - Pressure from dipole source (x-,y-,z-derivative of monopole at level 1,2,3 of dim4)
            varargout{4} = centerfreqs(gfscat_mon_p_f1nf,nf);             	% SCATTERED - Pressure from monopole source
            varargout{5} = gfscat_dip_p_f2s;                              	% SCATTERED - Pressure from dipole source (x-,y-,z-derivative of monopole at level 1,2,3 of dim4)
        end
        
        % Waveforms in TIME domain
        if strcmp(timeflag,'timey')
            gf_mon_p_ff = centerfreqs(gf_mon_p_f1nf,nf);
            gfscat_mon_p_ff = centerfreqs(gfscat_mon_p_f1nf,nf);
            gf_dip_p_ff = zeros(2*nf,nrxs,nsrx,ndim);
            gfscat_dip_p_ff = zeros(2*nf,nrxs,nsrx,ndim);
            for dd = 1:ndim
                gf_dip_p_dd = centerfreqs(gf_dip_p_f1nf(:,:,:,dd),nf);
                gf_dip_p_ff(:,:,:,dd) = gf_dip_p_dd;
                clear gf_dip_p_dd
                gfscat_dip_p_dd = centerfreqs(gfscat_dip_p_f1nf(:,:,:,dd),nf);
                gfscat_dip_p_ff(:,:,:,dd) = gfscat_dip_p_dd;
                clear gfscat_dip_p_dd
            end
            varargout{6} = time;                                                        % TIME vector
            varargout{7} = real(sqrt(2*nf).*ifft(fftshift(gf_mon_p_ff,1),[],1));        % TOTAL - Pressure from monopole source
            varargout{8} = real(sqrt(2*nf).*ifft(fftshift(gf_dip_p_ff,1),[],1));        % TOTAL - Pressure from dipole source (x-,y-,z-derivative of monopole at level 1,2,3 of dim4)
            varargout{9} = real(sqrt(2*nf).*ifft(fftshift(gfscat_mon_p_ff,1),[],1));	% SCATTERED - Pressure from monopole source
            varargout{10} = real(sqrt(2*nf).*ifft(fftshift(gfscat_dip_p_ff,1),[],1));	% SCATTERED - Pressure from dipole source (x-,y-,z-derivative of monopole at level 1,2,3 of dim4) 
        end
        
        % Output wavelet if applied
        if strcmp(wavlflag,'wavly')
            if strcmp(timeflag,'timen')
                if strcmp(freqflag,'freqy_1nf')
                    varargout{6} = wavelet_f1nf;                            % WAVELET, frequency (1:nf)
                elseif strcmp(freqflag,'freqy_1s')
                    varargout{6} = [0;wavelet_f1nf];                        % WAVELET, frequency (0:nf)
                elseif strcmp(freqflag,'freqy_2s')
                    varargout{6} = wavelet_f2s;                             % WAVELET, frequency (-nf:-1,0,1:nf-1)
                end
            elseif strcmp(timeflag,'timey')
                if strcmp(freqflag,'freqy_1nf')
                    varargout{11} = wavelet_f1nf;                           % WAVELET, frequency (1:nf)
                    varargout{12} = wavelet_t;                              % WAVELET, time
                elseif strcmp(freqflag,'freqy_1s')
                    varargout{11} = [0;wavelet_f1nf];                       % WAVELET, frequency (0:nf)
                    varargout{12} = wavelet_t;                              % WAVELET, time
                elseif strcmp(freqflag,'freqy_2s')
                    varargout{11} = wavelet_f2s;                            % WAVELET, frequency (-nf:-1,0,1:nf-1)
                    varargout{12} = wavelet_t;                              % WAVELET, time
                end
            end
        end
        
    elseif strcmp(freqflag,'freqn')
        
        % Waveforms in TIME domain
        if strcmp(timeflag,'timey')
            gf_mon_p_ff = centerfreqs(gf_mon_p_f1nf,nf);
            gfscat_mon_p_ff = centerfreqs(gfscat_mon_p_f1nf,nf);
            gf_dip_p_ff = zeros(2*nf,nrxs,nsrx,ndim);
            gfscat_dip_p_ff = zeros(2*nf,nrxs,nsrx,ndim);
            for dd = 1:ndim
                gf_dip_p_dd = centerfreqs(gf_dip_p_f1nf(:,:,:,dd),nf);
                gf_dip_p_ff(:,:,:,dd) = gf_dip_p_dd;
                clear gf_dip_p_dd
                gfscat_dip_p_dd = centerfreqs(gfscat_dip_p_f1nf(:,:,:,dd),nf);
                gfscat_dip_p_ff(:,:,:,dd) = gfscat_dip_p_dd;
                clear gfscat_dip_p_dd
            end
            varargout{1} = time;                                                        % TIME vector
            varargout{2} = real(sqrt(2*nf).*ifft(fftshift(gf_mon_p_ff,1),[],1));        % TOTAL - Pressure from monopole source
            varargout{3} = real(sqrt(2*nf).*ifft(fftshift(gf_dip_p_ff,1),[],1));        % TOTAL - Pressure from dipole source (x-,y-,z-derivative of monopole at level 1,2,3 of dim4)
            varargout{4} = real(sqrt(2*nf).*ifft(fftshift(gfscat_mon_p_ff,1),[],1));    % SCATTERED - Pressure from monopole source
            varargout{5} = real(sqrt(2*nf).*ifft(fftshift(gfscat_dip_p_ff,1),[],1));    % SCATTERED - Pressure from dipole source (x-,y-,z-derivative of monopole at level 1,2,3 of dim4)
            
            % Output wavelet if applied
            if strcmp(wavlflag,'wavly')
                varargout{6} = wavelet_t;                                   % WAVELET, time
            end
        end
    end    
    
    
% Varargout = TOTAL + DIRECT + SCATTERED wavefield
elseif (strcmp(scatflag,'scaty') && strcmp(sepflag,'dirscat'))
    
    if ~strcmp(freqflag,'freqn')
        
        % Waveforms in FREQUENCY domain
        if strcmp(freqflag,'freqy_1nf')
            varargout{1} = freq;                                            % FREQUENCY vector
            varargout{2} = gf_mon_p_f1nf;                                   % TOTAL - Pressure from monopole source
            varargout{3} = gf_dip_p_f1nf;                                   % TOTAL - Pressure from dipole source (x-,y-,z-derivative of monopole at level 1,2,3 of dim4)
            varargout{4} = gfdir_mon_p_f1nf;                                % DIRECT - Pressure from monopole source
            varargout{5} = gfdir_dip_p_f1nf;                                % DIRECT - Pressure from dipole source (x-,y-,z-derivative of monopole at level 1,2,3 of dim4)
            varargout{6} = gfscat_mon_p_f1nf;                               % SCATTERED - Pressure from monopole source
            varargout{7} = gfscat_dip_p_f1nf;                               % SCATTERED - Pressure from dipole source (x-,y-,z-derivative of monopole at level 1,2,3 of dim4)
        elseif strcmp(freqflag,'freqy_1s')
            varargout{1} = [0;freq];                                                                                    % FREQUENCY vector 
            varargout{2} = [zeros(1,nrxs,nsrx);gf_mon_p_f1nf(1:nf-1,:,:);real(gf_mon_p_f1nf(nf,:,:))];                  % TOTAL - Pressure from monopole source
            varargout{3} = [zeros(1,nrxs,nsrx,ndim);gf_dip_p_f1nf(1:nf-1,:,:,:);real(gf_dip_p_f1nf(nf,:,:,:))];         % TOTAL - Pressure from dipole source (x-,y-,z-derivative of monopole at level 1,2,3 of dim4)
            varargout{4} = [zeros(1,nrxs,nsrx);gfdir_mon_p_f1nf(1:nf-1,:,:);real(gfdir_mon_p_f1nf(nf,:,:))];            % DIRECT - Pressure from monopole source
            varargout{5} = [zeros(1,nrxs,nsrx,ndim);gfdir_dip_p_f1nf(1:nf-1,:,:,:);real(gfdir_dip_p_f1nf(nf,:,:,:))];   % DIRECT - Pressure from dipole source (x-,y-,z-derivative of monopole at level 1,2,3 of dim4)
            varargout{6} = [zeros(1,nrxs,nsrx);gfscat_mon_p_f1nf(1:nf-1,:,:);real(gfscat_mon_p_f1nf(nf,:,:))];          % SCATTERED - Pressure from monopole source
            varargout{7} = [zeros(1,nrxs,nsrx,ndim);gfscat_dip_p_f1nf(1:nf-1,:,:,:);real(gfscat_dip_p_f1nf(nf,:,:,:))]; % SCATTERED - Pressure from dipole source (x-,y-,z-derivative of monopole at level 1,2,3 of dim4)
        elseif strcmp(freqflag,'freqy_2s')
            gf_dip_p_f = zeros(2*nf,nrxs,nsrx,ndim);
            gfdir_dip_p_f2s = zeros(2*nf,nrxs,nsrx,ndim);
            gfscat_dip_p_f2s = zeros(2*nf,nrxs,nsrx,ndim);
            for dd = 1:ndim
                gf_dip_p_dd = centerfreqs(gf_dip_p_f1nf(:,:,:,dd),nf);
                gf_dip_p_f(:,:,:,dd) = gf_dip_p_dd;                         
                clear gf_dip_p_dd
                gfdir_dip_p_dd = centerfreqs(gfdir_dip_p_f1nf(:,:,:,dd),nf);
                gfdir_dip_p_f2s(:,:,:,dd) = gfdir_dip_p_dd;
                clear gfdir_dip_p_dd
                gfscat_dip_p_dd = centerfreqs(gfscat_dip_p_f1nf(:,:,:,dd),nf);
                gfscat_dip_p_f2s(:,:,:,dd) = gfscat_dip_p_dd;
                clear gfscat_dip_p_dd
            end
            varargout{1} = [flipdim(-freq,1);0;freq(1:end-1)];              % FREQUENCY vector 
            varargout{2} = centerfreqs(gf_mon_p_f1nf,nf);                   % TOTAL - Pressure from monopole source
            varargout{3} = gf_dip_p_f;                                      % TOTAL - Pressure from dipole source (x-,y-,z-derivative of monopole at level 1,2,3 of dim4)
            varargout{4} = centerfreqs(gfdir_mon_p_f1nf,nf);                % DIRECT - Pressure from monopole source
            varargout{5} = gfdir_dip_p_f2s;                                 % DIRECT - Pressure from dipole source (x-,y-,z-derivative of monopole at level 1,2,3 of dim4)
            varargout{6} = centerfreqs(gfscat_mon_p_f1nf,nf);               % SCATTERED - Pressure from monopole source
            varargout{7} = gfscat_dip_p_f2s;                                % SCATTERED - Pressure from dipole source (x-,y-,z-derivative of monopole at level 1,2,3 of dim4)
        end
        
        % Waveforms in TIME domain
        if strcmp(timeflag,'timey')
            gf_mon_p_ff = centerfreqs(gf_mon_p_f1nf,nf);
            gfdir_mon_p_ff = centerfreqs(gfdir_mon_p_f1nf,nf);
            gfscat_mon_p_ff = centerfreqs(gfscat_mon_p_f1nf,nf);
            gf_dip_p_ff = zeros(2*nf,nrxs,nsrx,ndim);
            gfdir_dip_p_ff = zeros(2*nf,nrxs,nsrx,ndim);
            gfscat_dip_p_ff = zeros(2*nf,nrxs,nsrx,ndim);
            for dd = 1:ndim
                gf_dip_p_dd = centerfreqs(gf_dip_p_f1nf(:,:,:,dd),nf);
                gf_dip_p_ff(:,:,:,dd) = gf_dip_p_dd;
                clear gf_dip_p_dd
                gfdir_dip_p_dd = centerfreqs(gfdir_dip_p_f1nf(:,:,:,dd),nf);
                gfdir_dip_p_ff(:,:,:,dd) = gfdir_dip_p_dd;
                clear gfdir_dip_p_dd
                gfscat_dip_p_dd = centerfreqs(gfscat_dip_p_f1nf(:,:,:,dd),nf);
                gfscat_dip_p_ff(:,:,:,dd) = gfscat_dip_p_dd;
                clear gfscat_dip_p_dd
            end
            varargout{8} = time;                                                        % TIME vector
            varargout{9} = real(sqrt(2*nf).*ifft(fftshift(gf_mon_p_ff,1),[],1));        % TOTAL - Pressure from monopole source
            varargout{10} = real(sqrt(2*nf).*ifft(fftshift(gf_dip_p_ff,1),[],1));       % TOTAL - Pressure from dipole source (x-,y-,z-derivative of monopole at level 1,2,3 of dim4)
            varargout{11} = real(sqrt(2*nf).*ifft(fftshift(gfdir_mon_p_ff,1),[],1));	% DIRECT - Pressure from monopole source
            varargout{12} = real(sqrt(2*nf).*ifft(fftshift(gfdir_dip_p_ff,1),[],1));    % DIRECT - Pressure from dipole source (x-,y-,z-derivative of monopole at level 1,2,3 of dim4)
            varargout{13} = real(sqrt(2*nf).*ifft(fftshift(gfscat_mon_p_ff,1),[],1));	% SCATTERED - Pressure from monopole source
            varargout{14} = real(sqrt(2*nf).*ifft(fftshift(gfscat_dip_p_ff,1),[],1));   % SCATTERED - Pressure from dipole source (x-,y-,z-derivative of monopole at level 1,2,3 of dim4) 
        end
        
        % Output wavelet if applied
        if strcmp(wavlflag,'wavly')
            if strcmp(timeflag,'timen')
                if strcmp(freqflag,'freqy_1nf')
                    varargout{8} = wavelet_f1nf;                            % WAVELET, frequency (1:nf)
                elseif strcmp(freqflag,'freqy_1s')
                    varargout{8} = [0;wavelet_f1nf];                        % WAVELET, frequency (0:nf)
                elseif strcmp(freqflag,'freqy_2s')
                    varargout{8} = wavelet_f2s;                             % WAVELET, frequency (-nf:-1,0,1:nf-1)
                end
            elseif strcmp(timeflag,'timey')
                if strcmp(freqflag,'freqy_1nf')
                    varargout{15} = wavelet_f1nf;                           % WAVELET, frequency (1:nf)
                    varargout{16} = wavelet_t;                              % WAVELET, time
                elseif strcmp(freqflag,'freqy_1s')
                    varargout{15} = [0;wavelet_f1nf];                       % WAVELET, frequency (0:nf)
                    varargout{16} = wavelet_t;                              % WAVELET, time
                elseif strcmp(freqflag,'freqy_2s')
                    varargout{15} = wavelet_f2s;                            % WAVELET, frequency (-nf:-1,0,1:nf-1)
                    varargout{16} = wavelet_t;                              % WAVELET, time
                end
            end
        end
        
    elseif strcmp(freqflag,'freqn')
        
        % Waveforms in TIME domain
        if strcmp(timeflag,'timey')
            gf_mon_p_ff = centerfreqs(gf_mon_p_f1nf,nf);
            gfdir_mon_p_ff = centerfreqs(gfdir_mon_p_f1nf,nf);
            gfscat_mon_p_ff = centerfreqs(gfscat_mon_p_f1nf,nf);
            gf_dip_p_ff = zeros(2*nf,nrxs,nsrx,ndim);
            gfdir_dip_p_ff = zeros(2*nf,nrxs,nsrx,ndim);
            gfscat_dip_p_ff = zeros(2*nf,nrxs,nsrx,ndim);
            for dd = 1:ndim
                gf_dip_p_dd = centerfreqs(gf_dip_p_f1nf(:,:,:,dd),nf);
                gf_dip_p_ff(:,:,:,dd) = gf_dip_p_dd;
                clear gf_dip_p_dd
                gfdir_dip_p_dd = centerfreqs(gfdir_dip_p_f1nf(:,:,:,dd),nf);
                gfdir_dip_p_ff(:,:,:,dd) = gfdir_dip_p_dd;
                clear gfscat_dip_p_dd
                gfscat_dip_p_dd = centerfreqs(gfscat_dip_p_f1nf(:,:,:,dd),nf);
                gfscat_dip_p_ff(:,:,:,dd) = gfscat_dip_p_dd;
                clear gfscat_dip_p_dd
            end
            varargout{1} = time;                                                        % TIME vector
            varargout{2} = real(sqrt(2*nf).*ifft(fftshift(gf_mon_p_ff,1),[],1));        % TOTAL - Pressure from monopole source
            varargout{3} = real(sqrt(2*nf).*ifft(fftshift(gf_dip_p_ff,1),[],1));        % TOTAL - Pressure from dipole source (x-,y-,z-derivative of monopole at level 1,2,3 of dim4)
            varargout{4} = real(sqrt(2*nf).*ifft(fftshift(gfdir_mon_p_ff,1),[],1));     % DIRECT - Pressure from monopole source
            varargout{5} = real(sqrt(2*nf).*ifft(fftshift(gfdir_dip_p_ff,1),[],1));     % DIRECT - Pressure from dipole source (x-,y-,z-derivative of monopole at level 1,2,3 of dim4)
            varargout{6} = real(sqrt(2*nf).*ifft(fftshift(gfscat_mon_p_ff,1),[],1));	% SCATTERED - Pressure from monopole source
            varargout{7} = real(sqrt(2*nf).*ifft(fftshift(gfscat_dip_p_ff,1),[],1));    % SCATTERED - Pressure from dipole source (x-,y-,z-derivative of monopole at level 1,2,3 of dim4)
            
            % Output wavelet if applied
            if strcmp(wavlflag,'wavly')
                varargout{8} = wavelet_t;                                   % WAVELET, time
            end
        end
    end
    
else
    error('Check inputs/outputs are consistent!')
end


end
