function [axis_out,data_out] = do_xcorr_int(axis_in,data_m,virtsrx,virtrcx,ro,c,dx,intype,sourceflag,ftflag,typeflag,methodflag,varargin)

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% This code is distributed as part of the source-code package 
%                   acoustic_modelling_foldy_v.4.1_seg 
% that accompanies Galetti et al. (2013). The package can be downloaded 
% from the Geophysics source-code archive at 
%                   http://software.seg.org/2013/0003
% Use of this code is subject to acceptance of the terms and conditions
% that can be found at http://software.seg.org/disclaimer.txt 
% Copyright 2013 by The Society of Exploration Geophysicists (SEG)
% Reference:
% Galetti, E., Halliday, D., Curtis, A. (2013) A simple and exact acoustic 
% wavefield modelling code for data processing, imaging and interferometry 
% applications. Geophysics.
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% [axis_out,data_out] = do_xcorr_int(axis_in,data_m,virtsrx,virtrcx,ro,c,dx,intype,sourceflag,ftflag,typeflag,methodflag,varargin)
%
% Performs CORRELATIONAL seismic interferometry (ACOUSTIC case) in either
% the frequency domain (using matrix multiplication one frequency at a
% time) or in the time domain (using time-domain xcorrelation). Either the
% full interferometry formulae (including dipole sources) or the
% monopole-only formulae can be used, from either Wapenaar & Fokkema (2006)
% or van Manen et al. (2005).
%
%
% Input:
%           axis_in     frequency vector if doing xcorr in frequency domain
%                       dt if doing xcorr in time domain
%           data_m      data from monopole sources having:
%                           - frequency/time along dim1
%                           - receivers along dim2
%                           - sources/records along dim3
%           virtsrx     - if doing inter-receiver interferometry: 
%                           ---> select which receivers (columns along dim2
%                                in data_m) will act as virtual sources
%                       - if doing inter-source interferometry: 
%                           ---> select which sources (columns along dim3
%                                in data_m) will act as sources
%           virtrcx     - if doing inter-receiver interferometry: 
%                           ---> select which receivers (columns along dim2
%                                in data_m) will act as receivers
%                       - if doing inter-source interferometry: 
%                           ---> select which sources (columns along dim3
%                                in data_m) will act as virtual receivers
%           ro          mass density of the medium
%           c           velocity of the medium
%           dx          spacing of sources along boundary
%           intype      type of interferometry
%                       'intrec' inter-receiver
%                       'intsrc' inter-source
%           sourceflag  source type: 'active' or 'passive'
%           ftflag      'freq'  do interferometry in frequency domain
%                       'time'  do interferometry in time domain
%           typeflag    'full'  do full interferometry (using monopole and
%                               dipole sources)
%                       'mono'  do monopole interferometry (using monopole
%                               sources only)
%           methodflag  'VMvi'  --- van Manen et al. (2005)
%                                   volume injection source
%                       'WFvir' --- Wapenaar & Fokkema (2006)
%                                   volume injection rate source
%                       'WFvi'  --- Wapenaar & Fokkema (2006)
%                                   volume injection source
%                       'VMvir' --- Equivalent to van Manen et al. (2005)
%                                   with a volume injection rate source
%           varargin    data from dipole sources having:
%                           - frequency/time along dim1
%                           - receivers along dim2
%                           - sources/records along dim3
%
% Output:
%           axis_out   frequency vector if doing xcorr in frequency domain
%                      time vector if doing xcorr in time domain
%           data_out   interferometry results having
%                           - frequency/time along dim1
%                           - receivers along dim2
%                           - virtual sources along dim3
%
% NOTE: the inter-receiver and inter-source formulae used within this code
% are listed in the user manual for the Foldy modelling code that this
% function is distributed with.
%
% References:
%       Curtis, A., Nicolson, H., Halliday, D., Trampert, J., Baptie, B.
%   (2009) Virtual seismometers in the subsurface of the earth from
%   seismic interferometry. Nature Geosci 2 (10): 700–704.
%       van Manen, D.-J., Robertsson, J.O.A., Curtis, A. (2005) Modeling of
%   Wave Propagation in Inhomogeneous Media. Physical Review Letters, 94:
%   164301.
%       Wapenaar, K., Fokkema, J. (2006) Green's function representations
%   for seismic interferometry, Geophysics, 71: SI33-SI46.
%
%
% Erica Galetti, July 2012
% Contact: erica.galetti@ed.ac.uk


% Error messages for invalid flags
if ~strcmp(sourceflag,'active') && ~strcmp(sourceflag,'passive')
    error('Invalid input for source type. Choose between ''active'' and ''passive''.')
elseif ~strcmp(ftflag,'freq') && ~strcmp(ftflag,'time')
    error('Invalid input for domain. Choose between ''freq'' and ''time''.')
elseif ~strcmp(typeflag,'full') && ~strcmp(typeflag,'mono')
    error('Invalid input for type flag. Choose between ''full'' and ''mono''.')
elseif ~strcmp(methodflag,'WFvi') && ~strcmp(methodflag,'WFvir') && ~strcmp(methodflag,'VMvi') && ~strcmp(methodflag,'VMvir')
    error('Invalid input for method flag. Choose between ''WFvi'', ''WFvir'', ''VMvi'' and ''VMvir''.')
end


% Output an error message if trying to do full interferometry using passive
% sources (no dipole sources in ambient noise!)
if strcmp(sourceflag,'passive') && strcmp(typeflag,'full')
    error('Can''t do full interferometry with passive noise sources - dipoles are not available!')
end


% Output an error message if trying to do inter-source interferometry using
% passive sources
if strcmp(sourceflag,'passive') && strcmp(intype,'intsrc')
    error('Can''t do inter-source interferometry with passive noise sources!')
end


% Get dipole responses from varargin
if strcmp(typeflag,'full')
    if nargin~=13
        error('Check inputs are consistent (e.g. you need to input dipole-source data to perform full interferometry).')
    else
        data_d = varargin{1};
    end
elseif strcmp(typeflag,'mono')
    if nargin~=12
        error('Check inputs are consistent (e.g. dipole responses not needed in input if doing monopole interferometry).')
    end
end


% Get number of samples, receivers and sources from input
ns = size(data_m,1);
nrcx = size(data_m,2);
if strcmp(typeflag,'full') && nrcx ~= size(data_d,2); error('There is a mismatch between the number of receivers in the monopole and dipole data.'); end
nsrx = size(data_m,3);
if strcmp(typeflag,'full') && nsrx ~= size(data_d,3); error('There is a mismatch between the number of sources in the monopole and dipole data.'); end
nvirtsrx = length(virtsrx);
nvirtrcx = length(virtrcx);


% Reshape dx to be a column vector
if strcmp(sourceflag,'active')
    if strcmp(intype,'intrec')
        dx = reshape(dx,nsrx,1);
    elseif strcmp(intype,'intsrc')
        dx = reshape(dx,nrcx,1);
    end
% If doing ambient noise interferometry, dx is not needed (set it to a
% column vector of ones of length equal to the number of records)
elseif strcmp(sourceflag,'passive')
    dx = ones(nsrx,1);
end


% Get density values
if numel(ro)==1
    if strcmp(typeflag,'full')
        if strcmp(intype,'intrec')
            ro = ones(nsrx,1).*ro;
        elseif strcmp(intype,'intsrc')
            ro = ones(nrcx,1).*ro;
        end
    end
elseif numel(ro)==nsrx
    if strcmp(typeflag,'full')
        if strcmp(intype,'intrec')
            ro = reshape(ro,nsrx,1);
        elseif strcmp(intype,'intsrc')
            ro = reshape(ro,nrcx,1);
        end
    elseif strcmp(typeflag,'mono')
        error('Density is assumed to be the same for all sources in monopole interferometry.')
    end
else
    error('Check density values are consistent with type of interferometry, number of sources/receivers etc.')
end



%% Interferometry in frequency domain

if strcmp(ftflag,'freq')
    
    % Allocate space for interferometry results
    int_f = zeros(ns,nvirtrcx,nvirtsrx);
    
    om = 2*pi.*axis_in; % angular frequencies
    
    % Do interferometry (frequencies along dim 1, receivers along dim 2, virtual sources along dim 3 of matrix of interferometry results)
    for ff = 1:ns
        
        if om(ff)~=0
            
            if strcmp(typeflag,'full')
                
                % Get responses at appropriate frequency
                tmp_m(1:nrcx,1:nsrx) = data_m(ff,:,:);
                tmp_d(1:nrcx,1:nsrx) = data_d(ff,:,:);
                
                if strcmp(intype,'intsrc')
                    tmp_m = tmp_m.';
                    tmp_d = tmp_d.';
                end
                
                % Evaluate GFs with seismic interferometry
                % Matrix multiplication automatically sums over sources
                if strcmp(methodflag,'WFvir'); tmp_int(1,:,:) = -1/(1i*om(ff)).*((tmp_d(virtrcx,:))*((tmp_m(virtsrx,:))./(repmat(ro.',nvirtsrx,1)).*(repmat(dx.',nvirtsrx,1)))'-(tmp_m(virtrcx,:))*((tmp_d(virtsrx,:))./(repmat(ro.',nvirtsrx,1)).*(repmat(dx.',nvirtsrx,1)))'); % Wapenaar & Fokkema (2006), eq. (19)
                elseif strcmp(methodflag,'WFvi'); tmp_int(1,:,:) = ((tmp_d(virtrcx,:))*((tmp_m(virtsrx,:))./(repmat(ro.',nvirtsrx,1)).*(repmat(dx.',nvirtsrx,1)))'-(tmp_m(virtrcx,:))*((tmp_d(virtsrx,:))./(repmat(ro.',nvirtsrx,1)).*(repmat(dx.',nvirtsrx,1)))'); % Wapenaar & Fokkema (2006), eq. (21)
                elseif strcmp(methodflag,'VMvi'); tmp_int(1,:,:) = ((tmp_d(virtrcx,:))*((tmp_m(virtsrx,:))./(repmat(ro.',nvirtsrx,1)).*(repmat(dx.',nvirtsrx,1)))'-(tmp_m(virtrcx,:))*((tmp_d(virtsrx,:))./(repmat(ro.',nvirtsrx,1)).*(repmat(dx.',nvirtsrx,1)))'); % van Manen et al. (2005), eq. (4)
                elseif strcmp(methodflag,'VMvir'); tmp_int(1,:,:) = 1/(1i*om(ff)).*((tmp_d(virtrcx,:))*((tmp_m(virtsrx,:))./(repmat(ro.',nvirtsrx,1)).*(repmat(dx.',nvirtsrx,1)))'-(tmp_m(virtrcx,:))*((tmp_d(virtsrx,:))./(repmat(ro.',nvirtsrx,1)).*(repmat(dx.',nvirtsrx,1)))'); % Follows from van Manen et al. (2005), eq. (4) using a volume injection rate source
                end
                int_f(ff,:,:) = int_f(ff,:,:) + tmp_int;
                
            elseif strcmp(typeflag,'mono')
                
                % Get responses at appropriate frequency
                tmp_m(1:nrcx,1:nsrx) = data_m(ff,:,:);
                
                if strcmp(intype,'intsrc')
                    tmp_m = tmp_m.';
                end
                
                % Evaluate GFs with seismic interferometry
                % Matrix multiplication automatically sums over sources
                if strcmp(methodflag,'WFvir'); tmp_int(1,:,:) = 2/(c*ro).*(tmp_m(virtrcx,:)*((tmp_m(virtsrx,:)).*(repmat(dx.',nvirtsrx,1)))'); % Wapenaar & Fokkema (2006), eq. (31)
                elseif strcmp(methodflag,'WFvi'); tmp_int(1,:,:) = -2*1i*om(ff)/(c*ro).*(tmp_m(virtrcx,:)*((tmp_m(virtsrx,:)).*(repmat(dx.',nvirtsrx,1)))'); % Wapenaar & Fokkema (2006), eq. (32)
                elseif strcmp(methodflag,'VMvi'); tmp_int(1,:,:) = -2*1i*om(ff)/(c*ro).*(tmp_m(virtrcx,:)*((tmp_m(virtsrx,:)).*(repmat(dx.',nvirtsrx,1)))'); % van Manen et al. (2005), eq. (4), simplified using the Wapenaar & Fokkema (2006) approach
                elseif strcmp(methodflag,'VMvir'); tmp_int(1,:,:) = -2/(c*ro).*(tmp_m(virtrcx,:)*((tmp_m(virtsrx,:)).*(repmat(dx.',nvirtsrx,1)))'); % Follows from van Manen et al. (2005), eq. (4) using a volume injection rate source
                end
                int_f(ff,:,:) = int_f(ff,:,:) + tmp_int;
                
            end
            
            clear tmp_int
            
        end
        
        clear tmp_m tmp_d
        
    end
    
    % Outputs
    axis_out = axis_in;
    data_out = int_f;
    
    
    
    %% Interferometry in time domain
    
elseif strcmp(ftflag,'time')
    
    % Allocate space for interferometry results
    int_t = zeros(2*ns-1,nvirtrcx,nvirtsrx);
    
    if strcmp(intype,'intrec')
        
        for nrs = 1:nvirtsrx % loop over virtual sources
            
            rs = virtsrx(nrs);
            
            for nrr = 1:nvirtrcx % loop over virtual receivers
                
                rr = virtrcx(nrr);
                
                for ss = 1:nsrx % loop over sources on boundary
                    
                    if  strcmp(typeflag,'full')
                        tmp_d_vr = data_d(:,rr,ss);
                        tmp_d_vs = data_d(:,rs,ss);
                        tmp_m_vr = data_m(:,rr,ss);
                        tmp_m_vs = data_m(:,rs,ss);
                        % Evaluate GFs with seismic interferometry
                        % Compute the integrand
                        if strcmp(methodflag,'WFvir'); tmp = -cumtrapz((conv(tmp_d_vr,tmp_m_vs(end:-1:1)) - conv(tmp_m_vr,tmp_d_vs(end:-1:1))).*dx(ss)./ro(ss)./sqrt(2.^nextpow2(ns*2-1))).*axis_in; % Wapenaar & Fokkema (2006), eq. (19)
                        elseif strcmp(methodflag,'WFvi'); tmp = (conv(tmp_d_vr,tmp_m_vs(end:-1:1)) - conv(tmp_m_vr,tmp_d_vs(end:-1:1))).*dx(ss)./ro(ss)./sqrt(2.^nextpow2(ns*2-1)); % Wapenaar & Fokkema (2006), eq. (21)
                        elseif strcmp(methodflag,'VMvi'); tmp = (conv(tmp_d_vr,tmp_m_vs(end:-1:1)) - conv(tmp_m_vr,tmp_d_vs(end:-1:1))).*dx(ss)./ro(ss)./sqrt(2.^nextpow2(ns*2-1)); % van Manen et al. (2005), eq. (4)
                        elseif strcmp(methodflag,'VMvir'); tmp = cumtrapz((conv(tmp_d_vr,tmp_m_vs(end:-1:1)) - conv(tmp_m_vr,tmp_d_vs(end:-1:1))).*dx(ss)./ro(ss)./sqrt(2.^nextpow2(ns*2-1))).*axis_in; % Follows from van Manen et al. (2005), eq. (4) using a volume injection rate source
                        end
                        int_t(:,nrr,nrs) = tmp + int_t(:,nrr,nrs); % sum over sources
                    elseif strcmp(typeflag,'mono')
                        tmp_m_vr = data_m(:,rr,ss);
                        tmp_m_vs = data_m(:,rs,ss);
                        % Evaluate GFs with seismic interferometry
                        % Compute the integrand
                        if strcmp(methodflag,'WFvir'); tmp = 2./(c*ro).*(conv(tmp_m_vr,tmp_m_vs(end:-1:1))./sqrt(2.^nextpow2(ns*2-1))).*dx(ss); % Wapenaar & Fokkema (2006), eq. (31)
                        elseif strcmp(methodflag,'WFvi'); tmp = -2./(c*ro).*gradient((conv(tmp_m_vr,tmp_m_vs(end:-1:1))./sqrt(2.^nextpow2(ns*2-1))).*dx(ss),axis_in); % Wapenaar & Fokkema (2006), eq. (32)
                        elseif strcmp(methodflag,'VMvi'); tmp = -2./(c*ro).*gradient((conv(tmp_m_vr,tmp_m_vs(end:-1:1))./sqrt(2.^nextpow2(ns*2-1))).*dx(ss),axis_in); % van Manen et al. (2005), eq. (4), simplified using the Wapenaar & Fokkema (2006) approach
                        elseif strcmp(methodflag,'VMvir'); tmp = -2./(c*ro).*(conv(tmp_m_vr,tmp_m_vs(end:-1:1))./sqrt(2.^nextpow2(ns*2-1))).*dx(ss); % Follows from van Manen et al. (2005), eq. (4) using a volume injection rate source
                        end
                        int_t(:,nrr,nrs) = tmp + int_t(:,nrr,nrs); % sum over sources
                    end
                    
                    clear tmp
                    
                end
                
            end
            
        end
        
    elseif strcmp(intype,'intsrc')
        
        for nss = 1:nvirtsrx % loop over virtual sources
            
            ss = virtsrx(nss);
            
            for nsr = 1:nvirtrcx % loop over virtual receivers
                
                sr = virtrcx(nsr);
                
                for rr = 1:nrcx % loop over receivers on boundary
                    
                    if  strcmp(typeflag,'full')
                        tmp_d_vr = data_d(:,rr,sr);
                        tmp_d_vs = data_d(:,rr,ss);
                        tmp_m_vr = data_m(:,rr,sr);
                        tmp_m_vs = data_m(:,rr,ss);
                        % Evaluate GFs with seismic interferometry
                        % Compute the integrand
                        if strcmp(methodflag,'WFvir'); tmp = -cumtrapz((conv(tmp_d_vr,tmp_m_vs(end:-1:1)) - conv(tmp_m_vr,tmp_d_vs(end:-1:1))).*dx(rr)./ro(rr)./sqrt(2.^nextpow2(ns*2-1))).*axis_in; % Wapenaar & Fokkema (2006), eq. (19)
                        elseif strcmp(methodflag,'WFvi'); tmp = (conv(tmp_d_vr,tmp_m_vs(end:-1:1)) - conv(tmp_m_vr,tmp_d_vs(end:-1:1))).*dx(rr)./ro(rr)./sqrt(2.^nextpow2(ns*2-1)); % Wapenaar & Fokkema (2006), eq. (21)
                        elseif strcmp(methodflag,'VMvi'); tmp = (conv(tmp_d_vr,tmp_m_vs(end:-1:1)) - conv(tmp_m_vr,tmp_d_vs(end:-1:1))).*dx(rr)./ro(rr)./sqrt(2.^nextpow2(ns*2-1)); % van Manen et al. (2005), eq. (4)
                        elseif strcmp(methodflag,'VMvir'); tmp = cumtrapz((conv(tmp_d_vr,tmp_m_vs(end:-1:1)) - conv(tmp_m_vr,tmp_d_vs(end:-1:1))).*dx(rr)./ro(rr)./sqrt(2.^nextpow2(ns*2-1))).*axis_in; % Follows from van Manen et al. (2005), eq. (4) using a volume injection rate source
                        end
                        int_t(:,nsr,nss) = tmp + int_t(:,nsr,nss); % sum over sources
                    elseif strcmp(typeflag,'mono')
                        tmp_m_vr = data_m(:,rr,sr);
                        tmp_m_vs = data_m(:,rr,ss);
                        % Evaluate GFs with seismic interferometry
                        % Compute the integrand
                        if strcmp(methodflag,'WFvir'); tmp = 2./(c*ro).*(conv(tmp_m_vr,tmp_m_vs(end:-1:1))./sqrt(2.^nextpow2(ns*2-1))).*dx(rr); % Wapenaar & Fokkema (2006), eq. (31)
                        elseif strcmp(methodflag,'WFvi'); tmp = -2./(c*ro).*gradient((conv(tmp_m_vr,tmp_m_vs(end:-1:1))./sqrt(2.^nextpow2(ns*2-1))).*dx(rr),axis_in); % Wapenaar & Fokkema (2006), eq. (32)
                        elseif strcmp(methodflag,'VMvi'); tmp = -2./(c*ro).*gradient((conv(tmp_m_vr,tmp_m_vs(end:-1:1))./sqrt(2.^nextpow2(ns*2-1))).*dx(rr),axis_in); % van Manen et al. (2005), eq. (4), simplified using the Wapenaar & Fokkema (2006) approach
                        elseif strcmp(methodflag,'VMvir'); tmp = -2./(c*ro).*(conv(tmp_m_vr,tmp_m_vs(end:-1:1))./sqrt(2.^nextpow2(ns*2-1))).*dx(rr); % Follows from van Manen et al. (2005), eq. (4) using a volume injection rate source
                        end
                        int_t(:,nsr,nss) = tmp + int_t(:,nsr,nss); % sum over sources
                    end
                    
                    clear tmp
                    
                end
                
            end
            
        end
        
    end
    
    % Outputs
    axis_out = (-(ns-1):1:ns-1).*axis_in;
    data_out = real(int_t);
    
end

end
