function [hline,hfill]=wiggles(data,time,offset,refine,scale,color,orient,fill)

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% This code is distributed as part of the source-code package 
%                   acoustic_modelling_foldy_v.4.1_seg 
% that accompanies Galetti et al. (2013). The package can be downloaded 
% from the Geophysics source-code archive at 
%                   http://software.seg.org/2013/0003
% Use of this code is subject to acceptance of the terms and conditions
% that can be found at http://software.seg.org/disclaimer.txt 
% Copyright 2013 by The Society of Exploration Geophysicists (SEG)
% Reference:
% Galetti, E., Halliday, D., Curtis, A. (2013) A simple and exact acoustic 
% wavefield modelling code for data processing, imaging and interferometry 
% applications. Geophysics.
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
%   [hline,hfill]=wiggles(data,time,offset,refine,scale,color,orient,fill)
%
%   Plots a shaded wiggle plot and returns a set of handles (line and fill)
%   for each wiggle in the plot.
% 
%   Input:
%           data        columns of waveform data (time along dim1, offsets
%                       along dim2)
%
%   Optional input:
%           time        time vector
%           offset      offset vector
%           refine      interpolates the deta on a finer time axis of
%                       length refine*length(time) (default = 10)
%           scale       scaling factor (default = 1)
%           color       color of wiggles (default = 'k')
%           orient      plot orientation:
%                          'down'       offsets along x-axis, time along
%                                       y-axis (default)
%                          'across'     time along x-axis, offsets along
%                                       y-axis
%           fill        fill positive or negative values?
%                           'pos'   fill positive values (default)
%                           'neg'   fill negative values
%
%   Output:
%           hline       line handles
%           hfill       fill handles
%
%
%   Erica Galetti, May 2012
%   Contact: erica.galetti@ed.ac.uk



% Define defaults
if nargin < 1
    error('Need to input at least waveform data!')
elseif nargin < 2
    [nt,nx] = size(data);
    time = 1:nt;
    offset = 1:nx;
    refine = 10;
    scale = 1;
    color = 'k';
    orient = 'down';
    fill = 'pos';
elseif nargin < 3
    [nt,nx] = size(data);
    offset = 1:nx;
    refine = 10;
    scale = 1;
    color = 'k';
    orient = 'down';
    fill = 'pos';
elseif nargin < 4
    [nt,nx] = size(data);
    refine = 10;
    scale = 1;
    color = 'k';
    orient = 'down';
    fill = 'pos';
elseif nargin < 5
    [nt,nx] = size(data);
    scale = 1;
    color = 'k';
    orient = 'down';
    fill = 'pos';
elseif nargin < 6
    [nt,nx] = size(data);
    color = 'k';
    orient = 'down';
    fill = 'pos';
elseif nargin < 7
    [nt,nx] = size(data);
    orient = 'down';
    fill = 'pos';
elseif nargin < 8
    [nt,nx] = size(data);
    fill = 'pos';
elseif nargin == 8
    [nt,nx] = size(data);
end

time = reshape(time,nt,1);
offset = reshape(offset,nx,1);
if scale == 0; scale = 1; end
if refine < 1; refine = 10; end

% Refine data by interpolation
ntn = refine*nt;
timen(1:ntn,1) = linspace(min(time),max(time),ntn);
datan = zeros(ntn,nx);
for xx = 1:nx
    datan(:,xx) = interp1(time,data(:,xx),timen);
end

datanmax = max(max(abs(datan))); % Maximum absolute amplitude in the dataset
offsort = sort(offset); % Sort offsets
dx = mean(diff(offsort)); % Find average spacing

% Scale the data
datan = dx.*datan./datanmax;
datan = datan.*scale;

% Allocate space for handles
hfill = zeros(nx,1);
hline = zeros(nx,1);

for xx = 1:nx % Loop over traces
    
    trace = datan(:,xx);
    
    % Normalise trace (one-bit)
    strace = sign(trace);
    %strace(strace==0) = 1; % Set to 1 if amplitude = 0
    
    % Find crossing points
    intloc = diff(strace);
    intlocind = find(abs(intloc)==2);
    nint = length(intlocind);
    crossings = zeros(nint,2);
    
    for ii = 1:nint % Loop over crossing points
        
        % Find local time and amplitide at the two extremes delimiting
        % crossing point ii
        loctime = timen(intlocind(ii):intlocind(ii)+1);
        locampl = trace(intlocind(ii):intlocind(ii)+1);
        
        % Interpolate to get a better dfinition
        loctimen = linspace(min(loctime),max(loctime),100);
        locampln = interp1(loctime,locampl,loctimen);
        
        % The position corresponding to the minimum absolute value of the
        % amplitude (amplitude nearest to zero) is a good approximation of
        % the position of the crossing point
        [~,crossind] = min(abs(locampln));
        crossings(ii,1) = loctimen(crossind);
    end
    
    % Include the crossing point times and positions in the dataset
    [timevals,ind] = sort([timen;crossings(:,1)]);
    tracenew = [trace;crossings(:,2)]; clear trace crossings
    tracenew = tracenew(ind);
    
    % Add zeros at the ends (otherwise the plot messes up!)
    trace_x = [0;tracenew;0];
    time_x = [timevals(1);timevals;timevals(end)];
    clear tracenew timevals
    
    % Find the positive amplitudes
    posind = find(trace_x>=0);
    negind = find(trace_x<=0);
    posampl = zeros(size(trace_x));
    posampl(posind) = trace_x(posind);
    negampl = zeros(size(trace_x));
    negampl(negind) = trace_x(negind);
    
    
    % Plot the wiggles, one trace at a time
    hold on
    if strcmp(fill,'pos')
        if strcmp(orient,'down')
            % Plot the positive amplitudes - filled
            hfill(xx) = patch(posampl+offset(xx),time_x,color);
            % Remove the zero-amplitude line
            line(ones(size(time_x)).*offset(xx),time_x,'color','w');
            % Plot the negative amplidudes
            hline(xx) = line(negampl+offset(xx),time_x,'color',color);
            set(gca,'YDir','reverse')
            set(gca,'XAxisLocation','top')
        elseif strcmp(orient,'across')
            % Plot the positive amplitudes - filled
            hfill(xx) = patch(time_x,posampl+offset(xx),color);
            % Remove the zero-amplitude line
            line(time_x,ones(size(time_x)).*offset(xx),'color','w');
            % Plot the negative amplidudes
            hline(xx) = line(time_x,negampl+offset(xx),'color',color);
        end
    elseif strcmp(fill,'neg')
        if strcmp(orient,'down')
            % Plot the negative amplitudes - filled
            hfill(xx) = patch(negampl+offset(xx),time_x,color);
            % Remove the zero-amplitude line
            line(ones(size(time_x)).*offset(xx),time_x,'color','w');
            % Plot the positive amplidudes
            hline(xx) = line(posampl+offset(xx),time_x,'color',color);
            set(gca,'YDir','reverse')
            set(gca,'XAxisLocation','top')
        elseif strcmp(orient,'across')
            % Plot the negative amplitudes - filled
            hfill(xx) = patch(time_x,negampl+offset(xx),color);
            % Remove the zero-amplitude line
            line(time_x,ones(size(time_x)).*offset(xx),'color','w');
            % Plot the positive amplidudes
            hline(xx) = line(time_x,posampl+offset(xx),'color',color);
        end
        
    end
   
    clear trace_x time_x
    
end

box on
hold off
axis tight

end
