%% Acoustic modelling and interferometry example - 2D

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% This code is distributed as part of the source-code package 
%                   acoustic_modelling_foldy_v.4.1_seg 
% that accompanies Galetti et al. (2013). The package can be downloaded 
% from the Geophysics source-code archive at 
%                   http://software.seg.org/2013/0003
% Use of this code is subject to acceptance of the terms and conditions
% that can be found at http://software.seg.org/disclaimer.txt 
% Copyright 2013 by The Society of Exploration Geophysicists (SEG)
% Reference:
% Galetti, E., Halliday, D., Curtis, A. (2013) A simple and exact acoustic 
% wavefield modelling code for data processing, imaging and interferometry 
% applications. Geophysics.
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% This script serves as an example of how the acoustic GF modelling code
% can be used to model acoustic wavefields from a boundary of sources to an
% array of enclosed receivers, and how to apply interferometry to construct
% the GF signal between them. A circular boundary of sources is used for
% interferometry. Cross-correlation is performed in the time domain (using
% the matlab function conv and time-reversing one of the traces). The
% results match the true Green's functions obtained by direct modelling
% between the receivers in the array.
%
% Figure 1: Geometry (left)
%           True modelled traces from receiver R to the receiver array
%           (right)
% Figure 2: Green's functions from R to the receiver array in time domain:
%               top left:   true modelled traces
%               top middle: interferometric traces, causal part
%               top right:  interferometric traces, acausal part
%               bottom:     comparison between the true trace highlighted
%                           in blue and the corresponding interferometric
%                           trace highlighted in red
%
%
% References:
%       Groenenboom, J., Snieder, R. (1995) Attenuation, dispersion, and
%   anisotropy by multiple scattering of transmitted waves through
%   distributions of scatterers. J. Acoust. Soc. Am. 98 (6): 3482-3492.
%       Snieder, R. (2009) A Guided Tour of Mathematical Methods for the
%   Physical Sciences, 2nd ed., Cambridge University Press.
%       van Manen, D.-J., Robertsson, J.O.A., Curtis, A. (2005) Modeling of 
%   Wave Propagation in Inhomogeneous Media. Physical Review Letters, 94: 
%   164301.
%       Wapenaar, K., Fokkema, J. (2006) Green's function representations  
%   for seismic interferometry, Geophysics, 71: SI33-SI46.
%
%
% Required:
% makecircbound.m           - creates a circular boundary of points with a
%                             certain radius or point spacing
% do_xcorr_int.m            - performs cross-correlational interferometry
% acoustic_p_gfs_direct_f.m - models direct wave between two points in an
%                             acoustic medium in the frequency domain only
% model_gfs_p_scat_acoust.m - models scattered waves in an acoustic medium
% centerfreqs.m             - creates a two-sided centred frequency
%                             spectrum from positive frequencies
% rickerwavl_time.m         - creates Ricker wavelet for source
% wiggles.m                 - creates a wiggle plot
%
%
% Erica Galetti, July 2012
% Contact: erica.galetti@ed.ac.uk


clear all
close all



%% van Manen or Wapenaar?
method = 'VMvi';    
    % 'VMvi'    --- van Manen et al. (2005)
    %                   volume injection source
    % 'WFvir'   --- Wapenaar & Fokkema (2006)
    %                   volume injection rate source
    % 'WFvi'    --- Wapenaar & Fokkema (2006)
    %                   volume injection source
    % 'VMvir'   --- Equivalent to van Manen et al. (2005) 
    %                   with a volume injection rate source          

    
%% Type of interferometry
inttype = 'full'; % Monopole ('mono') or full ('full')


%% Include scatterers?
scattering = 'yes'; % 'yes' or 'no'

% Define scatterer locations
if strcmp(scattering,'yes')
    scats = [-50,-150,0;280,50,0;-250,200,0];
    scats(:,4) = -4; % Imaginary part of the scattering amplitude - has to be within the range [-4,0] (see Groenenboom & Snieder, 1995)
end


%% Normalise amplitudes?
normamp = 'no'; % 'yes' or 'no'


%% Source wavelet type
wavltype = 'wavly_pos'; % the positive-time wavelet is not zero-phase - for an example with zero-phase wavelet, see the source-receiver interferometry example


%% Type of source
if strcmp(method,'WFvir')
    srctype='volinjratepos';
elseif strcmp(method,'WFvi')
    srctype='volinjpos';
elseif strcmp(method,'VMvir')
    srctype='volinjrateneg';
elseif strcmp(method,'VMvi')
    srctype='volinjneg';
end


%% Define a series of input parameters

nf = 200;
nf = 2^(nextpow2(nf)); % round number of frequencies to the nearest power of 2 (makes fft/ifft run faster)
maxf = 100; % max frequency
fc = 30; % central frequency of Ricker wavelet
dt = 1/(2*maxf); % time step

ro = 1; % medium density
c = 1000; % constant background velocity
nyq = c/maxf; % minimum Nyquist wavelength


%% Type of geometry
dim = '2d';
if strcmp(dim,'1d'); ndim = 1;
elseif strcmp(dim,'2d') || strcmp(dim,'2d_far'); ndim = 2;
elseif strcmp(dim,'3d'); ndim = 3;
end


%% Define source and receiver positions

% Define source boundary
% Points on circle
rad_s = 400; % radius of source boundary (gets adjusted depending on source spacing 4 lines below)
dx_s = nyq/2; % spacing of sources on boundary is half the minimum Nyquist wavelength
[coords,radnew] = makecircbound(rad_s,dx_s,'fixds',0,0);
rad_s = radnew;
nsrx = size(coords,1);
srx(1:nsrx,1) = coords(:,1); % boundary x-coordinate
srx(1:nsrx,2) = coords(:,2); % boundary y-coordinate
srx(1:nsrx,3) = 0; % boundary z-coordinate
% Calculate distance between sources (offsets_s) and ds
offsets_s = zeros(nsrx,1);
ds = zeros(nsrx,1);
offsets_s(1:nsrx) = sqrt(diff(srx([1:nsrx 1],1)).^2 + diff(srx([1:nsrx 1],2)).^2);
ds(1:nsrx) = offsets_s(1:nsrx)/2 + offsets_s([nsrx 1:nsrx-1])/2;
clear offsets_s coords radnew

% Calculate boundary normal (the boundary is a circle centred at the
% origin, so its x and y components are simply the cos and sin of the
% angles between the source-position-vector and the x axis).
boundnorm = zeros(nsrx,ndim);
for dd = 1:ndim; boundnorm(:,dd) = srx(:,dd)/rad_s; end


% Define position of receivers
rcA=[-150,0,0];
rcB(:,1)=50:10:350;
rcB(:,2:3)=0;
rcx = [rcA;rcB];
nrcx = size(rcx,1);



%% Model the real Green's function between A and B
if strcmp(scattering,'yes')
    [time1,gftrue_mon_p_t,~,wavelet_t] = model_gfs_p_scat_acoust(rcx(1,:),rcx,maxf,nf,c,dim,srctype,'srcdip','scaty',wavltype,'freqn','timey','tot',scats,fc);
elseif strcmp(scattering,'no')
    [time1,gftrue_mon_p_t,~,wavelet_t] = model_gfs_p_scat_acoust(rcx(1,:),rcx,maxf,nf,c,dim,srctype,'srcdip','scatn',wavltype,'freqn','timey','tot',fc);
end

% Define two-sided time vector
time2 = [flipud(-time1(2:end));time1(1:end)]; % negative and positive times

% Wavelet in frequency domain
wavelet_f = fft([wavelet_t;zeros(size(wavelet_t))])./sqrt(2*2*nf);
wavelet_f = wavelet_f(2:2*nf+1,:,:); % take only half of the frequency spectrum

% Frequencies
df = maxf/(2*nf);
freq2 =  (1:2*nf).*df;
freqs=[0;freq2.'];
om = 2*pi.*freq2; % angular frequencies



%% Model the boundary to receiver-network Green's functions using the model_gfs_p_scat_acoust function

% mgfs_t = monopole source, time domain
% dgfs_t = dipole source, time domain

% Allocate space
if strcmp(inttype,'mono')
    mgfs_t=zeros(2*nf,nrcx,nsrx);
elseif strcmp(inttype,'full')
    mgfs_t=zeros(2*nf,nrcx,nsrx);
    dgfs_t=zeros(2*nf,nrcx,nsrx);
end

% Loop over boundary sources

for ss = 1:nsrx
    
    if strcmp(scattering,'yes')
        
        if strcmp(inttype,'mono')
            
            % Model pressure from source ss to all receivers
            [~,gf_mon_p_t,~,~] = model_gfs_p_scat_acoust(srx(ss,:),rcx,maxf,nf,c,dim,srctype,'srcdip','scaty',wavltype,'freqn','timey','tot',scats,fc);
            mgfs_t(:,:,ss) = gf_mon_p_t; % monopole wavefield - time domain
            
        elseif strcmp(inttype,'full')
       
            % Model pressure from source ss to all receivers
            [~,gf_mon_p_t,gf_dip_p_t,~] = model_gfs_p_scat_acoust(srx(ss,:),rcx,maxf,nf,c,dim,srctype,'srcdip','scaty',wavltype,'freqn','timey','tot',scats,fc);
            mgfs_t(:,:,ss) = gf_mon_p_t(:,:,:); % monopole wavefield - time domain
            for dd = 1:ndim; dgfs_t(:,:,ss) = dgfs_t(:,:,ss) + gf_dip_p_t(:,:,1,dd).*boundnorm(ss,dd); end % weight dipole components by the outward normal components - this gives the full dipole wavefield 

        end
        
    elseif strcmp(scattering,'no')
        
        if strcmp(inttype,'mono')
                                    
            % Model pressure from source ss to all receivers
            [~,gf_mon_p_t,~,~] = model_gfs_p_scat_acoust(srx(ss,:),rcx,maxf,nf,c,dim,srctype,'srcdip','scatn',wavltype,'freqn','timey','tot',fc);
            mgfs_t(:,:,ss) = gf_mon_p_t; % monopole wavefield - time domain

        elseif strcmp(inttype,'full')
                        
            % Model pressure from source ss to all receivers
            [~,gf_mon_p_t,gf_dip_p_t,~] = model_gfs_p_scat_acoust(srx(ss,:),rcx,maxf,nf,c,dim,srctype,'srcdip','scatn',wavltype,'freqn','timey','tot',fc);
            mgfs_t(:,:,ss) = gf_mon_p_t; % monopole wavefield - time domain
            for dd = 1:ndim; dgfs_t(:,:,ss) = dgfs_t(:,:,ss) + gf_dip_p_t(:,:,1,dd).*boundnorm(ss,dd); end % weight dipole components by the outward normal components - this gives the full dipole wavefield 

        end
    end
    
    if mod(ss,20)==0; fprintf(['Done source ' int2str(ss) ' of ' int2str(nsrx) '.\n']); end
    
    clear gf_mon_p_t gf_dip_p_t
    
end

% Apply Tukey window to taper trace edges (useful in case any scattered
% arrivals occur at the end of the trace, as the traces need to be padded
% with zeros before moving to the frequency domain for cross-correlation)
% and pad with zeros to get traces twice as long.

% Construct Tukey window (this assumes the Signal Processing Toolbox is not
% available and avoids calling the tukeywin function, giving similar results)
ntap = 10; % number of time samples to taper on each side of the trace (make sure it is much less than half the total number of time samples!)
theta_t = linspace(0,1,ntap+1).';
taper = (1+cos(pi.*theta_t))./2;
% Create Tukey window as taper-ones-taper
tuk = [taper(end:-1:1);ones(2*nf-2*(ntap+1),1,1);taper];
clear ntap theta_t taper

gftrue_mon_p_t = [gftrue_mon_p_t.*repmat(tuk,[1 size(gftrue_mon_p_t,2) size(gftrue_mon_p_t,3)]);zeros(2*nf,size(gftrue_mon_p_t,2),size(gftrue_mon_p_t,3))];
mgfs_t = [mgfs_t.*repmat(tuk,[1 size(mgfs_t,2) size(mgfs_t,3)]);zeros(2*nf,size(mgfs_t,2),size(mgfs_t,3))];
if strcmp(inttype,'full'); dgfs_t = [dgfs_t.*repmat(tuk,[1 size(dgfs_t,2) size(dgfs_t,3)]);zeros(2*nf,size(dgfs_t,2),size(dgfs_t,3))]; end

% Move to frequency domain
gftrue_mon_p_f = fft(gftrue_mon_p_t,[],1)./sqrt(2*2*nf);
clear gftrue_mon_p_t
gftrue_mon_p_f = gftrue_mon_p_f(2:2*nf+1,:,:); % take only half of the frequency spectrum
mgfs_f = fft(mgfs_t,[],1)./sqrt(2*2*nf);
clear mgfs_t
mgfs_f = mgfs_f(2:2*nf+1,:,:); % take only half of the frequency spectrum
if strcmp(inttype,'full');
    dgfs_f = fft(dgfs_t,[],1)./sqrt(2*2*nf);
    clear dgfs_t
    dgfs_f = dgfs_f(2:2*nf+1,:,:); % take only half of the frequency spectrum
end



%% Perform xcorr interferometry from boundary S of sources in frequency domain using matrix multiplication one frequency at a time
% Use function do_xcorr_int to do cross-correlational interferometry

if  strcmp(inttype,'full')
    
    % Evaluate GFs with seismic interferometry
    [~,full_int_Ss_f] = do_xcorr_int(freq2,mgfs_f,1,1:nrcx,ro,c,ds,'intrec','active','freq',inttype,method,dgfs_f);
    
elseif strcmp(inttype,'mono')
    
    % Evaluate GFs with seismic interferometry
    [~,mon_int_Ss_f] = do_xcorr_int(freq2,mgfs_f,1,1:nrcx,ro,c,ds,'intrec','active','freq',inttype,method);

end

clear  mgfs_f dgfs_f



%% Select results
if  strcmp(inttype,'full')
    int_Ss_f = [sqrt(2).*full_int_Ss_f(1:2*nf-1,:,:);full_int_Ss_f(2*nf,:,:)]; % We have taken only half of the frequency spectrum, so we have to adjust the amplitudes
elseif strcmp(inttype,'mono')
    int_Ss_f = [sqrt(2).*mon_int_Ss_f(1:2*nf-1,:,:);mon_int_Ss_f(2*nf,:,:)]; % We have taken only half of the frequency spectrum, so we have to adjust the amplitudes
end
clear full_int_Ss_f mon_int_Ss_f
  


%% The true and interferometric GFs are:


% True GF
gftrue_f = gftrue_mon_p_f.*repmat(conj(wavelet_f),[1 size(gftrue_mon_p_f,2) size(gftrue_mon_p_f,3)]); % Convolve spectrum of true trace with conjugate of source wavelet
gftrue_f = [sqrt(2).*gftrue_f(1:2*nf-1,:,:);gftrue_f(2*nf,:,:)]; % We have taken only half of the frequency spectrum, so we have to adjust the amplitudes
gftrue_f_2sided = centerfreqs(gftrue_f,2*nf); % Get two-sided frequency spectrum for ifft
gftrue_t_2sided = fftshift(sqrt(2*2*nf)*ifft(fftshift(gftrue_f_2sided,1),[],1),1); % Move to time domain
gftrue_t_2sided = real([gftrue_t_2sided(end:-1:2*nf+2,:,:);gftrue_t_2sided(2*nf+1:end,:,:)]);
clear gftrue_f_2sided
gftrue_t = gftrue_t_2sided(2*nf:end,:,:);

% Interferometric GFs from boundary S of sources
gfint_f_2sided = centerfreqs(int_Ss_f,2*nf); % Get two-sided frequency spectrum for ifft
gfint_t_2sided = fftshift(sqrt(2*2*nf)*ifft(fftshift(gfint_f_2sided,1),[],1),1); % Move to time domain
gfint_t_2sided = real(gfint_t_2sided(2:end,:,:));
gfint_c_t = gfint_t_2sided(2*nf:end,:,:);
gfint_a_t = flipud(gfint_t_2sided(1:2*nf,:,:));
clear gfint_f_2sided int_Ss_f

% Normalise amplitudes
if strcmp(normamp,'yes')
    gftrue_t_2sided = gftrue_t_2sided./repmat(max(abs(gftrue_t_2sided)),[2*2*nf-1 1 1]);
    gftrue_t = gftrue_t./repmat(max(abs(gftrue_t)),[2*nf 1 1]);
    gfint_t_2sided = gfint_t_2sided./repmat(max(abs(gfint_t_2sided)),[2*2*nf-1 1 1]);
    gfint_c_t = gfint_c_t./repmat(max(abs(gfint_c_t)),[2*nf 1 1]);
    gfint_a_t = gfint_a_t./repmat(max(abs(gfint_a_t)),[2*nf 1 1]);
end



%% Green's functions

% True GF
gf_true_t = gftrue_t(:,2:end,1);
gf_true_t_2sided = gftrue_t_2sided(:,2:end,1);
clear gftrue*

% Interferometric GFs from boundary S of sources
gf_int_t_2sided = gfint_t_2sided(:,2:end,1);
gf_int_c_t = gfint_c_t(:,2:end,1);
gf_int_a_t = gfint_a_t(:,2:end,1);
clear gfint*



%% Plots

scrnsz = get(0,'screensize');
tr = 10; % Trace for single plot
amaxt=max(max(abs(gf_true_t_2sided(:,tr)),abs(gf_int_t_2sided(:,tr))));

% Plot geometry and true trace
fig1 = figure;
fig1w = 1200;
fig1h = 600;
pos = get(fig1,'Position');
outpos = get(fig1,'OuterPosition');
bord = outpos - pos; clear pos outpos
topb = bord(4)-abs(bord(2));
leftb = abs(bord(1));
set(fig1,'Position',[leftb scrnsz(4)-(topb+fig1h) fig1w fig1h])
% Plot geometry
sub1=subplot(121);
hold on
src=plot(srx(1:6:end,1),srx(1:6:end,2),'r*','markersize',8);
rec=plot(rcx([1 2:2:end],1),rcx([1 2:2:end],2),'bv','markersize',8,'markerfacecolor','b');
if strcmp(scattering,'yes')
    scat=plot(scats(:,1),scats(:,2),'go','markersize',8,'markerfacecolor','g');
end
axis equal
axis([min(srx(:,1)-50) max(srx(:,1)+50) min(srx(:,2)-50) max(srx(:,2)+50)])
box on
set(gca,'fontsize',14)
xlabel ('x (m)','fontsize',14)
ylabel ('y (m)','fontsize',14)
title ('Geometry','fontsize',14)
text(rcx(1,1),rcx(1,2)-40,'R','fontsize',14,'horizontalalignment','center')
if strcmp(scattering,'no')
    legend([src rec],'Source','Receiver','location','best')
elseif strcmp(scattering,'yes')
    legend([src rec scat],'Source','Receiver','Scatterer','location','best')
end
% Plot true traces
sub2=subplot(122);
hold on
wiggles(gf_true_t(1:(find(time1>=0.6*max(time1),1,'first')),:,1),time1(1:(find(time1>=0.6*max(time1),1,'first'))),sqrt((rcx(2:end,1)-rcx(1,1)).^2+(rcx(2:end,2)-rcx(1,2)).^2),10,1);
box on
set(gca,'fontsize',14)
xlabel ('Offset (m)','fontsize',14)
ylabel ('Time (s)','fontsize',14)

% Plot results: true GF, causal and acausal interferometric GF 
fig2=figure;
fig2w=1200;
fig2h=600;
pos = get(fig1,'Position');
set(fig2,'Position',[pos(1)+leftb scrnsz(4)-(2*topb+fig2h) fig2w fig2h])
% Plot true traces
subp1=subplot(231);
hold on
[hl,hf]=wiggles(gf_true_t(1:(find(time1>=0.6*max(time1),1,'first')),:,1),time1(1:(find(time1>=0.6*max(time1),1,'first'))),sqrt((rcx(2:end,1)-rcx(1,1)).^2+(rcx(2:end,2)-rcx(1,2)).^2),10,1);
box on
set(hl(tr),'color','b')
set(hf(tr),'edgecolor','b','facecolor','b')
set(gca,'fontsize',14)
xlabel ('Offset (m)','fontsize',14)
ylabel ('Time (s)','fontsize',14)
title ('True traces','fontsize',14)
% Plot interferometry causal
subp2=subplot(232);
hold on
[hl,hf]=wiggles(gf_int_c_t(1:(find(time1>=0.6*max(time1),1,'first')),:,1),time1(1:(find(time1>=0.6*max(time1),1,'first'))),sqrt((rcx(2:end,1)-rcx(1,1)).^2+(rcx(2:end,2)-rcx(1,2)).^2),10,1);
box on
set(hl(tr),'color','r')
set(hf(tr),'edgecolor','r','facecolor','r')
set(gca,'fontsize',14)
xlabel ('Offset (m)','fontsize',14)
ylabel ('Time (s)','fontsize',14)
title ('Interferometric traces - causal','fontsize',14)
% Plot interferometry acausal
subp3=subplot(233);
hold on
[hl,hf]=wiggles(gf_int_a_t(1:(find(time1>=0.6*max(time1),1,'first')),:,1),time1(1:(find(time1>=0.6*max(time1),1,'first'))),sqrt((rcx(2:end,1)-rcx(1,1)).^2+(rcx(2:end,2)-rcx(1,2)).^2),10,1);
box on
set(hl(tr),'color','r')
set(hf(tr),'edgecolor','r','facecolor','r')
set(gca,'fontsize',14)
xlabel ('Offset (m)','fontsize',14)
ylabel ('Time (s)','fontsize',14)
title ('Interferometric traces - acausal','fontsize',14)
% Plot interferometry - one trace, comparison
subp4=subplot(2,3,4:6);
hold on
line([0 0],[-amaxt amaxt],'Linestyle','--','color','k')
true=plot(time2,gf_true_t_2sided(:,tr),'b','linewidth',2);
int=plot(time2,gf_int_t_2sided(:,tr),'r--','linewidth',2);
axis([0.6*min(time2) 0.6*max(time2) -amaxt amaxt])
box on
set(gca,'fontsize',14)
xlabel ('Time (s)','fontsize',14)
if strcmp(normamp,'yes'); ylabel ('Normalised Amplitude','fontsize',14)
elseif strcmp(normamp,'no'); ylabel ('Amplitude','fontsize',14);end
legend([true int],'True trace','Interferometric trace')
title (['True trace vs. interferometric trace at offset ' num2str(sqrt((rcx(tr,1)-rcx(1,1)).^2+(rcx(tr,2)-rcx(1,2)).^2)) ' m'],'fontsize',14)
text(0.6*min(time2)/2,-amaxt/2,'Acausal','fontsize',14,'horizontalalignment','center')
text(0.6*max(time2)/2,-amaxt/2,'Causal','fontsize',14,'horizontalalignment','center')

set(subp1,'Position',[0.1 0.4 0.2 0.5])
set(subp2,'Position',[0.4 0.4 0.2 0.5])
set(subp3,'Position',[0.7 0.4 0.2 0.5])
set(subp4,'Position',[0.1 0.1 0.8 0.2])



