%% Acoustic modelling and interferometry example - 2D

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% This code is distributed as part of the source-code package 
%                   acoustic_modelling_foldy_v.4.1_seg 
% that accompanies Galetti et al. (2013). The package can be downloaded 
% from the Geophysics source-code archive at 
%                   http://software.seg.org/2013/0003
% Use of this code is subject to acceptance of the terms and conditions
% that can be found at http://software.seg.org/disclaimer.txt 
% Copyright 2013 by The Society of Exploration Geophysicists (SEG)
% Reference:
% Galetti, E., Halliday, D., Curtis, A. (2013) A simple and exact acoustic 
% wavefield modelling code for data processing, imaging and interferometry 
% applications. Geophysics.
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% This script serves as an example of how the acoustic GF modelling code
% can be used to model acoustic wavefields from two enclosed sources to
% a boundary of receivers, and how to apply interferometry to construct
% the GF signal between them. A circular boundary of receivers is used for
% interferometry. Cross-correlation is performed in the frequency domain
% (using matrix multiplication one frequency at a time). The results match
% the true Green's functions obtained by direct modelling between the two
% sources.
%
% Figure 1: Geometry (top)
%           True modelled trace (bottom)
% Figure 2: Two-sided Green's functions in time domain:
%               black solid trace - true Green's function 
%               red dashed trace - interferometric Green's function
% Figure 3: Comparison: 'true' Green's function (black solid trace -
%                       obtained from the causal part of the true Green's 
%                       function in Figure 2 according to the 
%                       interferometry formulae) vs. interferometric
%                       Green's function (red dashed trace)
%                       top:            traces in time domain
%                       bottom left:    frequency spectra 
%                       bottom centre:  real part of frequency spectra 
%                       bottom right:   imaginary part of frequency spectra
%
%
% References:
%       Curtis, A., Nicolson, H., Halliday, D., Trampert, J., Baptie, B.
%   (2009) Virtual seismometers in the subsurface of the earth from
%   seismic interferometry. Nature Geosci 2 (10): 700–704.
%       Groenenboom, J., Snieder, R. (1995) Attenuation, dispersion, and
%   anisotropy by multiple scattering of transmitted waves through
%   distributions of scatterers. J. Acoust. Soc. Am. 98 (6): 3482-3492.
%       Snieder, R. (2009) A Guided Tour of Mathematical Methods for the
%   Physical Sciences, 2nd ed., Cambridge University Press.
%       van Manen, D.-J., Robertsson, J.O.A., Curtis, A. (2005) Modeling of
%   Wave Propagation in Inhomogeneous Media. Physical Review Letters, 94:
%   164301.
%       Wapenaar, K., Fokkema, J. (2006) Green's function representations
%   for seismic interferometry, Geophysics, 71: SI33-SI46.
%
%
% Required:
% makecircbound.m           - creates a circular boundary of points with a
%                             certain radius or point spacing
% do_xcorr_int.m            - performs cross-correlational interferometry
% acoustic_p_gfs_direct_f.m - models direct wave between two points in an
%                             acoustic medium in the frequency domain only
% model_gfs_p_scat_acoust.m - models scattered waves in an acoustic medium
% centerfreqs.m             - creates a two-sided centred frequency
%                             spectrum from positive frequencies
% rickerwavl_time.m         - creates Ricker wavelet for source
%
%
% Erica Galetti, July 2012
% Contact: erica.galetti@ed.ac.uk


clear all
close all



%% van Manen or Wapenaar?
method = 'WFvir';    
    % 'VMvi'    --- van Manen et al. (2005)
    %                   volume injection source
    % 'WFvir'   --- Wapenaar & Fokkema (2006)
    %                   volume injection rate source
    % 'WFvi'    --- Wapenaar & Fokkema (2006)
    %                   volume injection source
    % 'VMvir'   --- Equivalent to van Manen et al. (2005) 
    %                   with a volume injection rate source          

    
%% Type of interferometry
inttype = 'full'; % Monopole ('mono') or full ('full')


%% Include scatterers?
scattering = 'yes'; % 'yes' or 'no'

% Define scatterer locations
if strcmp(scattering,'yes')
    scats = [-20,-80,0;250,150,0;220,180,0];
    scats(:,4) = -2; % Imaginary part of the scattering amplitude - has to be within the range [-4,0] (see Groenenboom & Snieder, 1995)
end


%% Normalise amplitudes?
normamp = 'no'; % 'yes' or 'no'


%% Source wavelet type
wavltype = 'wavly_pos'; % the positive-time wavelet is not zero-phase - for an example with zero-phase wavelet, see the source-receiver interferometry example


%% Type of source
if strcmp(method,'WFvir')
    srctype='volinjratepos';
elseif strcmp(method,'WFvi')
    srctype='volinjpos';
elseif strcmp(method,'VMvir')
    srctype='volinjrateneg';
elseif strcmp(method,'VMvi')
    srctype='volinjneg';
end


%% Define a series of input parameters

nf = 200;
nf = 2^(nextpow2(nf)); % round number of frequencies to the nearest power of 2 (makes fft/ifft run faster)
maxf = 100; % max frequency
fc = 30; % central frequency of Ricker wavelet
dt = 1/(2*maxf); % time step

ro = 1; % medium density
c = 1000; % constant background velocity
nyq = c/maxf; % minimum Nyquist wavelength


%% Type of geometry
dim = '2d';
if strcmp(dim,'1d'); ndim = 1;
elseif strcmp(dim,'2d') || strcmp(dim,'2d_far'); ndim = 2;
elseif strcmp(dim,'3d'); ndim = 3;
end


%% Define source and receiver positions

% Define receiver boundary
% Points on circle
rad_r = 400; % radius of receiver boundary (gets adjusted depending on source spacing 4 lines below)
dx_r = nyq/2; % spacing of receivers on boundary is half the minimum Nyquist wavelength
[coords,radnew] = makecircbound(rad_r,dx_r,'fixds',0,0);
rad_r = radnew;
nrcx = size(coords,1);
rcx(1:nrcx,1) = coords(:,1); % boundary x-coordinate
rcx(1:nrcx,2) = coords(:,2); % boundary y-coordinate
rcx(1:nrcx,3) = 0; % boundary z-coordinate
% Calculate distance between receivers (offsets_r) and dr
offsets_r = zeros(nrcx,1);
dr = zeros(nrcx,1);
offsets_r(1:nrcx) = sqrt(diff(rcx([1:nrcx 1],1)).^2 + diff(rcx([1:nrcx 1],2)).^2);
dr(1:nrcx) = offsets_r(1:nrcx)/2 + offsets_r([nrcx 1:nrcx-1])/2;
clear offsets_r coords radnew

% Calculate boundary normal (the boundary is a circle centred at the
% origin, so its x and y components are simply the cos and sin of the
% angles between the receiver-position-vector and the x axis).
boundnorm = zeros(nrcx,ndim);
for dd = 1:ndim; boundnorm(:,dd) = rcx(:,dd)/rad_r; end


% Define position of sources
srA=[-150,0,0];
srB=[50,40,0];
srx = [srA;srB];
nsrx = size(srx,1);



%% Model the real Green's function between A and B
if strcmp(scattering,'yes')
    [time1,gftrue_mon_p_t,~,wavelet_t] = model_gfs_p_scat_acoust(srx,srx,maxf,nf,c,dim,srctype,'recdip','scaty',wavltype,'freqn','timey','tot',scats,fc);
elseif strcmp(scattering,'no')
    [time1,gftrue_mon_p_t,~,wavelet_t] = model_gfs_p_scat_acoust(srx,srx,maxf,nf,c,dim,srctype,'recdip','scatn',wavltype,'freqn','timey','tot',fc);
end

% Define two-sided time vector
time2 = [flipud(-time1(2:end));time1(1:end)]; % negative and positive times

% Wavelet in frequency domain
wavelet_f = fft([wavelet_t;zeros(size(wavelet_t))])./sqrt(2*2*nf);
wavelet_f = wavelet_f(2:2*nf+1,:,:); % take only half of the frequency spectrum

% Frequencies
df = maxf/(2*nf);
freq2 =  (1:2*nf).*df;
freqs=[0;freq2.'];
om = 2*pi.*freq2; % angular frequencies



%% Model the sources to receiver-boundary Green's functions using the model_gfs_p_scat_acoust function

% mgfs_t = monopole source, time domain
% dgfs_t = dipole source, time domain

% Allocate space
if strcmp(inttype,'mono')
    mgfs_t=zeros(2*nf,nrcx,nsrx);
elseif strcmp(inttype,'full')
    mgfs_t=zeros(2*nf,nrcx,nsrx);
    dgfs_t=zeros(2*nf,nrcx,nsrx);
end

% Loop over boundary receivers

for rr = 1:nrcx
    
    if strcmp(scattering,'yes')
        
        if strcmp(inttype,'mono')
            
            % Model pressure from all sources to receiver rr
            [~,gf_mon_p_t,~,~] = model_gfs_p_scat_acoust(srx,rcx(rr,:),maxf,nf,c,dim,srctype,'recdip','scaty',wavltype,'freqn','timey','tot',scats,fc);
            mgfs_t(:,rr,:) = gf_mon_p_t; % monopole wavefield - time domain
            
        elseif strcmp(inttype,'full')
       
            % Model pressure from all sources to receiver rr
            [~,gf_mon_p_t,gf_dip_p_t,~] = model_gfs_p_scat_acoust(srx,rcx(rr,:),maxf,nf,c,dim,srctype,'recdip','scaty',wavltype,'freqn','timey','tot',scats,fc);
            mgfs_t(:,rr,:) = gf_mon_p_t; % monopole wavefield - time domain
            for dd = 1:ndim; dgfs_t(:,rr,:) = dgfs_t(:,rr,:) + gf_dip_p_t(:,1,:,dd).*boundnorm(rr,dd); end % weight dipole components by the outward normal components - this gives the full dipole wavefield 

        end
        
    elseif strcmp(scattering,'no')
        
        if strcmp(inttype,'mono')
                                    
            % Model pressure from all sources to receiver rr
            [~,gf_mon_p_t,~,~] = model_gfs_p_scat_acoust(srx,rcx(rr,:),maxf,nf,c,dim,srctype,'recdip','scatn',wavltype,'freqn','timey','tot',fc);
            mgfs_t(:,rr,:) = gf_mon_p_t; % monopole wavefield - time domain

        elseif strcmp(inttype,'full')
                        
            % Model pressure from all sources to receiver rr
            [~,gf_mon_p_t,gf_dip_p_t,~] = model_gfs_p_scat_acoust(srx,rcx(rr,:),maxf,nf,c,dim,srctype,'recdip','scatn',wavltype,'freqn','timey','tot',fc);
            mgfs_t(:,rr,:) = gf_mon_p_t; % monopole wavefield - time domain
            for dd = 1:ndim; dgfs_t(:,rr,:) = dgfs_t(:,rr,:) + gf_dip_p_t(:,1,:,dd).*boundnorm(rr,dd); end % weight dipole components by the outward normal components - this gives the full dipole wavefield 

        end
    end
    
    if mod(rr,20)==0; fprintf(['Done receiver ' int2str(rr) ' of ' int2str(nrcx) '.\n']); end
    
    clear gf_mon_p_t gf_dip_p_t
    
end

% Apply Tukey window to taper trace edges (useful in case any scattered
% arrivals occur at the end of the trace, as the traces need to be padded
% with zeros before moving to the frequency domain for cross-correlation)
% and pad with zeros to get traces twice as long.

% Construct Tukey window (this assumes the Signal Processing Toolbox is not
% available and avoids calling the tukeywin function, giving similar results)
ntap = 10; % number of time samples to taper on each side of the trace (make sure it is much less than half the total number of time samples!)
theta_t = linspace(0,1,ntap+1).';
taper = (1+cos(pi.*theta_t))./2;
% Create Tukey window as taper-ones-taper
tuk = [taper(end:-1:1);ones(2*nf-2*(ntap+1),1,1);taper];
clear ntap theta_t taper

gftrue_mon_p_t = [gftrue_mon_p_t.*repmat(tuk,[1 size(gftrue_mon_p_t,2) size(gftrue_mon_p_t,3)]);zeros(2*nf,size(gftrue_mon_p_t,2),size(gftrue_mon_p_t,3))];
mgfs_t = [mgfs_t.*repmat(tuk,[1 size(mgfs_t,2) size(mgfs_t,3)]);zeros(2*nf,size(mgfs_t,2),size(mgfs_t,3))];
if strcmp(inttype,'full'); dgfs_t = [dgfs_t.*repmat(tuk,[1 size(dgfs_t,2) size(dgfs_t,3)]);zeros(2*nf,size(dgfs_t,2),size(dgfs_t,3))]; end

% Move to frequency domain
gftrue_mon_p_f = fft(gftrue_mon_p_t,[],1)./sqrt(2*2*nf);
clear gftrue_mon_p_t
gftrue_mon_p_f = gftrue_mon_p_f(2:2*nf+1,:,:); % take only half of the frequency spectrum
mgfs_f = fft(mgfs_t,[],1)./sqrt(2*2*nf);
clear mgfs_t
mgfs_f = mgfs_f(2:2*nf+1,:,:); % take only half of the frequency spectrum
if strcmp(inttype,'full');
    dgfs_f = fft(dgfs_t,[],1)./sqrt(2*2*nf);
    clear dgfs_t
    dgfs_f = dgfs_f(2:2*nf+1,:,:); % take only half of the frequency spectrum
end


%% Perform xcorr interferometry from boundary S' of sources in frequency domain using matrix multiplication one frequency at a time
% Use function do_xcorr_int to do cross-correlational interferometry

if  strcmp(inttype,'full')
    
    % Evaluate GFs with seismic interferometry
    [~,full_int_Ss_f] = do_xcorr_int(freq2,mgfs_f,1:nsrx,1:nsrx,ro,c,dr,'intsrc','active','freq',inttype,method,dgfs_f);
    
elseif strcmp(inttype,'mono')
    
    % Evaluate GFs with seismic interferometry
    [~,mon_int_Ss_f] = do_xcorr_int(freq2,mgfs_f,1:nsrx,1:nsrx,ro,c,dr,'intsrc','active','freq',inttype,method);

end

clear  mgfs_f dgfs_f



%% Select results
if  strcmp(inttype,'full')
    int_Ss_f = [sqrt(2).*full_int_Ss_f(1:2*nf-1,:,:);full_int_Ss_f(2*nf,:,:)]; % We have taken only half of the frequency spectrum, so we have to adjust the amplitudes
elseif strcmp(inttype,'mono')
    int_Ss_f = [sqrt(2).*mon_int_Ss_f(1:2*nf-1,:,:);mon_int_Ss_f(2*nf,:,:)]; % We have taken only half of the frequency spectrum, so we have to adjust the amplitudes
end
clear full_int_Ss_f mon_int_Ss_f
  


%% The true and interferometric GFs are:


% True GF
gftrue_f = gftrue_mon_p_f.*repmat(conj(wavelet_f),[1 size(gftrue_mon_p_f,2) size(gftrue_mon_p_f,3)]); % Convolve spectrum of true trace with conjugate of source wavelet
gftrue_f = [sqrt(2).*gftrue_f(1:2*nf-1,:,:);gftrue_f(2*nf,:,:)]; % We have taken only half of the frequency spectrum, so we have to adjust the amplitudes
gftrue_f_2sided = centerfreqs(gftrue_f,2*nf); % Get two-sided frequency spectrum for ifft
gftrue_t_2sided = fftshift(sqrt(2*2*nf)*ifft(fftshift(gftrue_f_2sided,1),[],1),1); % Move to time domain
gftrue_t_2sided = real([gftrue_t_2sided(end:-1:2*nf+2,:,:);gftrue_t_2sided(2*nf+1:end,:,:)]);
clear gftrue_f_2sided
gftrue_t = gftrue_t_2sided(2*nf:end,:,:);

% True GF as stack of true GF and its complex conjugate - for comparison
% with interferometry results
if strcmp(method,'WFvir'); gftrue_f_comp = gftrue_f + conj(gftrue_f);
elseif strcmp(method,'WFvi'); gftrue_f_comp = gftrue_f - conj(gftrue_f);
elseif strcmp(method,'VMvi'); gftrue_f_comp = conj(gftrue_f) - gftrue_f;
elseif strcmp(method,'VMvir'); gftrue_f_comp = gftrue_f + conj(gftrue_f);
end
gftrue_f_2sided_comp = centerfreqs(gftrue_f_comp,2*nf); % Get two-sided frequency spectrum for ifft
gftrue_t_2sided_comp = fftshift(sqrt(2*2*nf)*ifft(fftshift(gftrue_f_2sided_comp,1),[],1),1); % Move to time domain
gftrue_t_2sided_comp = real(gftrue_t_2sided_comp(2:end,:,:));
clear gftrue_f_2sided_comp

% Interferometric GFs from boundary S' of receivers
gfint_f = int_Ss_f;
gfint_f_2sided = centerfreqs(int_Ss_f,2*nf); % Get two-sided frequency spectrum for ifft
gfint_t_2sided = fftshift(sqrt(2*2*nf)*ifft(fftshift(gfint_f_2sided,1),[],1),1); % Move to time domain
gfint_t_2sided = real(gfint_t_2sided(2:end,:,:));
clear gfint_f_2sided int_Ss_f

% Normalise amplitudes
if strcmp(normamp,'yes')
    gftrue_t_2sided = gftrue_t_2sided./repmat(max(abs(gftrue_t_2sided)),[2*2*nf-1 1 1]);
    gftrue_t = gftrue_t./repmat(max(abs(gftrue_t)),[2*nf 1 1]);
    gftrue_f_comp = gftrue_f_comp./repmat(max(abs(gftrue_f_comp)),[2*nf 1 1]);
    gftrue_t_2sided_comp = gftrue_t_2sided_comp./repmat(max(abs(gftrue_t_2sided_comp)),[2*2*nf-1 1 1]);
    gfint_f = gfint_f./repmat(max(abs(gfint_f)),[2*nf 1 1]);
    gfint_t_2sided = gfint_t_2sided./repmat(max(abs(gfint_t_2sided)),[2*2*nf-1 1 1]);
end



%% Green's functions AB (from A to B)

% True GF
AB_true_t = gftrue_t(:,2,1);
AB_true_t_2sided = gftrue_t_2sided(:,2,1);

% True GF for comparison
AB_true_f_comp = [0;gftrue_f_comp(:,2,1)];
AB_true_t_2sided_comp = gftrue_t_2sided_comp(:,2,1);
clear gftrue*

% Interferometric GFs from boundary S' of receivers
AB_int_f = [0;gfint_f(:,2,1)];
AB_int_t_2sided = gfint_t_2sided(:,2,1);
clear gfint*



%% Plots

scrnsz = get(0,'screensize');
amaxt=max(max([abs(AB_true_t_2sided),abs(AB_int_t_2sided)]));
amaxf=max(max([abs(AB_true_f_comp),abs(AB_int_f)]));

% Plot geometry and true trace
fig1 = figure;
fig1w = 800;
fig1h = 600;
pos = get(fig1,'Position');
outpos = get(fig1,'OuterPosition');
bord = outpos - pos; clear pos outpos
topb = bord(4)-abs(bord(2));
leftb = abs(bord(1));
set(fig1,'Position',[leftb scrnsz(4)-(topb+fig1h) fig1w fig1h])
% Plot geometry
sub1=subplot(211);
hold on
src=plot(srx(:,1),srx(:,2),'r*','markersize',8);
rec=plot(rcx(1:6:end,1),rcx(1:6:end,2),'bv','markersize',8,'markerfacecolor','b');
if strcmp(scattering,'yes')
    scat=plot(scats(:,1),scats(:,2),'go','markersize',8,'markerfacecolor','g');
end
axis equal
axis([min(rcx(:,1)-50) max(rcx(:,1)+50) min(rcx(:,2)-50) max(rcx(:,2)+50)])
box on
set(gca,'fontsize',14)
xlabel ('x (m)','fontsize',14)
ylabel ('y (m)','fontsize',14)
title ('Geometry','fontsize',14)
text(srx(1,1),srx(1,2)-60,'A','fontsize',14,'horizontalalignment','center')
text(srx(2,1),srx(2,2)-60,'B','fontsize',14,'horizontalalignment','center')
if strcmp(scattering,'no')
    legend([src rec],'Source','Receiver','location','bestoutside')
elseif strcmp(scattering,'yes')
    legend([src rec scat],'Source','Receiver','Scatterer','location','bestoutside')
end
% Plot true trace
sub2=subplot(212);
hold on
plot(time1,AB_true_t,'k','linewidth',2)
axis([0 0.6*max(time1) -max(abs(AB_true_t)) max(abs(AB_true_t))])
box on
set(gca,'fontsize',14)
xlabel ('Time (s)','fontsize',14)
if strcmp(normamp,'yes'); ylabel ('Normalised Amplitude','fontsize',14)
elseif strcmp(normamp,'no'); ylabel ('Amplitude','fontsize',14);end
title ('True trace obtained for a source at A and a receiver at B','fontsize',14)

set(sub1,'Position',[0.275 0.45 0.45 0.45])
set(sub2,'Position',[0.1 0.075 0.8 0.2])


% Plot results: true GF and interferometric GF 
% Time domain, acausal and causal parts
fig2=figure;
fig2w=1200;
fig2h=300;
pos = get(fig1,'Position');
set(fig2,'Position',[pos(1)+leftb scrnsz(4)-(2*topb+fig2h) fig2w fig2h])
hold on
line([0 0],[-amaxt amaxt],'Linestyle','--','color','k')
true=plot(time2,AB_true_t_2sided,'k','linewidth',2);
int=plot(time2,AB_int_t_2sided,'r--','linewidth',2);
axis([0.6*min(time2) 0.6*max(time2) -amaxt amaxt])
box on
set(gca,'fontsize',14)
xlabel ('Time (s)','fontsize',14)
if strcmp(normamp,'yes'); ylabel ('Normalised Amplitude','fontsize',14)
elseif strcmp(normamp,'no'); ylabel ('Amplitude','fontsize',14);end
legend([true int],'True trace','Interferometric trace')
if strcmp(inttype,'full')
    title ('True trace vs. interferometric trace (inter-source interferometry with monopoles and dipoles)','fontsize',14)
elseif strcmp(inttype,'mono')
    title ('True trace vs. interferometric trace (inter-source interferometry with monopoles only)','fontsize',14)
end
text(0.6*min(time2)/2,-amaxt/2,'Acausal','fontsize',14,'horizontalalignment','center')
text(0.6*max(time2)/2,-amaxt/2,'Causal','fontsize',14,'horizontalalignment','center')



% Plot results: true GF (with acausal part calculated according to the
% interferometric formulae) and interferometric GF
% Time and frequency domain (absolute, real and imaginary)
fig3=figure;
fig3w=1200;
fig3h=600;
pos = get(fig2,'Position');
set(fig3,'Position',[pos(1)+leftb scrnsz(4)-(3*topb+fig3h) fig3w fig3h])
% Time domain
subplot(211)
hold on
true=plot(time2,AB_true_t_2sided_comp,'k','linewidth',2);
int=plot(time2,AB_int_t_2sided,'r--','linewidth',2);
axis([0.6*min(time2) 0.6*max(time2) -amaxt amaxt])
box on
set(gca,'fontsize',14)
xlabel ('Time (s)','fontsize',14)
if strcmp(normamp,'yes'); ylabel ('Normalised Amplitude','fontsize',14)
elseif strcmp(normamp,'no'); ylabel ('Amplitude','fontsize',14);end
legend([true int],'''True'' trace','Interferometric trace')
title ('Time domain','fontsize',14)
% Frequency domain - spectrum
subplot(234)
hold on
plot(freqs,abs(AB_true_f_comp),'k','linewidth',2);
plot(freqs,abs(AB_int_f),'r--','linewidth',2);
axis([0 max(freqs) 0 amaxf])
box on
set(gca,'fontsize',14)
xlabel ('Frequency (Hz)','fontsize',14)
if strcmp(normamp,'yes'); ylabel ('Normalised Amplitude','fontsize',14)
elseif strcmp(normamp,'no'); ylabel ('Amplitude','fontsize',14);end
title ('Frequency domain - absolute','fontsize',14)
% Frequency domain - real
subplot(235)
hold on
plot(freqs,real(AB_true_f_comp),'k','linewidth',2);
plot(freqs,real(AB_int_f),'r--','linewidth',2);
axis([0 max(freqs) -amaxf amaxf])
box on
set(gca,'fontsize',14)
xlabel ('Frequency (Hz)','fontsize',14)
if strcmp(normamp,'yes'); ylabel ('Normalised Amplitude','fontsize',14)
elseif strcmp(normamp,'no'); ylabel ('Amplitude','fontsize',14);end
title ('Frequency domain - real','fontsize',14)
% Frequency domain - imaginary
subplot(236)
hold on
plot(freqs,imag(AB_true_f_comp),'k','linewidth',2);
plot(freqs,imag(AB_int_f),'r--','linewidth',2);
axis([0 max(freqs) -amaxf amaxf])
box on
set(gca,'fontsize',14)
xlabel ('Frequency (Hz)','fontsize',14)
if strcmp(normamp,'yes'); ylabel ('Normalised Amplitude','fontsize',14)
elseif strcmp(normamp,'no'); ylabel ('Amplitude','fontsize',14);end
title ('Frequency domain - imaginary','fontsize',14)
