%% Acoustic modelling and source-receiver interferometry example - 2D

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% This code is distributed as part of the source-code package 
%                   acoustic_modelling_foldy_v.4.1_seg 
% that accompanies Galetti et al. (2013). The package can be downloaded 
% from the Geophysics source-code archive at 
%                   http://software.seg.org/2013/0003
% Use of this code is subject to acceptance of the terms and conditions
% that can be found at http://software.seg.org/disclaimer.txt 
% Copyright 2013 by The Society of Exploration Geophysicists (SEG)
% Reference:
% Galetti, E., Halliday, D., Curtis, A. (2013) A simple and exact acoustic 
% wavefield modelling code for data processing, imaging and interferometry 
% applications. Geophysics.
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% This script serves as an example of how the acoustic GF modelling code
% can be used to model acoustic wavefields from a boundary of sources to an
% enclosed receiver, and from an enclosed source to a boundary of
% receivers, and how to apply interferometry to construct the GF signal
% between the enclosed source and receiver. A circular boundary of sources
% and a circular boundary of receivers are used for interferometry.
% Cross-correlation is performed in the frequency domain (using matrix
% multiplication one frequency at a time). The results of interferometry
% match the true GF obtained by direct modelling of the GF between the
% source and the receiver.
%
%
% WARNING: BECAUSE SOURCE-RECEIVER INTERFEROMETRY USES TWO BOUNDARIES, A
% LOT OF WAVEFORMS NEED TO BE MODELLED - MAKE SURE THE COMPUTER YOU ARE
% USING TO RUN THIS EXAMPLE HAS ENOUGH MEMORY TO STORE ALL THE VARIABLES
% WITHOUT CRASHING! A MEMORY WARNING MESSAGE IS ISSUED IN CASE THE
% NECESSARY MEMORY EXCEEDS A VALUE SET BY THE USER AT LINE 90.
%
%
% Figure 1: Geometry (top)
%           True modelled trace (bottom)
% Figure 2: Two-sided Green's functions in time domain:
%               black solid trace - true Green's function
%               red dashed trace - interferometric Green's function
% Figure 3: Comparison: 'true' Green's function (black solid trace -
%                       obtained from the causal part of the true Green's
%                       function in Figure 2 according to the
%                       interferometry formulae) vs. interferometric
%                       Green's function (red dashed trace)
%                       top:            traces in time domain
%                       bottom left:    frequency spectra
%                       bottom centre:  real part of frequency spectra
%                       bottom right:   imaginary part of frequency spectra
%
%
% References:
%       Curtis, A., Halliday, D. (2010) Source-receiver wave ﬁeld
%   interferometry. Physical Review E 81 (4): 046601.
%       Curtis, A., Nicolson, H., Halliday, D., Trampert, J., Baptie, B.
%   (2009) Virtual seismometers in the subsurface of the earth from
%   seismic interferometry. Nature Geosci 2 (10): 700–704.
%       Groenenboom, J., Snieder, R. (1995) Attenuation, dispersion, and
%   anisotropy by multiple scattering of transmitted waves through
%   distributions of scatterers. J. Acoust. Soc. Am. 98 (6): 3482-3492.
%       Snieder, R. (2009) A Guided Tour of Mathematical Methods for the
%   Physical Sciences, 2nd ed., Cambridge University Press.
%       van Manen, D.-J., Robertsson, J.O.A., Curtis, A. (2005) Modeling of
%   Wave Propagation in Inhomogeneous Media. Physical Review Letters, 94:
%   164301.
%       Wapenaar, K., Fokkema, J. (2006) Green's function representations
%   for seismic interferometry, Geophysics, 71: SI33-SI46.
%
%
% Required:
% makecircbound.m           - creates a circular boundary of points with a
%                             certain radius or point spacing
% do_xcorr_int.m            - performs cross-correlational interferometry
% acoustic_p_gfs_direct_f.m - models direct wave between two points in an
%                             acoustic medium in the frequency domain only
% model_gfs_p_scat_acoust.m - models scattered waves in an acoustic medium
% centerfreqs.m             - creates a two-sided centred frequency
%                             spectrum from positive frequencies
% rickerwavl_time.m         - creates Ricker wavelet for source
%
%
% Erica Galetti, July 2012
% Contact: erica.galetti@ed.ac.uk


clear all
close all

maxmem = 3; % define max memory available (in Gb) - the code computes a rough estimate of required memory and issues a warning message if this limit is exceeded


%% van Manen or Wapenaar?
method = 'WFvi';
% 'VMvi'    --- van Manen et al. (2005)
%                   volume injection source
% 'WFvir'   --- Wapenaar & Fokkema (2006)
%                   volume injection rate source
% 'WFvi'    --- Wapenaar & Fokkema (2006)
%                   volume injection source
% 'VMvir'   --- Equivalent to van Manen et al. (2005)
%                   with a volume injection rate source


%% Type of interferometry
inttype = 'mono'; % Monopole ('mono') or full ('full')


%% Include scatterers?
scattering = 'yes'; % 'yes' or 'no'

% Define scatterer locations
if strcmp(scattering,'yes')
    scats = [-50,-150,0];
    scats(:,4) = -4; % Imaginary part of the scattering amplitude - has to be within the range [-4,0] (see Groenenboom & Snieder, 1995)
end


%% Normalise amplitudes?
normamp = 'no'; % 'yes' or 'no'


%% Source wavelet type
wavltype = 'wavly_centr'; % use the centred-on-zero-time source wavelet because it's zero-phase


%% Type of source
if strcmp(method,'WFvir')
    srctype='volinjratepos';
elseif strcmp(method,'WFvi')
    srctype='volinjpos';
elseif strcmp(method,'VMvir')
    srctype='volinjrateneg';
elseif strcmp(method,'VMvi')
    srctype='volinjneg';
end


%% Type of geometry
dim = '2d';
if strcmp(dim,'1d'); ndim = 1;
elseif strcmp(dim,'2d') || strcmp(dim,'2d_far'); ndim = 2;
elseif strcmp(dim,'3d'); ndim = 3;
end


%% Define a series of input parameters

nf = 200;
nf = 2^(nextpow2(nf)); % round number of frequencies to the nearest power of 2 (makes fft/ifft run faster)
maxf = 100; % max frequency
fc = 15; % central frequency of Ricker wavelet
dt = 1/(2*maxf); % time step

ro = 1; % medium density
c = 1000; % constant background velocity


% Model the source wavelet using the Foldy code
[freqw,~,~,wavelet_f] = model_gfs_p_scat_acoust([0,0,0],[0,0,0],maxf,nf,c,dim,srctype,'srcdip','scatn',wavltype,'freqy_1nf','timen','tot',fc);

% Take the maximum modelled frequency to be the first frequency (>fc) that has amplitude less than 1/400 of the peak amplitude at fc
% (this limits the number of sources on the boundary and hence the amount of memory required)
ind = find(freqw>=fc,1,'first');
maxfmod=freqw(ind-1+find(abs(wavelet_f(ind:end))<(0.0025*max(abs(wavelet_f))),1,'first')); 
nyq = c/maxfmod; % minimum Nyquist wavelength
clear ind freqw wavelet_f

% If memory is not an issue, uncomment the following line (reduces the spacing between boundary sources/receivers)
% nyq = c/maxf;



%% Define source and receiver positions

% Define source boundary
% Points on circle
rad_s = 600; % radius of source boundary (gets adjusted depending on source spacing 4 lines below)
dx_s = nyq/2; % spacing of sources on boundary is half the minimum Nyquist wavelength
[coords,radnew] = makecircbound(rad_s,dx_s,'fixds',0,0);
rad_s = radnew;
nsrx = size(coords,1);
srx(1:nsrx,1) = coords(:,1); % boundary x-coordinate
srx(1:nsrx,2) = coords(:,2); % boundary y-coordinate
srx(1:nsrx,3) = 0; % boundary z-coordinate
% Calculate distance between sources (offsets_s) and ds
offsets_s = zeros(nsrx,1);
ds = zeros(nsrx,1);
offsets_s(1:nsrx) = sqrt(diff(srx([1:nsrx 1],1)).^2 + diff(srx([1:nsrx 1],2)).^2);
ds(1:nsrx) = offsets_s(1:nsrx)/2 + offsets_s([nsrx 1:nsrx-1])/2;
clear offsets_s coords radnew

% Calculate boundary normal (the boundary is a circle centred at the
% origin, so its x and y components are simply the cos and sin of the
% angles between the source-position-vector and the x axis).
boundnorm_s = zeros(nsrx,ndim);
for dd = 1:ndim; boundnorm_s(:,dd) = srx(:,dd)/rad_s; end


% Define receiver boundary - MAIN
% Points on circle
rad_r = 350; % radius of receiver boundary (gets adjusted depending on source spacing 4 lines below)
dx_r = nyq/2; % spacing of receivers on boundary is half the minimum Nyquist wavelength
[coords,radnew] = makecircbound(rad_r,dx_r,'fixds',0,0);
rad_r = radnew;
nrcx = size(coords,1);
rcx(1:nrcx,1) = coords(:,1); % boundary x-coordinate
rcx(1:nrcx,2) = coords(:,2); % boundary y-coordinate
rcx(1:nrcx,3) = 0; % boundary z-coordinate
% Calculate distance between receivers (offsets_r) and dr
offsets_r = zeros(nrcx,1);
dr = zeros(nrcx,1);
offsets_r(1:nrcx) = sqrt(diff(rcx([1:nrcx 1],1)).^2 + diff(rcx([1:nrcx 1],2)).^2);
dr(1:nrcx) = offsets_r(1:nrcx)/2 + offsets_r([nrcx 1:nrcx-1])/2;
clear offsets_r coords radnew

% Calculate boundary normal (the boundary is a circle centred at the
% origin, so its x and y components are simply the cos and sin of the
% angles between the source-position-vector and the x axis).
boundnorm_r = zeros(nrcx,ndim);
for dd = 1:ndim; boundnorm_r(:,dd) = rcx(:,dd)/rad_r; end


% Define secondary receiver boundaries which are used to calculate the
% spatial derivative Green's functions at the main receiver boundary.
if strcmp(inttype,'full')
    drr = nyq/20;
    rcx_out = zeros(size(rcx));
    rcx_in = zeros(size(rcx));
    rcx_2out = zeros(size(rcx));
    rcx_2in = zeros(size(rcx));
    for dd = 1:ndim;
        rcx_out(:,dd) = rcx(:,dd)+drr.*boundnorm_r(:,dd);
        rcx_in(:,dd) = rcx(:,dd)-drr.*boundnorm_r(:,dd);
        rcx_2out(:,dd) = rcx(:,dd)+2*drr.*boundnorm_r(:,dd);
        rcx_2in(:,dd) = rcx(:,dd)-2*drr.*boundnorm_r(:,dd);
    end
end


% Define position of central source and receiver
srA=[-100,-20,0];
rcB=[100,20,0];
srx_int = srA;
nsrx_int = size(srx_int,1);
rcx_int = rcB;
nrcx_int = size(rcx_int,1);

% All sources
srxtot = [srx_int;srx];
nsrxtot = size(srxtot,1);

% All receivers
rcxtot = [rcx_int;rcx];
nrcxtot = size(rcxtot,1);
if strcmp(inttype,'full')
    rcxouttot = [rcx_int;rcx_out];
    rcxintot = [rcx_int;rcx_in];
    rcx2outtot = [rcx_int;rcx_2out];
    rcx2intot = [rcx_int;rcx_2in];
end



%% Model the real Green's function between A and B and get wavelet, time and frequency information

fprintf('Modelling true Green''s function between A and B...\n');

if strcmp(scattering,'yes')
    [time1,gftrue_mon_p_t,~,wavelet_t] = model_gfs_p_scat_acoust(srx_int,rcx_int,maxf,nf,c,dim,srctype,'srcdip','scaty',wavltype,'freqn','timey','tot',scats,fc);
elseif strcmp(scattering,'no')
    [time1,gftrue_mon_p_t,~,wavelet_t] = model_gfs_p_scat_acoust(srx_int,rcx_int,maxf,nf,c,dim,srctype,'srcdip','scatn',wavltype,'freqn','timey','tot',fc);
end

% Apply Tukey window to taper trace edges (useful in case any scattered
% arrivals occur at the end of the trace, as the traces need to be padded
% with zeros before moving to the frequency domain - see below)
% and pad with zeros to get traces twice as long.

% Construct Tukey window (this assumes the Signal Processing Toolbox is not
% available and avoids calling the tukeywin function, giving similar results)
ntap = 4; % number of time samples to taper on each side of the trace (make sure it is much less than half the total number of time samples!)
theta_t = linspace(0,1,ntap+1).';
taper = (1+cos(pi.*theta_t))./2;
% Create Tukey window as taper-ones-taper
tuk = [taper(end:-1:1);ones(2*nf-2*(ntap+1),1,1);taper];
clear ntap theta_t taper
    
gftrue_mon_p_t = [gftrue_mon_p_t.*repmat(tuk,[1 size(gftrue_mon_p_t,2) size(gftrue_mon_p_t,3)]);zeros(2*nf,size(gftrue_mon_p_t,2),size(gftrue_mon_p_t,3))];

% Move to frequency domain
gftrue_mon_p_f = fft(gftrue_mon_p_t,[],1)./sqrt(2*2*nf);
clear gftrue_mon_p_t
gftrue_mon_p_f = gftrue_mon_p_f(2:2*nf+1,:,:); % take only half of the frequency spectrum

% Define two-sided time vector
time2 = [flipud(-time1(2:end));time1(1:end)]; % negative and positive times

% Wavelet in frequency domain
wavelet_f = fft([wavelet_t(1:nf);zeros(size(wavelet_t));wavelet_t(nf+1:end)])./sqrt(2*2*nf);
wavelet_f = wavelet_f(2:2*nf+1,:,:); % take only half of the frequency spectrum

% Frequencies
df = maxf/(2*nf);
freq2 =  (1:2*nf).*df;
freqs=[0;freq2.'];
om = 2*pi.*freq2; % angular frequencies



%% Memory warning

% In this section, an estimate of the memory used so far and the memory that
% will be used to model the waveforms and perform interferometry is made.
% If the result exceeds the maximum memory limit set by the user at line
% 75, a warning message is issued.

% Find memory used so far
vars = whos;
mm1 = 0; for vv=1:length(vars); mm1 = mm1 + vars(vv,1).bytes; end
clear vars
mm1 = mm1/1e6; % convert to Mb

% Find memory used to store all source-to-receiver modelled wavefields
% Each variable in an array takes 8 bytes if real, 16 bytes if complex
mm2 = 8*(2*2*nf)*nrcxtot*nsrx;         % monopole, boundary sources to all receivers, time domain
mm2 = mm2/1e6; % convert to Mb
if strcmp(inttype,'full')
    mm3 = 9*8*(2*2*nf)*nrcxtot*nsrx;   % dipole, boundary sources to all receivers, time domain
    mm3 = mm3/1e6; % convert to Mb
end
mm4 = 8*(2*2*nf)*nrcxtot*nsrx;         % monopole or dipole, boundary sources to all receivers, frequency domain (this corresponds to the time to frequency domain conversion step)
mm4 = mm4/1e6; % convert to Mb
if strcmp(inttype,'full')              % inter-receiver interferometry results - this is just the space allocated to the results, but more memory is necessary to perform interferometry
    mm5 = 2*8*(2*2*nf)*nrcx*nrcx_int;
    mm5 = mm5/1e6; % convert to Mb
elseif strcmp(inttype,'mono')
    mm5 = 8*(2*2*nf)*nrcx*nrcx_int;
    mm5 = mm5/1e6; % convert to Mb
end
    
if strcmp(inttype,'mono')
    memreq = mm1 + mm2 + mm4 + mm5;
    clear mm1 mm2 mm4 mm5
elseif strcmp(inttype,'full')
    memreq = mm1 + mm2 + mm3 + mm4 + mm5;
    clear mm1 mm2 mm3 mm4 mm5
end
memreq = memreq/1e3; % convert to Gb

if memreq > maxmem
    choice = questdlg(['With the current geometry, modelling all wavefields and performing interferometry requires more than ' num2str(memreq,'%.2f') ' Gb of memory. Are you sure you want to continue?'],'MEMORY USAGE WARNING','Yes','No','No');
    switch choice
        case 'No'
            error('memory:exceeded','\nNecessary memory exceeds limit set by user - process terminated.\n\nYou can reduce the amount of memory used by decreasing the size of the boundaries, decreasing the spacing between boundary sources/receivers, decreasing the maximum frequency modelled, or choosing monopole interferometry rather than full inteferometry.');
        case 'Yes'
            disp('Memory warning issued - user selected to continue.');
    end
end


%% Model Green's functions from boundary sources to all receivers using the model_gfs_p_scat_acoust function

% Apply Tukey window to taper trace edges (useful in case any scattered
% arrivals occur at the end of the trace, as the traces need to be padded
% with zeros before moving to the frequency domain for cross-correlation)
% and pad with zeros to get traces twice as long.

% mgfs_t = monopole source, time domain
% dgfs_t = dipole source, time domain - uses memory mm3

fprintf('Modelling Green''s functions from boundary sources to all receivers...\n');

% Allocate space
mgfs_t=zeros(2*2*nf,nrcxtot,nsrx);          % uses memory mm2
if strcmp(inttype,'full')                   % all variables together use memory mm3
    mgfs_out_t=zeros(2*2*nf,nrcxtot,nsrx);
    mgfs_in_t=zeros(2*2*nf,nrcxtot,nsrx);
    mgfs_2out_t=zeros(2*2*nf,nrcxtot,nsrx);
    mgfs_2in_t=zeros(2*2*nf,nrcxtot,nsrx);
    dgfs_t=zeros(2*2*nf,nrcxtot,nsrx);
    dgfs_out_t=zeros(2*2*nf,nrcxtot,nsrx);
    dgfs_in_t=zeros(2*2*nf,nrcxtot,nsrx);
    dgfs_2out_t=zeros(2*2*nf,nrcxtot,nsrx);
    dgfs_2in_t=zeros(2*2*nf,nrcxtot,nsrx);
end

% Loop over boundary sources and all receivers - slow, but reduces the
% amount of memory used at a time

% To avoid looping over receivers (speeds up the code but uses more
% memory), uncomment the following line and comment the receiver-loop 'for'
% and 'end' lines.
rr=':';

for ss = 1:nsrx % this loop is over the boundary sources
    %for rr = 1:nrcxtot % this loop is over all receivers (central + boundary)
        if strcmp(scattering,'yes')
            % Model pressure from source ss to all receivers
            % Monopole to main receiver boundary
            [~,gf_mon_p_t,~,~] = model_gfs_p_scat_acoust(srx(ss,:),rcxtot(rr,:),maxf,nf,c,dim,srctype,'srcdip','scaty',wavltype,'freqn','timey','tot',scats,fc);
            mgfs_t(1:2*nf,rr,ss) = gf_mon_p_t.*repmat(tuk,[1 size(gf_mon_p_t,2) size(gf_mon_p_t,3)]); % monopole wavefield - frequency domain
            if strcmp(inttype,'full')
                % Model pressure from source ss to all receivers
                % Dipole to main receiver boundary
                [~,~,gf_dip_p_t,~] = model_gfs_p_scat_acoust(srx(ss,:),rcxtot(rr,:),maxf,nf,c,dim,srctype,'srcdip','scaty',wavltype,'freqn','timey','tot',scats,fc);
                for dd = 1:ndim; dgfs_t(1:2*nf,rr,ss) = dgfs_t(1:2*nf,rr,ss) + gf_dip_p_t(:,:,1,dd).*repmat(tuk,[1 size(gf_dip_p_t,2) size(gf_dip_p_t,3)]).*boundnorm_s(ss,dd); end % weight dipole components by the outward normal components - this gives the full dipole wavefield
                % Monopole and dipole to secondary receiver boundaries
                [~,gf_out_mon_p_t,gf_out_dip_p_t,~] = model_gfs_p_scat_acoust(srx(ss,:),rcxouttot(rr,:),maxf,nf,c,dim,srctype,'srcdip','scaty',wavltype,'freqn','timey','tot',scats,fc);
                mgfs_out_t(1:2*nf,rr,ss) = gf_out_mon_p_t.*repmat(tuk,[1 size(gf_out_mon_p_t,2) size(gf_out_mon_p_t,3)]); % monopole wavefield - frequency domain
                for dd = 1:ndim; dgfs_out_t(1:2*nf,rr,ss) = dgfs_out_t(1:2*nf,rr,ss) + gf_out_dip_p_t(:,:,1,dd).*repmat(tuk,[1 size(gf_out_dip_p_t,2) size(gf_out_dip_p_t,3)]).*boundnorm_s(ss,dd); end % weight dipole components by the outward normal components - this gives the full dipole wavefield
                [~,gf_in_mon_p_t,gf_in_dip_p_t,~] = model_gfs_p_scat_acoust(srx(ss,:),rcxintot(rr,:),maxf,nf,c,dim,srctype,'srcdip','scaty',wavltype,'freqn','timey','tot',scats,fc);
                mgfs_in_t(1:2*nf,rr,ss) = gf_in_mon_p_t.*repmat(tuk,[1 size(gf_in_mon_p_t,2) size(gf_in_mon_p_t,3)]); % monopole wavefield - frequency domain
                for dd = 1:ndim; dgfs_in_t(1:2*nf,rr,ss) = dgfs_in_t(1:2*nf,rr,ss) + gf_in_dip_p_t(:,:,1,dd).*repmat(tuk,[1 size(gf_in_dip_p_t,2) size(gf_in_dip_p_t,3)]).*boundnorm_s(ss,dd); end % weight dipole components by the outward normal components - this gives the full dipole wavefield
                [~,gf_2out_mon_p_t,gf_2out_dip_p_t,~] = model_gfs_p_scat_acoust(srx(ss,:),rcx2outtot(rr,:),maxf,nf,c,dim,srctype,'srcdip','scaty',wavltype,'freqn','timey','tot',scats,fc);
                mgfs_2out_t(1:2*nf,rr,ss) = gf_2out_mon_p_t.*repmat(tuk,[1 size(gf_2out_mon_p_t,2) size(gf_2out_mon_p_t,3)]); % monopole wavefield - frequency domain
                for dd = 1:ndim; dgfs_2out_t(1:2*nf,rr,ss) = dgfs_2out_t(1:2*nf,rr,ss) + gf_2out_dip_p_t(:,:,1,dd).*repmat(tuk,[1 size(gf_2out_dip_p_t,2) size(gf_2out_dip_p_t,3)]).*boundnorm_s(ss,dd); end % weight dipole components by the outward normal components - this gives the full dipole wavefield
                [~,gf_2in_mon_p_t,gf_2in_dip_p_t,~] = model_gfs_p_scat_acoust(srx(ss,:),rcx2intot(rr,:),maxf,nf,c,dim,srctype,'srcdip','scaty',wavltype,'freqn','timey','tot',scats,fc);
                mgfs_2in_t(1:2*nf,rr,ss) = gf_2in_mon_p_t.*repmat(tuk,[1 size(gf_2in_mon_p_t,2) size(gf_2in_mon_p_t,3)]); % monopole wavefield - frequency domain
                for dd = 1:ndim; dgfs_2in_t(1:2*nf,rr,ss) = dgfs_2in_t(1:2*nf,rr,ss) + gf_2in_dip_p_t(:,:,1,dd).*repmat(tuk,[1 size(gf_2in_dip_p_t,2) size(gf_2in_dip_p_t,3)]).*boundnorm_s(ss,dd); end % weight dipole components by the outward normal components - this gives the full dipole wavefield
            end
        elseif strcmp(scattering,'no')
            % Model pressure from source ss to all receivers
            % Monopole to main receiver boundary
            [~,gf_mon_p_t,~,~] = model_gfs_p_scat_acoust(srx(ss,:),rcxtot(rr,:),maxf,nf,c,dim,srctype,'srcdip','scatn',wavltype,'freqn','timey','tot',fc);
            mgfs_t(1:2*nf,rr,ss) = gf_mon_p_t.*repmat(tuk,[1 size(gf_mon_p_t,2) size(gf_mon_p_t,3)]); % monopole wavefield - frequency domain
            if strcmp(inttype,'full')
                % Model pressure from source ss to all receivers
                % Dipole to main receiver boundary
                [~,~,gf_dip_p_t,~] = model_gfs_p_scat_acoust(srx(ss,:),rcxtot(rr,:),maxf,nf,c,dim,srctype,'srcdip','scatn',wavltype,'freqn','timey','tot',fc);
                for dd = 1:ndim; dgfs_t(1:2*nf,rr,ss) = dgfs_t(1:2*nf,rr,ss) + gf_dip_p_t(:,:,1,dd).*repmat(tuk,[1 size(gf_dip_p_t,2) size(gf_dip_p_t,3)]).*boundnorm_s(ss,dd); end % weight dipole components by the outward normal components - this gives the full dipole wavefield
                % Monopole and dipole to secondary receiver boundaries
                [~,gf_out_mon_p_t,gf_out_dip_p_t,~] = model_gfs_p_scat_acoust(srx(ss,:),rcxouttot(rr,:),maxf,nf,c,dim,srctype,'srcdip','scatn',wavltype,'freqn','timey','tot',fc);
                mgfs_out_t(1:2*nf,rr,ss) = gf_out_mon_p_t.*repmat(tuk,[1 size(gf_out_mon_p_t,2) size(gf_out_mon_p_t,3)]); % monopole wavefield - frequency domain
                for dd = 1:ndim; dgfs_out_t(1:2*nf,rr,ss) = dgfs_out_t(1:2*nf,rr,ss) + gf_out_dip_p_t(:,:,1,dd).*repmat(tuk,[1 size(gf_out_dip_p_t,2) size(gf_out_dip_p_t,3)]).*boundnorm_s(ss,dd); end % weight dipole components by the outward normal components - this gives the full dipole wavefield
                [~,gf_in_mon_p_t,gf_in_dip_p_t,~] = model_gfs_p_scat_acoust(srx(ss,:),rcxintot(rr,:),maxf,nf,c,dim,srctype,'srcdip','scatn',wavltype,'freqn','timey','tot',fc);
                mgfs_in_t(1:2*nf,rr,ss) = gf_in_mon_p_t.*repmat(tuk,[1 size(gf_in_mon_p_t,2) size(gf_in_mon_p_t,3)]); % monopole wavefield - frequency domain
                for dd = 1:ndim; dgfs_in_t(1:2*nf,rr,ss) = dgfs_in_t(1:2*nf,rr,ss) + gf_in_dip_p_t(:,:,1,dd).*repmat(tuk,[1 size(gf_in_dip_p_t,2) size(gf_in_dip_p_t,3)]).*boundnorm_s(ss,dd); end % weight dipole components by the outward normal components - this gives the full dipole wavefield
                [~,gf_2out_mon_p_t,gf_2out_dip_p_t,~] = model_gfs_p_scat_acoust(srx(ss,:),rcx2outtot(rr,:),maxf,nf,c,dim,srctype,'srcdip','scatn',wavltype,'freqn','timey','tot',fc);
                mgfs_2out_t(1:2*nf,rr,ss) = gf_2out_mon_p_t.*repmat(tuk,[1 size(gf_2out_mon_p_t,2) size(gf_2out_mon_p_t,3)]); % monopole wavefield - frequency domain
                for dd = 1:ndim; dgfs_2out_t(1:2*nf,rr,ss) = dgfs_2out_t(1:2*nf,rr,ss) + gf_2out_dip_p_t(:,:,1,dd).*repmat(tuk,[1 size(gf_2out_dip_p_t,2) size(gf_2out_dip_p_t,3)]).*boundnorm_s(ss,dd); end % weight dipole components by the outward normal components - this gives the full dipole wavefield
                [~,gf_2in_mon_p_t,gf_2in_dip_p_t,~] = model_gfs_p_scat_acoust(srx(ss,:),rcx2intot(rr,:),maxf,nf,c,dim,srctype,'srcdip','scatn',wavltype,'freqn','timey','tot',fc);
                mgfs_2in_t(1:2*nf,rr,ss) = gf_2in_mon_p_t.*repmat(tuk,[1 size(gf_2in_mon_p_t,2) size(gf_2in_mon_p_t,3)]); % monopole wavefield - frequency domain
                for dd = 1:ndim; dgfs_2in_t(1:2*nf,rr,ss) = dgfs_2in_t(1:2*nf,rr,ss) + gf_2in_dip_p_t(:,:,1,dd).*repmat(tuk,[1 size(gf_2in_dip_p_t,2) size(gf_2in_dip_p_t,3)]).*boundnorm_s(ss,dd); end % weight dipole components by the outward normal components - this gives the full dipole wavefield
            end
        end
        
        if mod(ss,20)==0; fprintf(['Done source ' int2str(ss) ' of ' int2str(nsrx) '.\n']); end
        
        clear gf_mon_p_t gf_dip_p_t gf_out_mon_p_t gf_out_dip_p_t gf_in_mon_p_t gf_in_dip_p_t gf_2out_mon_p_t gf_2out_dip_p_t gf_2in_mon_p_t gf_2in_dip_p_t
    %end
end


% Move to frequency domain - for each conversion, memory mm4 is used to
% temporarily move the variables to frequency domain
mgfs_f = fft(mgfs_t,[],1)./sqrt(2*2*nf);
clear mgfs_t
mgfs_f = mgfs_f(2:2*nf+1,:,:); % take only half of the frequency spectrum
if strcmp(inttype,'full');
    dgfs_f = fft(dgfs_t,[],1)./sqrt(2*2*nf);
    clear dgfs_t
    dgfs_f = dgfs_f(2:2*nf+1,:,:); % take only half of the frequency spectrum
    mgfs_out_f = fft(mgfs_out_t,[],1)./sqrt(2*2*nf);
    clear mgfs_out_t
    mgfs_out_f = mgfs_out_f(2:2*nf+1,:,:); % take only half of the frequency spectrum
    mgfs_in_f = fft(mgfs_in_t,[],1)./sqrt(2*2*nf);
    clear mgfs_in_t
    mgfs_in_f = mgfs_in_f(2:2*nf+1,:,:); % take only half of the frequency spectrum
    mgfs_2out_f = fft(mgfs_2out_t,[],1)./sqrt(2*2*nf);
    clear mgfs_2out_t
    mgfs_2out_f = mgfs_2out_f(2:2*nf+1,:,:); % take only half of the frequency spectrum
    mgfs_2in_f = fft(mgfs_2in_t,[],1)./sqrt(2*2*nf);
    clear mgfs_2in_t
    mgfs_2in_f = mgfs_2in_f(2:2*nf+1,:,:); % take only half of the frequency spectrum
    dgfs_out_f = fft(dgfs_out_t,[],1)./sqrt(2*2*nf);
    clear dgfs_out_t
    dgfs_out_f = dgfs_out_f(2:2*nf+1,:,:); % take only half of the frequency spectrum
    dgfs_in_f = fft(dgfs_in_t,[],1)./sqrt(2*2*nf);
    clear dgfs_in_t
    dgfs_in_f = dgfs_in_f(2:2*nf+1,:,:); % take only half of the frequency spectrum
    dgfs_2out_f = fft(dgfs_2out_t,[],1)./sqrt(2*2*nf);
    clear dgfs_2out_t
    dgfs_2out_f = dgfs_2out_f(2:2*nf+1,:,:); % take only half of the frequency spectrum
    dgfs_2in_f = fft(dgfs_2in_t,[],1)./sqrt(2*2*nf);
    clear dgfs_2in_t
    dgfs_2in_f = dgfs_2in_f(2:2*nf+1,:,:); % take only half of the frequency spectrum
end



%% Perform inter-receiver interferometry to get GFs between central receiver and boundary of receivers
% i.e. turn central receiver into a virtual source
% Results use memory mm5.

if  strcmp(inttype,'full')
    
    % Evaluate GFs with seismic interferometry
    [~,full_int_Ss_mon_f] = do_xcorr_int(freq2,mgfs_f,1:nrcx_int,nrcx_int+1:nrcxtot,ro,c,ds,'intrec','active','freq',inttype,method,dgfs_f);
    [~,full_int_Ss_out_f] = do_xcorr_int(freq2,mgfs_out_f,1:nrcx_int,nrcx_int+1:nrcxtot,ro,c,ds,'intrec','active','freq',inttype,method,dgfs_out_f);
    [~,full_int_Ss_in_f] = do_xcorr_int(freq2,mgfs_in_f,1:nrcx_int,nrcx_int+1:nrcxtot,ro,c,ds,'intrec','active','freq',inttype,method,dgfs_in_f);
    [~,full_int_Ss_2out_f] = do_xcorr_int(freq2,mgfs_2out_f,1:nrcx_int,nrcx_int+1:nrcxtot,ro,c,ds,'intrec','active','freq',inttype,method,dgfs_2out_f);
    [~,full_int_Ss_2in_f] = do_xcorr_int(freq2,mgfs_2in_f,1:nrcx_int,nrcx_int+1:nrcxtot,ro,c,ds,'intrec','active','freq',inttype,method,dgfs_2in_f);
    
    % Calculate the derivative Green's function using Richardson's
    % extrapolation.
    full_int_Ss_dip1_f = (full_int_Ss_out_f-full_int_Ss_in_f)./(2*drr);
    full_int_Ss_dip2_f = (full_int_Ss_2out_f-full_int_Ss_2in_f)./(4*drr);
    clear full_int_Ss_out_f full_int_Ss_in_f full_int_Ss_2out_f full_int_Ss_2in_f
    full_int_Ss_dip_f = 1/3.*(4.*full_int_Ss_dip1_f-full_int_Ss_dip2_f);
    clear full_int_Ss_dip1_f full_int_Ss_dip2_f
    
elseif strcmp(inttype,'mono')
    
    % Evaluate GFs with seismic interferometry
    [~,mon_int_Ss_mon_f] = do_xcorr_int(freq2,mgfs_f,1:nrcx_int,nrcx_int+1:nrcxtot,ro,c,ds,'intrec','active','freq',inttype,method);

end

clear  mgfs_f dgfs_f mgfs_out_f mgfs_in_f mgfs_2out_f mgfs_2in_f dgfs_out_f dgfs_in_f dgfs_2out_f dgfs_2in_f

fprintf('Done inter-receiver interferometry.\n');



%% Select results
if  strcmp(inttype,'full')
    full_int_Ss_mon_f = [sqrt(2).*full_int_Ss_mon_f(1:2*nf-1,:,:);full_int_Ss_mon_f(2*nf,:,:)]; % We have taken only half of the frequency spectrum, so we have to adjust the amplitudes
    full_int_Ss_dip_f = [sqrt(2).*full_int_Ss_dip_f(1:2*nf-1,:,:);full_int_Ss_dip_f(2*nf,:,:)];
elseif strcmp(inttype,'mono')
    mon_int_Ss_mon_f = [sqrt(2).*mon_int_Ss_mon_f(1:2*nf-1,:,:);mon_int_Ss_mon_f(2*nf,:,:)]; % We have taken only half of the frequency spectrum, so we have to adjust the amplitudes
end


%% Move inter-receiver results to the time domain to keep causal component only
if strcmp(inttype,'full')
    full_int_Ss_mon_f_2sided = centerfreqs(full_int_Ss_mon_f,2*nf); % Get two-sided frequency spectrum for ifft
    full_int_Ss_dip_f_2sided = centerfreqs(full_int_Ss_dip_f,2*nf);
    full_int_Ss_mon_t_2sided = sqrt(2*2*nf)*ifft(fftshift(full_int_Ss_mon_f_2sided,1),[],1); % Move to time domain
    full_int_Ss_dip_t_2sided = sqrt(2*2*nf)*ifft(fftshift(full_int_Ss_dip_f_2sided,1),[],1);
    clear full_int_Ss_mon_f_2sided full_int_Ss_dip_f_2sided full_int_Ss_mon_f full_int_Ss_dip_f
    if strcmp(method,'VMvi') % flip polarity if using the van Manen vi method
        full_int_Ss_mon_t_2sided = -full_int_Ss_mon_t_2sided;
        full_int_Ss_dip_t_2sided = -full_int_Ss_dip_t_2sided;
    end
    % Apply Tukey window, pad with zeros and move to frequency domain
    full_int_Ss_mon_t = full_int_Ss_mon_t_2sided(1:2*nf,:,:);
    clear full_int_Ss_mon_t_2sided
    full_int_Ss_dip_t = full_int_Ss_dip_t_2sided(1:2*nf,:,:);
    clear full_int_Ss_dip_t_2sided
    full_int_Ss_mon_t = [full_int_Ss_mon_t.*repmat(tuk,[1 size(full_int_Ss_mon_t,2) size(full_int_Ss_mon_t,3)]);zeros(2*nf,size(full_int_Ss_mon_t,2),size(full_int_Ss_mon_t,3))];
    full_int_Ss_dip_t = [full_int_Ss_dip_t.*repmat(tuk,[1 size(full_int_Ss_dip_t,2) size(full_int_Ss_dip_t,3)]);zeros(2*nf,size(full_int_Ss_dip_t,2),size(full_int_Ss_dip_t,3))];
    full_int_Ss_mon_f = fft(real(full_int_Ss_mon_t),2*2*nf,1)/sqrt(2*2*nf);
    full_int_Ss_dip_f = fft(real(full_int_Ss_dip_t),2*2*nf,1)/sqrt(2*2*nf);
    clear full_int_Ss_mon_t full_int_Ss_dip_t
    full_int_Ss_mon_f = full_int_Ss_mon_f(2:2*nf+1,:,:);  % take only half of the frequency spectrum
    full_int_Ss_dip_f = full_int_Ss_dip_f(2:2*nf+1,:,:);  % take only half of the frequency spectrum
elseif strcmp(inttype,'mono')
    mon_int_Ss_mon_f_2sided = centerfreqs(mon_int_Ss_mon_f,2*nf); % Get two-sided frequency spectrum for ifft
    mon_int_Ss_mon_t_2sided = sqrt(2*2*nf)*ifft(fftshift(mon_int_Ss_mon_f_2sided,1),[],1); % Move to time domain
    clear mon_int_Ss_mon_f_2sided
    if strcmp(method,'VMvi') % flip polarity if using the van Manen vi method
        mon_int_Ss_mon_t_2sided = -mon_int_Ss_mon_t_2sided;
    end
    % Apply Tukey window, pad with zeros and move to frequency domain
    mon_int_Ss_mon_t = mon_int_Ss_mon_t_2sided(1:2*nf,:,:);
    clear mon_int_Ss_mon_t_2sided
    mon_int_Ss_mon_t = [mon_int_Ss_mon_t.*repmat(tuk,[1 size(mon_int_Ss_mon_t,2) size(mon_int_Ss_mon_t,3)]);zeros(2*nf,size(mon_int_Ss_mon_t,2),size(mon_int_Ss_mon_t,3))];
    mon_int_Ss_mon_f = fft(real(mon_int_Ss_mon_t),2*2*nf,1)/sqrt(2*2*nf);
    clear mon_int_Ss_mon_t
    mon_int_Ss_mon_f = mon_int_Ss_mon_f(2:2*nf+1,:,:);  % take only half of the frequency spectrum
end



%% Model Green's functions from central source to boundary receivers using the model_gfs_p_scat_acoust function

% Apply Tukey window to taper trace edges (useful in case any scattered
% arrivals occur at the end of the trace, as the traces need to be padded
% with zeros before moving to the frequency domain for cross-correlation)
% and pad with zeros to get traces twice as long.

% cmgfs_t = monopole source, time domain
% cdgfs_t = dipole source, time domain

fprintf('Modelling Green''s functions between the central source and the receiver boundary...\n');

% Allocate space
cmgfs_t=zeros(2*2*nf,nrcx,nsrx_int);
if strcmp(inttype,'full')
    cdgfs_t=zeros(2*2*nf,nrcx,nsrx_int);
end

% Loop over boundary receivers
for rr = 1:nrcx
    if strcmp(scattering,'yes')
        % Model pressure from all sources to receiver rr - monopole
        [~,gf_mon_p_t,~,~] = model_gfs_p_scat_acoust(srx_int,rcx(rr,:),maxf,nf,c,dim,srctype,'recdip','scaty',wavltype,'freqn','timey','tot',scats,fc);
        cmgfs_t(1:2*nf,rr,:) = gf_mon_p_t.*repmat(tuk,[1 size(gf_mon_p_t,2) size(gf_mon_p_t,3)]); % monopole wavefield - time domain
        if strcmp(inttype,'full')
            % Model pressure from all sources to receiver rr - dipole
            [~,~,gf_dip_p_t,~] = model_gfs_p_scat_acoust(srx_int,rcx(rr,:),maxf,nf,c,dim,srctype,'recdip','scaty',wavltype,'freqn','timey','tot',scats,fc);
            for dd = 1:ndim; cdgfs_t(1:2*nf,rr,:) = cdgfs_t(1:2*nf,rr,:) + gf_dip_p_t(:,1,:,dd).*repmat(tuk,[1 size(gf_dip_p_t,2) size(gf_dip_p_t,3)]).*boundnorm_r(rr,dd); end % weight dipole components by the outward normal components - this gives the full dipole wavefield
        end
    elseif strcmp(scattering,'no')
        % Model pressure from all sources to receiver rr - monopole
        [~,gf_mon_p_t,~,~] = model_gfs_p_scat_acoust(srx_int,rcx(rr,:),maxf,nf,c,dim,srctype,'recdip','scatn',wavltype,'freqn','timey','tot',fc);
        cmgfs_t(1:2*nf,rr,:) = gf_mon_p_t.*repmat(tuk,[1 size(gf_mon_p_t,2) size(gf_mon_p_t,3)]); % monopole wavefield - time domain
        if strcmp(inttype,'full')
            % Model pressure from all sources to receiver rr - dipole
            [~,~,gf_dip_p_t,~] = model_gfs_p_scat_acoust(srx_int,rcx(rr,:),maxf,nf,c,dim,srctype,'recdip','scatn',wavltype,'freqn','timey','tot',fc);
            for dd = 1:ndim; cdgfs_t(1:2*nf,rr,:) = cdgfs_t(1:2*nf,rr,:) + gf_dip_p_t(:,1,:,dd).*repmat(tuk,[1 size(gf_dip_p_t,2) size(gf_dip_p_t,3)]).*boundnorm_r(rr,dd); end % weight dipole components by the outward normal components - this gives the full dipole wavefield
        end
    end
    
    if mod(rr,20)==0; fprintf(['Done receiver ' int2str(rr) ' of ' int2str(nrcx) '.\n']); end
    
    clear gf_mon_p_t gf_dip_p_t
    
end

% Move to frequency domain
cmgfs_f = fft(cmgfs_t,[],1)./sqrt(2*2*nf);
clear cmgfs_t
cmgfs_f = cmgfs_f(2:2*nf+1,:,:); % take only half of the frequency spectrum
if strcmp(inttype,'full');
    cdgfs_f = fft(cdgfs_t,[],1)./sqrt(2*2*nf);
    clear cdgfs_t
    cdgfs_f = cdgfs_f(2:2*nf+1,:,:); % take only half of the frequency spectrum
end



%% Perform inter-source interferometry using boundary R of receivers to get the GF between source A and receiver B in frequency domain (using matrix multiplication one frequency at a time)

% Merge the results of inter-receiver interferometry (which turned the
% central receiver into a virtual source) and the wavefields modelled from
% the central source to the boundary of receivers

if strcmp(inttype,'full')
    mgfs_f = cat(3,cmgfs_f,full_int_Ss_mon_f);
    clear cmgfs_f full_int_Ss_mon_f
    dgfs_f = cat(3,cdgfs_f,full_int_Ss_dip_f);
    clear cdgfs_f full_int_Ss_dip_f
elseif strcmp(inttype,'mono')
    mgfs_f = cat(3,cmgfs_f,mon_int_Ss_mon_f);
    clear cmgfs_f mon_int_Ss_mon_f
end


if  strcmp(inttype,'full')
    
    % Evaluate GFs with seismic interferometry
    [~,full_int_Rs_f] = do_xcorr_int(freq2,mgfs_f,1:nsrx_int,nsrx_int+1:(nsrx_int+nrcx_int),ro,c,dr,'intsrc','active','freq',inttype,method,dgfs_f);
    
elseif strcmp(inttype,'mono')
    
    % Evaluate GFs with seismic interferometry
    [~,mon_int_Rs_f] = do_xcorr_int(freq2,mgfs_f,1:nsrx_int,nsrx_int+1:(nsrx_int+nrcx_int),ro,c,dr,'intsrc','active','freq',inttype,method);

end

clear  mgfs_f dgfs_f

fprintf('Done inter-source interferometry.\n');



%% Select results
if  strcmp(inttype,'full')
    int_Rs_f = [sqrt(2).*full_int_Rs_f(1:2*nf-1,:,:);full_int_Rs_f(2*nf,:,:)]; % We have taken only half of the frequency spectrum, so we have to adjust the amplitudes
elseif strcmp(inttype,'mono')
    int_Rs_f = [sqrt(2).*mon_int_Rs_f(1:2*nf-1,:,:);mon_int_Rs_f(2*nf,:,:)]; % We have taken only half of the frequency spectrum, so we have to adjust the amplitudes
end
clear full_int_Ss_mon_f full_int_Ss_dip_f full_int_Rs_f mon_int_Ss_f mon_int_Rs_f




%% The GFs obtained with the two methods are:

% True GFs
gftrue_f = gftrue_mon_p_f.*repmat(conj(wavelet_f),[1 size(gftrue_mon_p_f,2) size(gftrue_mon_p_f,3)]).*repmat(conj(wavelet_f),[1 size(gftrue_mon_p_f,2) size(gftrue_mon_p_f,3)]); % Convolve spectrum of true trace with conjugate of source wavelet
clear gftrue_mon_p_f
gftrue_f = [sqrt(2).*gftrue_f(1:2*nf-1,:,:);gftrue_f(2*nf,:,:)]; % We have taken only half of the frequency spectrum, so we have to adjust the amplitudes
gftrue_f_2sided = centerfreqs(gftrue_f,2*nf); % Get two-sided frequency spectrum for ifft
gftrue_t_2sided = fftshift(sqrt(2*2*nf)*ifft(fftshift(gftrue_f_2sided,1),[],1),1); % Move to time domain
gftrue_t_2sided = real([gftrue_t_2sided(end:-1:2*nf+2,:,:);gftrue_t_2sided(2*nf+1:end,:,:)]);
clear gftrue_f_2sided
gftrue_t = gftrue_t_2sided(2*nf:end,:,:);

% True GF as stack of true GF and its complex conjugate - for comparison
% with interferometry results
if strcmp(method,'WFvir'); gftrue_f_comp = gftrue_f + conj(gftrue_f);
elseif strcmp(method,'WFvi'); gftrue_f_comp = gftrue_f - conj(gftrue_f);
elseif strcmp(method,'VMvi'); gftrue_f_comp = conj(gftrue_f) - gftrue_f;
elseif strcmp(method,'VMvir'); gftrue_f_comp = gftrue_f + conj(gftrue_f);
end
gftrue_f_2sided_comp = centerfreqs(gftrue_f_comp,2*nf); % Get two-sided frequency spectrum for ifft
gftrue_t_2sided_comp = fftshift(sqrt(2*2*nf)*ifft(fftshift(gftrue_f_2sided_comp,1),[],1),1); % Move to time domain
gftrue_t_2sided_comp = real(gftrue_t_2sided_comp(2:end,:,:));
clear gftrue_f_2sided_comp

% Interferometric GFs from source-receiver
gfint_f = int_Rs_f;
gfint_f_2sided = centerfreqs(gfint_f,2*nf); % Get two-sided frequency spectrum for ifft
gfint_t_2sided = fftshift(sqrt(2*2*nf)*ifft(fftshift(gfint_f_2sided,1),[],1),1); % Move to time domain
gfint_t_2sided = real(gfint_t_2sided(2:end,:,:));
clear gfint_f_2sided int_Rs_f

% Normalise amplitudes
if strcmp(normamp,'yes')
    gftrue_t_2sided = gftrue_t_2sided./repmat(max(abs(gftrue_t_2sided)),[2*2*nf-1 1 1]);
    gftrue_t = gftrue_t./repmat(max(abs(gftrue_t)),[2*nf 1 1]);
    gftrue_f_comp = gftrue_f_comp./repmat(max(abs(gftrue_f_comp)),[2*nf 1 1]);
    gftrue_t_2sided_comp = gftrue_t_2sided_comp./repmat(max(abs(gftrue_t_2sided_comp)),[2*2*nf-1 1 1]);
    gfint_f = gfint_f./repmat(max(abs(gfint_f)),[2*nf 1 1]);
    gfint_t_2sided = gfint_t_2sided./repmat(max(abs(gfint_t_2sided)),[2*2*nf-1 1 1]);
end


%% Green's functions AB (from A to B)

% True GF
AB_true_t = gftrue_t(:,1,1);
AB_true_t_2sided = gftrue_t_2sided(:,1,1);

% True GF for comparison
AB_true_f_comp = [0;gftrue_f_comp(:,1,1)];
AB_true_t_2sided_comp = gftrue_t_2sided_comp(:,1,1);
clear gftrue*

% Interferometric GFs from source-receiver
AB_int_f = [0;gfint_f(:,1,1)];
AB_int_t_2sided = gfint_t_2sided(:,1,1);
clear gfint*



%% Plots
close all
scrnsz = get(0,'screensize');
amaxt=max(max([abs(AB_true_t_2sided),abs(AB_int_t_2sided)]));
amaxf=max(max([abs(AB_true_f_comp),abs(AB_int_f)]));

% Plot geometry and true trace
fig1 = figure;
fig1w = 800;
fig1h = 600;
pos = get(fig1,'Position');
outpos = get(fig1,'OuterPosition');
bord = outpos - pos; clear pos outpos
topb = bord(4)-abs(bord(2));
leftb = abs(bord(1));
set(fig1,'Position',[leftb scrnsz(4)-(topb+fig1h) fig1w fig1h])
% Plot geometry
sub1=subplot(211);
hold on
src=plot(srx(1:5:end,1),srx(1:5:end,2),'r*','markersize',8);
rec=plot(rcx(1:7:end,1),rcx(1:7:end,2),'bv','markersize',8,'markerfacecolor','b');
plot(srx_int(:,1),srx_int(:,2),'r*','markersize',8);
plot(rcx_int(:,1),rcx_int(:,2),'bv','markersize',8,'markerfacecolor','b');
if strcmp(scattering,'yes')
    scat=plot(scats(:,1),scats(:,2),'go','markersize',8,'markerfacecolor','g');
end
axis equal
axis([min(srx(:,1)-50) max(srx(:,1)+50) min(srx(:,2)-50) max(srx(:,2)+50)])
box on
set(gca,'fontsize',14)
xlabel ('x (m)','fontsize',14)
ylabel ('y (m)','fontsize',14)
title ('Geometry','fontsize',14)
text(srx_int(1,1),srx_int(1,2)-60,'A','fontsize',14,'horizontalalignment','center')
text(rcx_int(1,1),rcx_int(1,2)-60,'B','fontsize',14,'horizontalalignment','center')
if strcmp(scattering,'no')
    legend([src rec],'Source','Receiver','location','bestoutside')
elseif strcmp(scattering,'yes')
    legend([src rec scat],'Source','Receiver','Scatterer','location','bestoutside')
end
% Plot true trace
sub2=subplot(212);
hold on
plot(time1,AB_true_t,'k','linewidth',2)
axis([0 0.6*max(time1) -max(abs(AB_true_t)) max(abs(AB_true_t))])
box on
set(gca,'fontsize',14)
xlabel ('Time (s)','fontsize',14)
if strcmp(normamp,'yes'); ylabel ('Normalised Amplitude','fontsize',14)
elseif strcmp(normamp,'no'); ylabel ('Amplitude','fontsize',14);end
title ('True trace obtained for a source at A and a receiver at B','fontsize',14)

set(sub1,'Position',[0.275 0.45 0.45 0.45])
set(sub2,'Position',[0.1 0.075 0.8 0.2])


% Plot results: true GF and interferometric GF 
% Time domain, acausal and causal parts
fig2=figure;
fig2w=1200;
fig2h=300;
pos = get(fig1,'Position');
set(fig2,'Position',[pos(1)+leftb scrnsz(4)-(2*topb+fig2h) fig2w fig2h])
hold on
line([0 0],[-amaxt amaxt],'Linestyle','--','color','k')
true=plot(time2,AB_true_t_2sided,'k','linewidth',2);
int=plot(time2,AB_int_t_2sided,'r--','linewidth',2);
axis([0.6*min(time2) 0.6*max(time2) -amaxt amaxt])
box on
set(gca,'fontsize',14)
xlabel ('Time (s)','fontsize',14)
if strcmp(normamp,'yes'); ylabel ('Normalised Amplitude','fontsize',14)
elseif strcmp(normamp,'no'); ylabel ('Amplitude','fontsize',14);end
legend([true int],'True trace','Interferometric trace')
if strcmp(inttype,'full')
    title ('True trace vs. interferometric trace (inter-receiver interferometry with monopoles and dipoles)','fontsize',14)
elseif strcmp(inttype,'mono')
    title ('True trace vs. interferometric trace (inter-receiver interferometry with monopoles only)','fontsize',14)
end
text(0.6*min(time2)/2,-amaxt/2,'Acausal','fontsize',14,'horizontalalignment','center')
text(0.6*max(time2)/2,-amaxt/2,'Causal','fontsize',14,'horizontalalignment','center')



% Plot results: true GF (with acausal part calculated according to the
% interferometric formulae) and interferometric GF
% Time and frequency domain (absolute, real and imaginary)
fig3=figure;
fig3w=1200;
fig3h=600;
pos = get(fig2,'Position');
set(fig3,'Position',[pos(1)+leftb scrnsz(4)-(3*topb+fig3h) fig3w fig3h])
% Time domain
subplot(211)
hold on
true=plot(time2,AB_true_t_2sided_comp,'k','linewidth',2);
int=plot(time2,AB_int_t_2sided,'r--','linewidth',2);
axis([0.6*min(time2) 0.6*max(time2) -amaxt amaxt])
box on
set(gca,'fontsize',14)
xlabel ('Time (s)','fontsize',14)
if strcmp(normamp,'yes'); ylabel ('Normalised Amplitude','fontsize',14)
elseif strcmp(normamp,'no'); ylabel ('Amplitude','fontsize',14);end
legend([true int],'''True'' trace','Interferometric trace')
title ('Time domain','fontsize',14)
% Frequency domain - spectrum
subplot(234)
hold on
plot(freqs,abs(AB_true_f_comp),'k','linewidth',2);
plot(freqs,abs(AB_int_f),'r--','linewidth',2);
axis([0 maxfmod 0 amaxf])
box on
set(gca,'fontsize',14)
xlabel ('Frequency (Hz)','fontsize',14)
if strcmp(normamp,'yes'); ylabel ('Normalised Amplitude','fontsize',14)
elseif strcmp(normamp,'no'); ylabel ('Amplitude','fontsize',14);end
title ('Frequency domain - absolute','fontsize',14)
% Frequency domain - real
subplot(235)
hold on
plot(freqs,real(AB_true_f_comp),'k','linewidth',2);
plot(freqs,real(AB_int_f),'r--','linewidth',2);
axis([0 maxfmod -amaxf amaxf])
box on
set(gca,'fontsize',14)
xlabel ('Frequency (Hz)','fontsize',14)
if strcmp(normamp,'yes'); ylabel ('Normalised Amplitude','fontsize',14)
elseif strcmp(normamp,'no'); ylabel ('Amplitude','fontsize',14);end
title ('Frequency domain - real','fontsize',14)
% Frequency domain - imaginary
subplot(236)
hold on
plot(freqs,imag(AB_true_f_comp),'k','linewidth',2);
plot(freqs,imag(AB_int_f),'r--','linewidth',2);
axis([0 maxfmod -amaxf amaxf])
box on
set(gca,'fontsize',14)
xlabel ('Frequency (Hz)','fontsize',14)
if strcmp(normamp,'yes'); ylabel ('Normalised Amplitude','fontsize',14)
elseif strcmp(normamp,'no'); ylabel ('Amplitude','fontsize',14);end
title ('Frequency domain - imaginary','fontsize',14)
