%% Acoustic modelling and interferometry example - 2D

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% This code is distributed as part of the source-code package 
%                   acoustic_modelling_foldy_v.4.1_seg 
% that accompanies Galetti et al. (2013). The package can be downloaded 
% from the Geophysics source-code archive at 
%                   http://software.seg.org/2013/0003
% Use of this code is subject to acceptance of the terms and conditions
% that can be found at http://software.seg.org/disclaimer.txt 
% Copyright 2013 by The Society of Exploration Geophysicists (SEG)
% Reference:
% Galetti, E., Halliday, D., Curtis, A. (2013) A simple and exact acoustic 
% wavefield modelling code for data processing, imaging and interferometry 
% applications. Geophysics.
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% This script serves as an example of how the acoustic GF modelling code
% can be used to model acoustic wavefields from a boundary of sources to
% two enclosed receivers, and how to apply interferometry to construct the
% GF signal between them. In addition, it compares the output of
% interferometry executed with signals modelled with the Foldy method and
% with the finite-difference code fdelmodc (written by Jan Thorbecke at
% Delft). This FD code produces outputs which are in the Seismic Unix (SU)
% format. The FD code and SU source files are not provided within this
% package, but can be freely dowloaded - see References.
% A circular boundary of sources is used for interferometry and
% cross-correlation is performed in the frequency domain (using matrix
% multiplication one frequency at a time). The results are close to the true
% Green's functions obtained by direct modelling between the two receivers:
% - the interferometric Green's function constructed using traces modelled
% with the Foldy code matches the true Green's function relatively well,
% with any difference likely to be due to the fact that only monopole 
% sources are used (see example_inter_receiver_mon_dip_comp.m).
% - the interferometric Green's function constructed using traces modelled
% with the FD code has a lower accuracy due to numerical dispersion errors
% proper of finite-difference modelling.
%
% Figure 1: FD and Foldy traces from all sources to receiver A
% Figure 2: Geometry (top)
%           True modelled trace (bottom):
%               black solid trace - true Green's function (Foldy)
%               green dot-dashed trace - true Green's function (FD)
% Figure 3: Two-sided Green's functions in time domain:
%               black solid trace - true Green's function (Foldy)
%               red dashed trace - interferometric Green's function (Foldy)
%               green dot-dashed trace - interferometric Green's function (FD)
% Figure 4: Comparison: 'true' Green's function (black solid trace -
%                       obtained from the causal part of the true Green's 
%                       function in Figure 2 according to the 
%                       interferometry formulae) vs. interferometric
%                       Green's functions (red dashed trace - Foldy; green
%                       dot-dashed trace - FD)
%                       top:            traces in time domain
%                       bottom left:    frequency spectra 
%                       bottom centre:  real part of frequency spectra 
%                       bottom right:   imaginary part of frequency spectra
%%% FIGURES IN PAPER %%%
% Figure 5,8:  Geometry
% Figure 6,9:  Comparison: True Green's function calculated using the 
%                          analytical Green's function formulae vs. the true
%                          Green's function calculated using the FD modelling
%                          code fdelmodc.
% Figure 7,10: Comparison: true Green's function vs. interferometric Green's 
%                          functions, as full trace and zoomed-in panels.
%
%
% References:
%       Groenenboom, J., Snieder, R. (1995) Attenuation, dispersion, and
%   anisotropy by multiple scattering of transmitted waves through
%   distributions of scatterers. J. Acoust. Soc. Am. 98 (6): 3482-3492.
%       Thorbecke, J. W. and Draganov, D. (2011) Finite-difference modeling
%   experiments for seismic interferometry, Geophysics, 76: H1-H18
%       Snieder, R. (1999) Imaging and averaging in complex media, in
%   Diffuse waves in complex media, edited by J.P. Fouque, pp. 405-454,
%   Kluwer, Dordrecht
%       Snieder, R. (2009) A Guided Tour of Mathematical Methods for the
%   Physical Sciences, 2nd ed., Cambridge University Press.
%       van Manen, D.-J., Robertsson, J.O.A., Curtis, A. (2005) Modeling of 
%   Wave Propagation in Inhomogeneous Media. Physical Review Letters, 94: 
%   164301.
%       Wapenaar, K., Fokkema, J. (2006) Green's function representations  
%   for seismic interferometry, Geophysics, 71: SI33-SI46.
%
%
% Required:
% fdelmodc                  - FD modelling code written by Jan Thorbecke
%                             (code and manual can be found at
%                             http://janth.home.xs4all.nl/Software/Software.html)
% ReadSu.m                  - functions that read/write Seismic Unix (SU)
% WriteSuStructure.m          files (can be downloaded as part of the 
%                             SegyMAT package from http://segymat.sourceforge.net/)
% makecircbound.m           - creates a circular boundary of points with a
%                             certain radius or point spacing
% do_xcorr_int.m            - performs cross-correlational interferometry
% acoustic_p_gfs_direct_f.m - models direct wave between two points in an
%                             acoustic medium in the frequency domain only
% model_gfs_p_scat_acoust.m - models scattered waves in an acoustic medium
% centerfreqs.m             - creates a two-sided centred frequency
%                             spectrum from positive frequencies
% rickerwavl_time.m         - creates Ricker wavelet for source
%
%
% Erica Galetti, July 2012
% Contact: erica.galetti@ed.ac.uk


clear all
close all


%% Define paths
% Home directory - contains FD and Foldy modelling codes
homedir='/home/s0789170';

% FD modelling code path (within homedir)
fdpath=[ homedir '/PROGRAMS/FDELMODC/bin'];
% Examples path
expath=[ homedir '/matlab_codes/erica_modelling/acoustic_modelling_foldy/acoustic_modelling_foldy_v.4.1/acoustic_modelling_foldy_v.4.1_seg/paper_plots'];
% FD data directory (within examples path)
datadir=[ expath '/FD_data'];



%% Interferometry formula used
% fdelmodc uses volume injection sources by default
method = 'WFvi';    
% 'WFvi'    --- Wapenaar & Fokkema (2006)
%                   volume injection source

    
%% Type of interferometry
% Use only monopoles so that the source in the FD and Foldy case are
% located at the same point (using a dipole source in the FD code
% introduces a time-shift due to the positive and negative part of the
% dipole being located at different points)
inttype = 'mono';


%% Normalise amplitudes?
normamp = 'no'; % 'yes' or 'no'


%% Source wavelet type
wavltype = 'wavly_pos'; % the positive-time wavelet is not zero-phase - for an example with zero-phase wavelet, see the source-receiver interferometry example


%% Type of source
% fdelmodc uses volume injection sources by default
if strcmp(method,'WFvi')
    srctype='volinjpos';
end


%% Define a series of input parameters

nf = 1000;
nf = 2^(nextpow2(nf)); % round number of frequencies to the nearest power of 2 (makes fft/ifft run faster)
maxf = 500; % max frequency
fc = 10; % central frequency of Ricker wavelet
dt = 1/(2*maxf); % time step

% Time
time1 = dt.*(0:2*nf-1);
time1 = reshape(time1,2*nf,1);
% Define two-sided time vector
time2 = [flipud(-time1(2:end));time1(1:end)]; % negative and positive times

% Frequencies
df = maxf/(2*nf);
freq2 =  (1:2*nf).*df;
freqs=[0;freq2.'];
om = 2*pi.*freq2; % angular frequencies

ro = 1; % medium density
c = 600; % constant background velocity

% The FD code is 2D
dim = '2d'; 
ndim = 2;


%% Model the source wavelet (using the Foldy code)
% This wavelet is then used for both FD and Foldy modelling

% Go to FD data directory
cd(datadir);
delete('*.txt','*.su');

% Scaling factor between Foldy and FD timestep
tsc = 2;

% Model wavelet using the FD code only to get header values
system([ fdpath '/makewave file_out=wavelet.su nt=' int2str(2*nf*tsc) ' dt=' num2str(dt/tsc) ' fp=' num2str(fc) ' t0=' num2str(ceil((1/fc)/dt)*dt) ' scale=1 w=g2 verbose=1']);
[~,SuTraceHeaders,SuHeader]=ReadSu('wavelet.su');

% Model the source wavelet using the Foldy code
[freqw,~,~,~,~,~,wavelet_f,wavelet_t] = model_gfs_p_scat_acoust([0,0,0],[0,0,0],maxf*tsc,nf*tsc,c,dim,srctype,'srcdip','scatn',wavltype,'freqy_1nf','timey','tot',fc);

% Write wavelet to file
WriteSuStructure('wavelet.su',SuHeader,SuTraceHeaders,wavelet_t);

% Take the maximum modelled frequency to be the first frequency (>fc) that has amplitude less than 1/400 of the peak amplitude at fc (same as in the FD code)
ind = find(freqw>=fc,1,'first');
maxfmod=freqw(ind-1+find(abs(wavelet_f(ind:end))<(0.0025*max(abs(wavelet_f))),1,'first')); 
nyq = c/maxfmod; % minimum Nyquist wavelength

% If memory is not an issue, uncomment the following line (reduces the spacing between boundary sources/receivers)
% nyq = c/maxf;



%% Define source and receiver positions

% Define source boundary
% Points on circle
rad_s = 500; % radius of source boundary (gets adjusted depending on source spacing 4 lines below)
dx_s = nyq/2; % spacing of sources on boundary is half the minimum Nyquist wavelength
[coords,radnew] = makecircbound(rad_s,dx_s,'fixds',0,0);
rad_s = radnew;
nsrx = size(coords,1);
srx(1:nsrx,1) = coords(:,1); % boundary x-coordinate
srx(1:nsrx,2) = coords(:,2); % boundary y-coordinate
srx(1:nsrx,3) = 0; % boundary z-coordinate
% Calculate distance between sources (offsets_s) and ds
offsets_s = zeros(nsrx,1);
ds = zeros(nsrx,1);
offsets_s(1:nsrx) = sqrt(diff(srx([1:nsrx 1],1)).^2 + diff(srx([1:nsrx 1],2)).^2);
ds(1:nsrx) = offsets_s(1:nsrx)/2 + offsets_s([nsrx 1:nsrx-1])/2;
clear offsets_s coords radnew

% Calculate boundary normal (the boundary is a circle centred at the
% origin, so its x and y components are simply the cos and sin of the
% angles between the source-position-vector and the x axis).
boundnorm = zeros(nsrx,ndim);
for dd = 1:ndim; boundnorm(:,dd) = srx(:,dd)/rad_s; end


% Define position of receivers
rcA=[-90,-55,0];
rcB=[80,50,0];
rcx = [rcA;rcB];
nrcx = size(rcx,1);


% Character strings containing receiver positions as used in the FD
% modelling code (they are actually source positions, but we can use
% source-receiver reciprocity because we are in the acoustic case - it
% makes the execution of the FD code quicker)
arrayx='xrcva=';
arrayz='zrcva=';
for ss = 1:nsrx
    if ss==1
        arrayx=[arrayx num2str(srx(ss,1)) ];
        arrayz=[arrayz num2str(srx(ss,2)) ];
    else
        arrayx=[arrayx ',' num2str(srx(ss,1)) ];
        arrayz=[arrayz ',' num2str(srx(ss,2)) ];
    end
end



%% ================== FD MODELLING AND INTERFEROMETRY ================== %%

%% Create the velocity model
dh = 1;
npad = 400;
ntop = npad + rad_s; nbottom = npad + rad_s;
nleft = npad + rad_s; nright = npad + rad_s;
modelfile='model';
shotfile='shot';

system([ fdpath '/makemod file_base=' modelfile ' cp0=' num2str(c) ' ro0=' num2str(ro) ' sizex=' num2str(ceil(nleft+nright)) ' sizez=' num2str(ceil(ntop+nbottom)) ' dx=' num2str(dh) ' dz=' num2str(dh) ' orig=-' num2str(nleft) ',-' num2str(ntop) ]);


%% Model the real Green's function between A and B using the FD code
system([ fdpath '/fdelmodc file_cp=' modelfile '_cp.su file_den=' modelfile '_ro.su file_src=wavelet.su file_rcv=' shotfile ...
	' dtrcv=' num2str(dt) ...
    ' rec_ntsam=' num2str((2*nf)) ...
    ' ischeme=1 iorder=4' ...
    ' cfree=0 boundary=4 ntaper=' num2str(npad) ...
    ' tapleft=1 tapright=1 tapbottom=1 taptop=1' ...
    ' verbose=4' ...
    ' src_type=1 src_orient=1' ...
    ' xsrc=' num2str(rcx(1,1)) ' zsrc=' num2str(rcx(1,2)) ' nshot=1 nsrc=1 wav_random=0' ...
	' xrcva=' num2str(rcx(2,1)) ' zrcva=' num2str(rcx(2,2))  ...
	' rec_type_p=1 rec_type_vz=0 rec_int_vx=3 rec_int_vz=3' ...
	' sna_type_p=0 sna_type_vz=0' ]);

% Shift modelled trace by half a timestep (dt/2)
system([ fdpath '/basop file_in=' shotfile '_rp.su file_out=' shotfile '_rp2.su choice=shift shift=' num2str(dt/2/tsc) ]);
system(['mv ' shotfile '_rp2.su ' shotfile '_rp.su']);

% Read data
[Data,~,~]=ReadSu([ datadir '/' shotfile '_rp.su']);
gftrue_mon_p_t = zeros(2*nf,2,2);
gftrue_mon_p_t(:,2,1) = Data(1:2*nf);
gftrue_mon_p_t(:,1,2) = Data(1:2*nf);


%% Model the boundary to receiver-network Green's functions using the FD code

% mgfs_t = monopole source, time domain
% dgfs_t = dipole source, time domain

% Allocate space
mgfs_t=zeros(2*nf,nrcx,nsrx);

% Start counting time
tic

% Loop over receivers
for rr = 1:nrcx
    
    % Model pressure from source ss to all receivers
    % (we apply source-receiver reciprocity and invert sources and
    % receivers because of the way the FD code is set up)
    system(['rm ' shotfile '_rp.su']);
    system([ fdpath '/fdelmodc file_cp=' modelfile '_cp.su file_den=' modelfile '_ro.su file_src=wavelet.su file_rcv=' shotfile ...
        ' dtrcv=' num2str(dt) ...
        ' rec_ntsam=' num2str((2*nf)) ...
        ' ischeme=1 iorder=4' ...
        ' cfree=0 boundary=4 ntaper=' num2str(npad) ...
        ' tapleft=1 tapright=1 tapbottom=1 taptop=1' ...
        ' verbose=4' ...
        ' src_type=1 src_orient=1' ...
        ' xsrc=' num2str(rcx(rr,1)) ' zsrc=' num2str(rcx(rr,2)) ' nshot=1 nsrc=1 wav_random=0' ...
        ' ' arrayx ' ' arrayz  ...
        ' rec_type_p=1 rec_type_vz=0 rec_int_vx=3 rec_int_vz=3' ...
        ' sna_type_p=0 sna_type_vz=0' ]);
    
    % Stop counting time
    fprintf(['Time taken to model data from all sources to ' num2str(rr) '/' num2str(nrcx) ' receiver(s) - FD:\n']);
    toc
    
    % Shift modelled trace by half a timestep (dt/2)
    system([ fdpath '/basop file_in=' shotfile '_rp.su file_out=' shotfile '_rp2.su choice=shift shift=' num2str(dt/2/tsc) ]);
    system(['mv ' shotfile '_rp2.su ' shotfile '_rp.su']);

    % Read data
    [Data,~,~]=ReadSu([ datadir '/' shotfile '_rp.su']);
    mgfs_t(:,rr,:) = Data;
          
end

clear Data


% Apply Tukey window to taper trace edges (useful in case any scattered
% arrivals occur at the end of the trace, as the traces need to be padded
% with zeros before moving to the frequency domain for cross-correlation)
% and pad with zeros to get traces twice as long.

% Construct Tukey window (this assumes the Signal Processing Toolbox is not
% available and avoids calling the tukeywin function, giving similar results)
ntap = 10; % number of time samples to taper on each side of the trace (make sure it is much less than half the total number of time samples!)
theta_t = linspace(0,1,ntap+1).';
taper = (1+cos(pi.*theta_t))./2;
% Create Tukey window as taper-ones-taper
tuk = [taper(end:-1:1);ones(2*nf-2*(ntap+1),1,1);taper];
clear ntap theta_t taper

gftrue_mon_p_t = [gftrue_mon_p_t.*repmat(tuk,[1 size(gftrue_mon_p_t,2) size(gftrue_mon_p_t,3)]);zeros(2*nf,size(gftrue_mon_p_t,2),size(gftrue_mon_p_t,3))];
mgfs_t = [mgfs_t.*repmat(tuk,[1 size(mgfs_t,2) size(mgfs_t,3)]);zeros(2*nf,size(mgfs_t,2),size(mgfs_t,3))];

% Move to frequency domain
gftrue_mon_p_f = fft(gftrue_mon_p_t,[],1)./sqrt(2*2*nf);
clear gftrue_mon_p_t
gftrue_mon_p_f = gftrue_mon_p_f(2:2*nf+1,:,:); % take only half of the frequency spectrum
mgfs_f = fft(mgfs_t,[],1)./sqrt(2*2*nf);
% Plot traces (sources to receiver A)
theta = 2*pi/nsrx;
figure; subplot(121)
imagesc(radtodeg(0:theta:(2*pi-theta)),time1,reshape(mgfs_t(1:2*nf,1,:),2*nf,nsrx,1)); colormap('gray'); box on
set(gca,'fontsize',14)
xlabel ('Source position (degrees)','fontsize',14)
ylabel ('Time (s)','fontsize',14)
title ('Sources to A (FD modelling)','fontsize',14)
clear mgfs_t
mgfs_f = mgfs_f(2:2*nf+1,:,:); % take only half of the frequency spectrum


%% Perform xcorr interferometry from boundary S of sources in frequency domain using matrix multiplication one frequency at a time
% Use function do_xcorr_int to do cross-correlational interferometry

% Evaluate GFs with seismic interferometry
[~,mon_int_Ss_f] = do_xcorr_int(freq2,mgfs_f,1:nrcx,1:nrcx,ro,c,ds,'intrec','active','freq','mono',method);

clear  mgfs_f


%% Select results
int_Ss_f = [sqrt(2).*mon_int_Ss_f(1:2*nf-1,:,:);mon_int_Ss_f(2*nf,:,:)]; % We have taken only half of the frequency spectrum, so we have to adjust the amplitudes
clear mon_int_Ss_f
  

%% Recalculate wavelet at sampling interval dt
% Model the source wavelet using the Foldy code
clear wavelet_t wavelet_f

% Wavelet in time domain
[~,~,~,wavelet_t] = model_gfs_p_scat_acoust([0,0,0],[0,0,0],maxf,nf,c,dim,srctype,'srcdip','scatn',wavltype,'freqn','timey','tot',fc);

% Wavelet in frequency domain
wavelet_f = fft([wavelet_t;zeros(size(wavelet_t))])./sqrt(2*2*nf);
wavelet_f = wavelet_f(2:2*nf+1,:,:); % take only half of the frequency spectrum


%% The true and interferometric GFs are:

% True GF
gftrue_f = gftrue_mon_p_f.*repmat(conj(wavelet_f),[1 size(gftrue_mon_p_f,2) size(gftrue_mon_p_f,3)]); % Convolve spectrum of true trace with conjugate of source wavelet
gftrue_f = [sqrt(2).*gftrue_f(1:2*nf-1,:,:);gftrue_f(2*nf,:,:)]; % We have taken only half of the frequency spectrum, so we have to adjust the amplitudes
gftrue_f_2sided = centerfreqs(gftrue_f,2*nf); % Get two-sided frequency spectrum for ifft
gftrue_t_2sided = fftshift(sqrt(2*2*nf)*ifft(fftshift(gftrue_f_2sided,1),[],1),1); % Move to time domain
gftrue_t_2sided = real([gftrue_t_2sided(end:-1:2*nf+2,:,:);gftrue_t_2sided(2*nf+1:end,:,:)]);
clear gftrue_f_2sided
gftrue_t = gftrue_t_2sided(2*nf:end,:,:);

% True GF as stack of true GF and its complex conjugate - for comparison
% with interferometry results
if strcmp(method,'WFvir'); gftrue_f_comp = gftrue_f + conj(gftrue_f);
elseif strcmp(method,'WFvi'); gftrue_f_comp = gftrue_f - conj(gftrue_f);
elseif strcmp(method,'VMvi'); gftrue_f_comp = conj(gftrue_f) - gftrue_f;
elseif strcmp(method,'VMvir'); gftrue_f_comp = gftrue_f + conj(gftrue_f);
end
gftrue_f_2sided_comp = centerfreqs(gftrue_f_comp,2*nf); % Get two-sided frequency spectrum for ifft
gftrue_t_2sided_comp = fftshift(sqrt(2*2*nf)*ifft(fftshift(gftrue_f_2sided_comp,1),[],1),1); % Move to time domain
gftrue_t_2sided_comp = real(gftrue_t_2sided_comp(2:end,:,:));
clear gftrue_f_2sided_comp

% Interferometric GFs from boundary S of sources
gfint_f = int_Ss_f;
gfint_f_2sided = centerfreqs(int_Ss_f,2*nf); % Get two-sided frequency spectrum for ifft
gfint_t_2sided = fftshift(sqrt(2*2*nf)*ifft(fftshift(gfint_f_2sided,1),[],1),1); % Move to time domain
gfint_t_2sided = real(gfint_t_2sided(2:end,:,:));
clear gfint_f_2sided int_Ss_f

% Normalise amplitudes
if strcmp(normamp,'yes')
    gftrue_t_2sided = gftrue_t_2sided./repmat(max(abs(gftrue_t_2sided)),[2*2*nf-1 1 1]);
    gftrue_t = gftrue_t./repmat(max(abs(gftrue_t)),[2*nf 1 1]);
    gftrue_f_comp = gftrue_f_comp./repmat(max(abs(gftrue_f_comp)),[2*nf 1 1]);
    gftrue_t_2sided_comp = gftrue_t_2sided_comp./repmat(max(abs(gftrue_t_2sided_comp)),[2*2*nf-1 1 1]);
    gfint_f = gfint_f./repmat(max(abs(gfint_f)),[2*nf 1 1]);
    gfint_t_2sided = gfint_t_2sided./repmat(max(abs(gfint_t_2sided)),[2*2*nf-1 1 1]);
end



%% Green's functions AB (from A to B)

% True GF
AB_true_t_FD = gftrue_t(:,2,1);
AB_true_t_2sided_FD = gftrue_t_2sided(:,2,1);

% True GF for comparison
AB_true_f_comp_FD = [0;gftrue_f_comp(:,2,1)];
AB_true_t_2sided_comp_FD = gftrue_t_2sided_comp(:,2,1);
clear gftrue*

% Interferometric GFs from boundary S of sources
AB_int_f_FD = [0;gfint_f(:,2,1)];
AB_int_t_2sided_FD = gfint_t_2sided(:,2,1);
clear gfint*

%% =========================== END OF FD CASE ========================== %%

clearvars -except boundnorm df dim ds dt dh npad tsc fc freq2 freqs inttype maxf maxfmod ...
                    method ndim nf normamp nrcx nsrx nyq offsets_s om rad_s rcA rcB rcx ...
                    ro scattering srctype srx time1 time2 tuk c wavelet_f wavelet_t wavltype ...
                    AB_* theta

%% ================= FOLDY MODELLING AND INTERFEROMETRY ================ %%

%% Model the real Green's function between A and B using the Foldy method
[~,gftrue_mon_p_t,~,~] = model_gfs_p_scat_acoust(rcx,rcx,maxf,nf,c,dim,srctype,'srcdip','scatn',wavltype,'freqn','timey','tot',fc);


%% Model the boundary to receiver-network Green's functions using the Foldy method

% mgfs_t = monopole source, time domain
% dgfs_t = dipole source, time domain

% Allocate space
mgfs_t=zeros(2*nf,nrcx,nsrx);

% Start counting time
tic

% Loop over boundary sources
for ss = 1:nsrx
    
    % Model pressure from source ss to all receivers
    [~,gf_mon_p_t,~,~] = model_gfs_p_scat_acoust(srx(ss,:),rcx,maxf,nf,c,dim,srctype,'srcdip','scatn',wavltype,'freqn','timey','tot',fc);
    mgfs_t(:,:,ss) = gf_mon_p_t; % monopole wavefield - time domain
    
    if mod(ss,20)==0; fprintf(['Done source ' int2str(ss) ' of ' int2str(nsrx) '.\n']); end
    
    clear gf_mon_p_t gf_dip_p_t
    
end

% Stop counting time
fprintf(['Time taken to model data from all sources to ' num2str(nrcx) ' receiver(s) - Foldy:\n']);
toc

% Apply Tukey window to taper trace edges (useful in case any scattered
% arrivals occur at the end of the trace, as the traces need to be padded
% with zeros before moving to the frequency domain for cross-correlation)
% and pad with zeros to get traces twice as long.

gftrue_mon_p_t = [gftrue_mon_p_t.*repmat(tuk,[1 size(gftrue_mon_p_t,2) size(gftrue_mon_p_t,3)]);zeros(2*nf,size(gftrue_mon_p_t,2),size(gftrue_mon_p_t,3))];
mgfs_t = [mgfs_t.*repmat(tuk,[1 size(mgfs_t,2) size(mgfs_t,3)]);zeros(2*nf,size(mgfs_t,2),size(mgfs_t,3))];

% Move to frequency domain
gftrue_mon_p_f = fft(gftrue_mon_p_t,[],1)./sqrt(2*2*nf);
clear gftrue_mon_p_t
gftrue_mon_p_f = gftrue_mon_p_f(2:2*nf+1,:,:); % take only half of the frequency spectrum
mgfs_f = fft(mgfs_t,[],1)./sqrt(2*2*nf);
% Plot traces (sources to receiver A)
subplot(122)
imagesc(radtodeg(0:theta:(2*pi-theta)),time1,reshape(mgfs_t(1:2*nf,1,:),2*nf,nsrx,1)); colormap('gray'); box on
set(gca,'fontsize',14)
xlabel ('Source position (degrees)','fontsize',14)
ylabel ('Time (s)','fontsize',14)
title ('Sources to A (Foldy modelling)','fontsize',14)
clear mgfs_t
mgfs_f = mgfs_f(2:2*nf+1,:,:); % take only half of the frequency spectrum


%% Perform xcorr interferometry from boundary S of sources in frequency domain using matrix multiplication one frequency at a time
% Use function do_xcorr_int to do cross-correlational interferometry

% Evaluate GFs with seismic interferometry
[~,mon_int_Ss_f] = do_xcorr_int(freq2,mgfs_f,1:nrcx,1:nrcx,ro,c,ds,'intrec','active','freq','mono',method);

clear  mgfs_f


%% Select results
int_Ss_f = [sqrt(2).*mon_int_Ss_f(1:2*nf-1,:,:);mon_int_Ss_f(2*nf,:,:)]; % We have taken only half of the frequency spectrum, so we have to adjust the amplitudes
clear mon_int_Ss_f


%% The true and interferometric GFs are:

% True GF
gftrue_f = gftrue_mon_p_f.*repmat(conj(wavelet_f),[1 size(gftrue_mon_p_f,2) size(gftrue_mon_p_f,3)]); % Convolve spectrum of true trace with conjugate of source wavelet
gftrue_f = [sqrt(2).*gftrue_f(1:2*nf-1,:,:);gftrue_f(2*nf,:,:)]; % We have taken only half of the frequency spectrum, so we have to adjust the amplitudes
gftrue_f_2sided = centerfreqs(gftrue_f,2*nf); % Get two-sided frequency spectrum for ifft
gftrue_t_2sided = fftshift(sqrt(2*2*nf)*ifft(fftshift(gftrue_f_2sided,1),[],1),1); % Move to time domain
gftrue_t_2sided = real([gftrue_t_2sided(end:-1:2*nf+2,:,:);gftrue_t_2sided(2*nf+1:end,:,:)]);
clear gftrue_f_2sided
gftrue_t = gftrue_t_2sided(2*nf:end,:,:);

% True GF as stack of true GF and its complex conjugate - for comparison
% with interferometry results
if strcmp(method,'WFvir'); gftrue_f_comp = gftrue_f + conj(gftrue_f);
elseif strcmp(method,'WFvi'); gftrue_f_comp = gftrue_f - conj(gftrue_f);
elseif strcmp(method,'VMvi'); gftrue_f_comp = conj(gftrue_f) - gftrue_f;
elseif strcmp(method,'VMvir'); gftrue_f_comp = gftrue_f + conj(gftrue_f);
end
gftrue_f_2sided_comp = centerfreqs(gftrue_f_comp,2*nf); % Get two-sided frequency spectrum for ifft
gftrue_t_2sided_comp = fftshift(sqrt(2*2*nf)*ifft(fftshift(gftrue_f_2sided_comp,1),[],1),1); % Move to time domain
gftrue_t_2sided_comp = real(gftrue_t_2sided_comp(2:end,:,:));
clear gftrue_f_2sided_comp

% Interferometric GFs from boundary S of sources
gfint_f = int_Ss_f;
gfint_f_2sided = centerfreqs(int_Ss_f,2*nf); % Get two-sided frequency spectrum for ifft
gfint_t_2sided = fftshift(sqrt(2*2*nf)*ifft(fftshift(gfint_f_2sided,1),[],1),1); % Move to time domain
gfint_t_2sided = real(gfint_t_2sided(2:end,:,:));
clear gfint_f_2sided int_Ss_f

% Normalise amplitudes
if strcmp(normamp,'yes')
    gftrue_t_2sided = gftrue_t_2sided./repmat(max(abs(gftrue_t_2sided)),[2*2*nf-1 1 1]);
    gftrue_t = gftrue_t./repmat(max(abs(gftrue_t)),[2*nf 1 1]);
    gftrue_f_comp = gftrue_f_comp./repmat(max(abs(gftrue_f_comp)),[2*nf 1 1]);
    gftrue_t_2sided_comp = gftrue_t_2sided_comp./repmat(max(abs(gftrue_t_2sided_comp)),[2*2*nf-1 1 1]);
    gfint_f = gfint_f./repmat(max(abs(gfint_f)),[2*nf 1 1]);
    gfint_t_2sided = gfint_t_2sided./repmat(max(abs(gfint_t_2sided)),[2*2*nf-1 1 1]);
end



%% Green's functions AB (from A to B)

% True GF
AB_true_t = gftrue_t(:,2,1);
AB_true_t_2sided = gftrue_t_2sided(:,2,1);

% True GF for comparison
AB_true_f_comp = [0;gftrue_f_comp(:,2,1)];
AB_true_t_2sided_comp = gftrue_t_2sided_comp(:,2,1);
clear gftrue*

% Interferometric GFs from boundary S of sources
AB_int_f = [0;gfint_f(:,2,1)];
AB_int_t_2sided = gfint_t_2sided(:,2,1);
clear gfint*


%% ========================== END OF FOLDY CASE ======================== %%


%% Plots

scrnsz = get(0,'screensize');
amaxt=max(max([abs(AB_true_t_2sided),abs(AB_true_t_2sided_FD),abs(AB_int_t_2sided),abs(AB_int_t_2sided_FD)]));
amaxf=max(max([abs(AB_true_f_comp),abs(AB_int_f),abs(AB_int_f_FD)]));

% Plot geometry and true traces
fig1 = figure;
fig1w = 800;
fig1h = 600;
pos = get(fig1,'Position');
outpos = get(fig1,'OuterPosition');
bord = outpos - pos; clear pos outpos
topb = bord(4)-abs(bord(2));
leftb = abs(bord(1));
set(fig1,'Position',[leftb scrnsz(4)-(topb+fig1h) fig1w fig1h])
% Plot geometry
sub1=subplot(211);
hold on
src=plot(srx(1:6:end,1),srx(1:6:end,2),'r*','markersize',8);
rec=plot(rcx(:,1),rcx(:,2),'bv','markersize',8,'markerfacecolor','b');
axis equal
axis([min(srx(:,1)-50) max(srx(:,1)+50) min(srx(:,2)-50) max(srx(:,2)+50)])
box on
set(gca,'fontsize',14)
xlabel ('x (m)','fontsize',14)
ylabel ('y (m)','fontsize',14)
title ('Geometry','fontsize',14)
text(rcx(1,1),rcx(1,2)-60,'A','fontsize',14,'horizontalalignment','center')
text(rcx(2,1),rcx(2,2)-60,'B','fontsize',14,'horizontalalignment','center')
legend([src rec],'Source','Receiver','location','bestoutside')
% Plot true trace
sub2=subplot(212);
hold on
foldy=plot(time1,AB_true_t,'k','linewidth',2);
fd=plot(time1,AB_true_t_FD,'g-.','linewidth',2);
axis([0 max(time1) -amaxt amaxt])
box on
set(gca,'fontsize',14)
xlabel ('Time (s)','fontsize',14)
if strcmp(normamp,'yes'); ylabel ('Normalised Amplitude','fontsize',14)
elseif strcmp(normamp,'no'); ylabel ('Amplitude','fontsize',14);end
title ('True trace obtained for a source at A and a receiver at B','fontsize',14)
legend([fd foldy],'FD modelling','Foldy method')

set(sub1,'Position',[0.275 0.45 0.45 0.45])
set(sub2,'Position',[0.1 0.075 0.8 0.2])


% Plot results: true GF and interferometric GF 
% Time domain, acausal and causal parts
fig2=figure;
fig2w=1200;
fig2h=300;
pos = get(fig1,'Position');
set(fig2,'Position',[pos(1)+leftb scrnsz(4)-(2*topb+fig2h) fig2w fig2h])
% Complete traces
hold on
line([0 0],[-amaxt amaxt],'Linestyle','--','color','k')
true=plot(time2,AB_true_t_2sided,'k','linewidth',2);
int=plot(time2,AB_int_t_2sided,'r--','linewidth',2);
int2=plot(time2,AB_int_t_2sided_FD,'g-.','linewidth',2);
axis([min(time2) max(time2) -amaxt amaxt])
box on
set(gca,'fontsize',14)
xlabel ('Time (s)','fontsize',14)
if strcmp(normamp,'yes'); ylabel ('Normalised Amplitude','fontsize',14)
elseif strcmp(normamp,'no'); ylabel ('Amplitude','fontsize',14);end
legend([true int int2],'True trace (Foldy)','Interferometric trace (Foldy)','Interferometric trace (FD)')
title ('True trace vs. interferometric trace (inter-receiver interferometry with monopoles only)','fontsize',14)
text(min(time2)/2,-amaxt/2,'Acausal','fontsize',14,'horizontalalignment','center')
text(max(time2)/2,-amaxt/2,'Causal','fontsize',14,'horizontalalignment','center')


% Plot results: true GF (with acausal part calculated according to the
% interferometric formulae) and interferometric GF
% Time and frequency domain (absolute, real and imaginary)
fig3=figure;
fig3w=1200;
fig3h=600;
pos = get(fig2,'Position');
set(fig3,'Position',[pos(1)+leftb scrnsz(4)-(3*topb+fig3h) fig3w fig3h])
% Time domain
subplot(211)
hold on
true=plot(time2,AB_true_t_2sided_comp,'k','linewidth',2);
int=plot(time2,AB_int_t_2sided,'r--','linewidth',2);
int2=plot(time2,AB_int_t_2sided_FD,'g-.','linewidth',2);
axis([min(time2) max(time2) -amaxt amaxt])
box on
set(gca,'fontsize',14)
xlabel ('Time (s)','fontsize',14)
if strcmp(normamp,'yes'); ylabel ('Normalised Amplitude','fontsize',14)
elseif strcmp(normamp,'no'); ylabel ('Amplitude','fontsize',14);end
legend([true int int2],'''True'' trace (Foldy)','Interferometric trace (Foldy)','Interferometric trace (FD)')
title ('Time domain','fontsize',14)
% Frequency domain - spectrum
subplot(234)
hold on
plot(freqs,abs(AB_true_f_comp),'k','linewidth',2);
plot(freqs,abs(AB_int_f),'r--','linewidth',2);
plot(freqs,abs(AB_int_f_FD),'g-.','linewidth',2);
axis([0 maxfmod 0 amaxf])
box on
set(gca,'fontsize',14)
xlabel ('Frequency (Hz)','fontsize',14)
if strcmp(normamp,'yes'); ylabel ('Normalised Amplitude','fontsize',14)
elseif strcmp(normamp,'no'); ylabel ('Amplitude','fontsize',14);end
title ('Frequency domain - absolute','fontsize',14)
% Frequency domain - real
subplot(235)
hold on
plot(freqs,real(AB_true_f_comp),'k','linewidth',2);
plot(freqs,real(AB_int_f),'r--','linewidth',2);
plot(freqs,real(AB_int_f_FD),'g-.','linewidth',2);
axis([0 maxfmod -amaxf amaxf])
box on
set(gca,'fontsize',14)
xlabel ('Frequency (Hz)','fontsize',14)
if strcmp(normamp,'yes'); ylabel ('Normalised Amplitude','fontsize',14)
elseif strcmp(normamp,'no'); ylabel ('Amplitude','fontsize',14);end
title ('Frequency domain - real','fontsize',14)
% Frequency domain - imaginary
subplot(236)
hold on
plot(freqs,imag(AB_true_f_comp),'k','linewidth',2);
plot(freqs,imag(AB_int_f),'r--','linewidth',2);
plot(freqs,imag(AB_int_f_FD),'g-.','linewidth',2);
axis([0 maxfmod -amaxf amaxf])
box on
set(gca,'fontsize',14)
xlabel ('Frequency (Hz)','fontsize',14)
if strcmp(normamp,'yes'); ylabel ('Normalised Amplitude','fontsize',14)
elseif strcmp(normamp,'no'); ylabel ('Amplitude','fontsize',14);end
title ('Frequency domain - imaginary','fontsize',14)


% Plot geometry
fig4=figure;
fig4w=500;
fig4h=300;
pos = get(fig3,'Position');
set(fig4,'Position',[pos(1)+leftb scrnsz(4)-(4*topb+fig4h) fig4w fig4h])
% Plot
hold on
plot(srx(1:6:end,1),srx(1:6:end,2),'r*','markersize',8);
plot(rcx(:,1),rcx(:,2),'bv','markersize',8,'markerfacecolor','b');
axis equal
axis([min(srx(:,1)-50) max(srx(:,1)+50) min(srx(:,2)-50) max(srx(:,2)+50)]);
box on
set(gca,'fontsize',14,'XTick',-500:250:500,'YTick',-500:250:500)
xlabel ('x (m)','fontsize',14)
ylabel ('y (m)','fontsize',14)
%title ('Geometry','fontsize',14)
text(rcx(1,1),rcx(1,2)-80,'A','fontsize',14,'horizontalalignment','center')
text(rcx(2,1),rcx(2,2)-80,'B','fontsize',14,'horizontalalignment','center')


% Plot results: true GF and interferometric GFs 
% Time domain, causal part only and zoomed-in areas
fig5=figure;
fig5w=1200;
fig5h=600;
pos = get(fig4,'Position');
set(fig5,'Position',[pos(1)+leftb scrnsz(4)-(5*topb+fig5h) fig5w fig5h])
% Complete traces
axes('position',[0.1 0.1 0.8 0.8]);
sca=0.8/(2*amaxt);
sct=0.8/(0.5*max(time2));
hold on
foldy=plot(time1,AB_true_t,'k','linewidth',2);
fd=plot(time1,AB_true_t_FD,'m-.','linewidth',2);
axis([0 0.5*max(time2) -amaxt amaxt])
box on
set(gca,'fontsize',18)
xlabel ('Time (s)','fontsize',18)
if strcmp(normamp,'yes'); ylabel ('Normalised amplitude (pressure)','fontsize',18)
elseif strcmp(normamp,'no'); ylabel ('Amplitude (pressure)','fontsize',18);end
leg=legend([foldy fd],'True trace (Foldy)','True trace (FD)','location','northeast');
set(leg,'fontsize',16,'position',[0.74 0.76 0.15 0.1])
% Zoom 1
z1ax=[0.243 0.256 0.145*amaxt 0.175*amaxt];
z1=axes('position',[0.165 0.6 (z1ax(2)-z1ax(1))*sct*15 (z1ax(4)-z1ax(3))*sca*15]);
hold on
plot(time1,AB_true_t,'k','linewidth',2);
plot(time1,AB_true_t_FD,'m-.','linewidth',2);
axis(z1ax)
box on
set(z1,'fontsize',14)
% xlabel ('Time (s)','fontsize',10)
% if strcmp(normamp,'yes'); ylabel ('Normalised Amplitude','fontsize',10)
% elseif strcmp(normamp,'no'); ylabel ('Amplitude','fontsize',10);end
% Zoom 2
z2ax=[0.3421 0.3458 0.991*amaxt 1.0005*amaxt];
z2=axes('position',[0.425 0.67 (z2ax(2)-z2ax(1))*sct*50 (z2ax(4)-z2ax(3))*sca*50]);
hold on
plot(time1,AB_true_t,'k','linewidth',2);
plot(time1,AB_true_t_FD,'m-.','linewidth',2);
axis(z2ax)
box on
set(z2,'fontsize',14)
% xlabel ('Time (s)','fontsize',10)
% if strcmp(normamp,'yes'); ylabel ('Normalised Amplitude','fontsize',10)
% elseif strcmp(normamp,'no'); ylabel ('Amplitude','fontsize',10);end
% Zoom 3
z3ax=[0.295 0.303 -0.768*amaxt -0.75*amaxt];
z3=axes('position',[0.152 0.148 (z3ax(2)-z3ax(1))*sct*25 (z3ax(4)-z3ax(3))*sca*25]);
hold on
plot(time1,AB_true_t,'k','linewidth',2);
plot(time1,AB_true_t_FD,'m-.','linewidth',2);
axis(z3ax)
box on
set(z3,'fontsize',14)
% xlabel ('Time (s)','fontsize',10)
% if strcmp(normamp,'yes'); ylabel ('Normalised Amplitude','fontsize',10)
% elseif strcmp(normamp,'no'); ylabel ('Amplitude','fontsize',10);end
% Zoom 4
z4ax=[0.387 0.393 -0.445*amaxt -0.432*amaxt];
z4=axes('position',[0.42 0.148 (z4ax(2)-z4ax(1))*sct*35 (z4ax(4)-z4ax(3))*sca*35]);
hold on
plot(time1,AB_true_t,'k','linewidth',2);
plot(time1,AB_true_t_FD,'m-.','linewidth',2);
axis(z4ax)
box on
set(z4,'fontsize',14)
% xlabel ('Time (s)','fontsize',10)
% if strcmp(normamp,'yes'); ylabel ('Normalised Amplitude','fontsize',10)
% elseif strcmp(normamp,'no'); ylabel ('Amplitude','fontsize',10);end


% Plot results: true GF and interferometric GFs 
% Time domain, causal part only and zoomed-in areas
fig6=figure;
fig6w=1200;
fig6h=600;
pos = get(fig5,'Position');
set(fig6,'Position',[pos(1)+leftb scrnsz(4)-(6*topb+fig6h) fig6w fig6h])
% Complete traces
main=axes('position',[0.1 0.1 0.8 0.8]);
sca=0.8/(2*amaxt);
sct=0.8/(0.5*max(time2));
hold on
true=plot(time2,AB_true_t_2sided_comp,'k','linewidth',2);
int=plot(time2,AB_int_t_2sided,'r--','linewidth',2);
int2=plot(time2,AB_int_t_2sided_FD,'g-.','linewidth',2);
axis([0 0.5*max(time2) -amaxt amaxt])
box on
set(gca,'fontsize',18)
xlabel ('Time (s)','fontsize',18)
if strcmp(normamp,'yes'); ylabel ('Normalised amplitude (pressure)','fontsize',18)
elseif strcmp(normamp,'no'); ylabel ('Amplitude (pressure)','fontsize',18);end
leg=legend([true int int2],'True trace (Foldy)','Interferometric trace (Foldy)','Interferometric trace (FD)','location','northeast');
set(leg,'fontsize',16,'position',[0.64 0.76 0.25 0.1])
% Zoom 1
z1ax=[0.243 0.256 0.145*amaxt 0.175*amaxt];
z1=axes('position',[0.165 0.6 (z1ax(2)-z1ax(1))*sct*15 (z1ax(4)-z1ax(3))*sca*15]);
hold on
plot(time2,AB_true_t_2sided_comp,'k','linewidth',2);
plot(time2,AB_int_t_2sided,'r--','linewidth',2);
plot(time2,AB_int_t_2sided_FD,'g-.','linewidth',2);
axis(z1ax)
box on
set(z1,'fontsize',14)
% xlabel ('Time (s)','fontsize',10)
% if strcmp(normamp,'yes'); ylabel ('Normalised Amplitude','fontsize',10)
% elseif strcmp(normamp,'no'); ylabel ('Amplitude','fontsize',10);end
% Zoom 2
z2ax=[0.3421 0.3458 0.991*amaxt 1.0005*amaxt];
z2=axes('position',[0.425 0.67 (z2ax(2)-z2ax(1))*sct*50 (z2ax(4)-z2ax(3))*sca*50]);
hold on
plot(time2,AB_true_t_2sided_comp,'k','linewidth',2);
plot(time2,AB_int_t_2sided,'r--','linewidth',2);
plot(time2,AB_int_t_2sided_FD,'g-.','linewidth',2);
axis(z2ax)
box on
set(z2,'fontsize',14)
% xlabel ('Time (s)','fontsize',10)
% if strcmp(normamp,'yes'); ylabel ('Normalised Amplitude','fontsize',10)
% elseif strcmp(normamp,'no'); ylabel ('Amplitude','fontsize',10);end
% Zoom 3
z3ax=[0.295 0.303 -0.768*amaxt -0.75*amaxt];
z3=axes('position',[0.152 0.148 (z3ax(2)-z3ax(1))*sct*25 (z3ax(4)-z3ax(3))*sca*25]);
hold on
plot(time2,AB_true_t_2sided_comp,'k','linewidth',2);
plot(time2,AB_int_t_2sided,'r--','linewidth',2);
plot(time2,AB_int_t_2sided_FD,'g-.','linewidth',2);
axis(z3ax)
box on
set(z3,'fontsize',14)
% xlabel ('Time (s)','fontsize',10)
% if strcmp(normamp,'yes'); ylabel ('Normalised Amplitude','fontsize',10)
% elseif strcmp(normamp,'no'); ylabel ('Amplitude','fontsize',10);end
% Zoom 4
z4ax=[0.387 0.393 -0.445*amaxt -0.432*amaxt];
z4=axes('position',[0.42 0.148 (z4ax(2)-z4ax(1))*sct*35 (z4ax(4)-z4ax(3))*sca*35]);
hold on
plot(time2,AB_true_t_2sided_comp,'k','linewidth',2);
plot(time2,AB_int_t_2sided,'r--','linewidth',2);
plot(time2,AB_int_t_2sided_FD,'g-.','linewidth',2);
axis(z4ax)
box on
set(z4,'fontsize',14)
% xlabel ('Time (s)','fontsize',10)
% if strcmp(normamp,'yes'); ylabel ('Normalised Amplitude','fontsize',10)
% elseif strcmp(normamp,'no'); ylabel ('Amplitude','fontsize',10);end



%% %%% Figures in GEOPHYSICS format %%% %%

% Plot geometry
fig7=figure;
fig7w=3.33;
fig7h=3.33;
set(fig7,'units','inches','Pos',[1 1 fig7w fig7h])
% Plot
hold on
src=plot(srx(1:6:end,1),srx(1:6:end,2),'k*','markersize',6);
rec=plot(rcx(:,1),rcx(:,2),'kv','markersize',6,'markerfacecolor','k');
axis equal
axis([min(srx(:,1)-50) max(srx(:,1)+50) min(srx(:,2)-50) max(srx(:,2)+50)])
box on
set(gca,'fontsize',8,'XTick',-500:250:500,'YTick',-500:250:500)
xlabel ('x (m)','fontsize',8)
ylabel ('y (m)','fontsize',8)
text(rcx(1,1),rcx(1,2)-50,'A','fontsize',8,'FontAngle','italic','horizontalalignment','center')
text(rcx(2,1),rcx(2,2)-50,'B','fontsize',8,'FontAngle','italic','horizontalalignment','center')

% Plot results: true GF and interferometric GFs 
% Time domain, causal part only and zoomed-in areas
fig8=figure;
fig8w=4.33;
fig8h=6;
set(fig8,'units','inches','Pos',[5 1 fig8w fig8h])
% Complete traces
main1=axes('position',[0.1 0.1 0.8 0.8]);
sca=0.8/(2*amaxt);
sct=0.8/(0.5*max(time2));
hold on
set(gca,'ColorOrder',[0 0 0;0.4 0.4 0.4])
hold all
foldy=plot(time1,AB_true_t,'-','linewidth',2);
fd=plot(time1,AB_true_t_FD,'-.','linewidth',2);
axis([0 0.4*max(time2) -amaxt amaxt])
box on
set(gca,'fontsize',8)
%%%%% Following 8 lines adapted from http://www.mathworks.fr/matlabcentral/newsreader/view_thread/112386
oldLabels = str2num(get(gca,'YTickLabel'));
exp = 10^2;
newLabels = num2str(oldLabels*exp);
set(gca,'YTickLabel',newLabels,'units','normalized');
posAxes = get(gca,'position');
textBox = annotation('textbox','linestyle','none','margin',1,'string',['x 10^{' sprintf('%d',log10(1./exp)) '}']);
posAn = get(textBox,'position');
set(textBox,'fontsize',8,'position',[posAxes(1) posAxes(2)+posAxes(4) posAn(3) posAn(4)],'VerticalAlignment','bottom','HorizontalAlignment','left');
%%%%%
xlabel ('Time (s)','fontsize',8)
if strcmp(normamp,'yes'); ylabel ('Normalised amplitude (pressure)','fontsize',8)
elseif strcmp(normamp,'no'); ylabel ('Amplitude (pressure)','fontsize',8);end
leg1=legend([foldy fd],'True trace (Foldy)','True trace (FD)','location','southeast');
set(leg1,'fontsize',8)
% Zoom 1
z1ax=[0.243 0.256 0.155*amaxt 0.173*amaxt];
z1=axes('position',[0.18 0.6 (z1ax(2)-z1ax(1))*sct*20 (z1ax(4)-z1ax(3))*sca*20]);
set(gca,'ColorOrder',[0 0 0;0.4 0.4 0.4])
hold all
plot(time1,AB_true_t,'-',time1,AB_true_t_FD,'-.','linewidth',2);
axis(z1ax)
box on
set(z1,'fontsize',8,'XTick',[0.245;0.255],'XTickLabel',[0.245;0.255])
%%%%% Following 8 lines adapted from http://www.mathworks.fr/matlabcentral/newsreader/view_thread/112386
oldLabels = str2num(get(gca,'YTickLabel'));
exp = 10^3;
newLabels = num2str(oldLabels);
set(gca,'YTickLabel',newLabels,'units','normalized');
posAxes = get(gca,'position');
textBox = annotation('textbox','linestyle','none','margin',1,'string',['x 10^{' sprintf('%d',log10(1./exp)) '}']);
posAn = get(textBox,'position');
set(textBox,'fontsize',8,'position',[posAxes(1) posAxes(2)+posAxes(4) posAn(3) posAn(4)],'VerticalAlignment','bottom','HorizontalAlignment','left');
%%%%%
% Zoom 2
z2ax=[0.342 0.346 0.988*amaxt 1.0005*amaxt];
z2=axes('position',[0.53 0.615 (z2ax(2)-z2ax(1))*sct*50 (z2ax(4)-z2ax(3))*sca*50]);
set(gca,'ColorOrder',[0 0 0;0.4 0.4 0.4])
hold all
plot(time1,AB_true_t,'-',time1,AB_true_t_FD,'-.','linewidth',2);
axis(z2ax)
box on
set(z2,'fontsize',8,'XTick',[0.342;0.346],'XTickLabel',[0.342;0.346],'YTick',[0.0198;0.0199;0.02],'YTickLabel',[0.0198;0.0199;0.02])
%%%%% Following 8 lines adapted from http://www.mathworks.fr/matlabcentral/newsreader/view_thread/112386
oldLabels = str2num(get(gca,'YTickLabel'));
exp = 10^2;
newLabels = num2str(oldLabels*exp);
set(gca,'YTickLabel',newLabels,'units','normalized');
posAxes = get(gca,'position');
textBox = annotation('textbox','linestyle','none','margin',1,'string',['x 10^{' sprintf('%d',log10(1./exp)) '}']);
posAn = get(textBox,'position');
set(textBox,'fontsize',8,'position',[posAxes(1) posAxes(2)+posAxes(4) posAn(3) posAn(4)],'VerticalAlignment','bottom','HorizontalAlignment','left');
%%%%%
% Zoom 3
z3ax=[0.296 0.302 -0.767*amaxt -0.754*amaxt];
z3=axes('position',[0.185 0.16 (z3ax(2)-z3ax(1))*sct*35 (z3ax(4)-z3ax(3))*sca*35]);
set(gca,'ColorOrder',[0 0 0;0.4 0.4 0.4])
hold all
plot(time1,AB_true_t,'-',time1,AB_true_t_FD,'-.','linewidth',2);
axis(z3ax)
box on
set(z3,'fontsize',8,'XTick',[0.296;0.302],'XTickLabel',[0.296;0.302])
%%%%% Following 8 lines adapted from http://www.mathworks.fr/matlabcentral/newsreader/view_thread/112386
oldLabels = str2num(get(gca,'YTickLabel'));
exp = 10^2;
newLabels = num2str(oldLabels*exp);
set(gca,'YTickLabel',newLabels,'units','normalized');
posAxes = get(gca,'position');
textBox = annotation('textbox','linestyle','none','margin',1,'string',['x 10^{' sprintf('%d',log10(1./exp)) '}']);
posAn = get(textBox,'position');
set(textBox,'fontsize',8,'position',[posAxes(1) posAxes(2)+posAxes(4) posAn(3) posAn(4)],'VerticalAlignment','bottom','HorizontalAlignment','left');
%%%%%
% Zoom 4
z4ax=[0.387 0.393 -0.445*amaxt -0.432*amaxt];
z4=axes('position',[0.56 0.23 (z4ax(2)-z4ax(1))*sct*35 (z4ax(4)-z4ax(3))*sca*35]);
set(gca,'ColorOrder',[0 0 0;0.4 0.4 0.4])
hold all
plot(time1,AB_true_t,'-',time1,AB_true_t_FD,'-.','linewidth',2);
axis(z4ax)
box on
set(z4,'fontsize',8,'XTick',[0.388;0.392],'XTickLabel',[0.388;0.392])
%%%%% Following 8 lines adapted from http://www.mathworks.fr/matlabcentral/newsreader/view_thread/112386
oldLabels = str2num(get(gca,'YTickLabel'));
exp = 10^3;
newLabels = num2str(oldLabels);
set(gca,'YTickLabel',newLabels,'units','normalized');
posAxes = get(gca,'position');
textBox = annotation('textbox','linestyle','none','margin',1,'string',['x 10^{' sprintf('%d',log10(1./exp)) '}']);
posAn = get(textBox,'position');
set(textBox,'fontsize',8,'position',[posAxes(1) posAxes(2)+posAxes(4) posAn(3) posAn(4)],'VerticalAlignment','bottom','HorizontalAlignment','left');
%%%%%


% Plot results: true GF and interferometric GFs 
% Time domain, causal part only and zoomed-in areas
fig9=figure;
fig9w=4.33;
fig9h=6;
set(fig9,'units','inches','Pos',[10 1 fig9w fig9h])
% Complete traces
main2=axes('position',[0.1 0.1 0.8 0.8]);
sca=0.8/(2*amaxt);
sct=0.8/(0.5*max(time2));
hold on
set(gca,'ColorOrder',[0 0 0;0.6 0.6 0.6;0.2 0.2 0.2])
hold all
true=plot(time2,AB_true_t_2sided_comp,'-','linewidth',2);
int=plot(time2,AB_int_t_2sided,'--','linewidth',2);
int2=plot(time2,AB_int_t_2sided_FD,'-.','linewidth',2);
axis([0 0.4*max(time2) -amaxt amaxt])
box on
set(gca,'fontsize',8)
%%%%% Following 8 lines adapted from http://www.mathworks.fr/matlabcentral/newsreader/view_thread/112386
oldLabels = str2num(get(gca,'YTickLabel'));
exp = 10^2;
newLabels = num2str(oldLabels*exp);
set(gca,'YTickLabel',newLabels,'units','normalized');
posAxes = get(gca,'position');
textBox = annotation('textbox','linestyle','none','margin',1,'string',['x 10^{' sprintf('%d',log10(1./exp)) '}']);
posAn = get(textBox,'position');
set(textBox,'fontsize',8,'position',[posAxes(1) posAxes(2)+posAxes(4) posAn(3) posAn(4)],'VerticalAlignment','bottom','HorizontalAlignment','left');
%%%%%
xlabel ('Time (s)','fontsize',8)
if strcmp(normamp,'yes'); ylabel ('Normalised amplitude (pressure)','fontsize',8)
elseif strcmp(normamp,'no'); ylabel ('Amplitude (pressure)','fontsize',8);end
leg2=legend([true int int2],'True trace (Foldy)','Interferometric trace (Foldy)','Interferometric trace (FD)','location','southeast');
set(leg2,'fontsize',8)
% Zoom 1
z1ax=[0.243 0.256 0.155*amaxt 0.173*amaxt];
z1=axes('position',[0.18 0.6 (z1ax(2)-z1ax(1))*sct*20 (z1ax(4)-z1ax(3))*sca*20]);
set(gca,'ColorOrder',[0 0 0;0.6 0.6 0.6;0.2 0.2 0.2])
hold all
plot(time2,AB_true_t_2sided_comp,'-',time2,AB_int_t_2sided,'--',time2,AB_int_t_2sided_FD,'-.','linewidth',2);
axis(z1ax)
box on
set(z1,'fontsize',8,'XTick',[0.245;0.255],'XTickLabel',[0.245;0.255])
%%%%% Following 8 lines adapted from http://www.mathworks.fr/matlabcentral/newsreader/view_thread/112386
oldLabels = str2num(get(gca,'YTickLabel'));
exp = 10^3;
newLabels = num2str(oldLabels);
set(gca,'YTickLabel',newLabels,'units','normalized');
posAxes = get(gca,'position');
textBox = annotation('textbox','linestyle','none','margin',1,'string',['x 10^{' sprintf('%d',log10(1./exp)) '}']);
posAn = get(textBox,'position');
set(textBox,'fontsize',8,'position',[posAxes(1) posAxes(2)+posAxes(4) posAn(3) posAn(4)],'VerticalAlignment','bottom','HorizontalAlignment','left');
%%%%%
% Zoom 2
z2ax=[0.342 0.346 0.988*amaxt 1.0005*amaxt];
z2=axes('position',[0.53 0.615 (z2ax(2)-z2ax(1))*sct*50 (z2ax(4)-z2ax(3))*sca*50]);
set(gca,'ColorOrder',[0 0 0;0.6 0.6 0.6;0.2 0.2 0.2])
hold all
plot(time2,AB_true_t_2sided_comp,'-',time2,AB_int_t_2sided,'--',time2,AB_int_t_2sided_FD,'-.','linewidth',2);
axis(z2ax)
box on
set(z2,'fontsize',8,'XTick',[0.342;0.346],'XTickLabel',[0.342;0.346],'YTick',[0.0198;0.0199;0.02],'YTickLabel',[0.0198;0.0199;0.02])
%%%%% Following 8 lines adapted from http://www.mathworks.fr/matlabcentral/newsreader/view_thread/112386
oldLabels = str2num(get(gca,'YTickLabel'));
exp = 10^2;
newLabels = num2str(oldLabels*exp);
set(gca,'YTickLabel',newLabels,'units','normalized');
posAxes = get(gca,'position');
textBox = annotation('textbox','linestyle','none','margin',1,'string',['x 10^{' sprintf('%d',log10(1./exp)) '}']);
posAn = get(textBox,'position');
set(textBox,'fontsize',8,'position',[posAxes(1) posAxes(2)+posAxes(4) posAn(3) posAn(4)],'VerticalAlignment','bottom','HorizontalAlignment','left');
%%%%%
% Zoom 3
z3ax=[0.296 0.302 -0.767*amaxt -0.754*amaxt];
z3=axes('position',[0.185 0.16 (z3ax(2)-z3ax(1))*sct*35 (z3ax(4)-z3ax(3))*sca*35]);
set(gca,'ColorOrder',[0 0 0;0.6 0.6 0.6;0.2 0.2 0.2])
hold all
plot(time2,AB_true_t_2sided_comp,'-',time2,AB_int_t_2sided,'--',time2,AB_int_t_2sided_FD,'-.','linewidth',2);
axis(z3ax)
box on
set(z3,'fontsize',8,'XTick',[0.296;0.302],'XTickLabel',[0.296;0.302])
%%%%% Following 8 lines adapted from http://www.mathworks.fr/matlabcentral/newsreader/view_thread/112386
oldLabels = str2num(get(gca,'YTickLabel'));
exp = 10^2;
newLabels = num2str(oldLabels*exp);
set(gca,'YTickLabel',newLabels,'units','normalized');
posAxes = get(gca,'position');
textBox = annotation('textbox','linestyle','none','margin',1,'string',['x 10^{' sprintf('%d',log10(1./exp)) '}']);
posAn = get(textBox,'position');
set(textBox,'fontsize',8,'position',[posAxes(1) posAxes(2)+posAxes(4) posAn(3) posAn(4)],'VerticalAlignment','bottom','HorizontalAlignment','left');
%%%%%
% Zoom 4
z4ax=[0.387 0.393 -0.445*amaxt -0.432*amaxt];
z4=axes('position',[0.56 0.23 (z4ax(2)-z4ax(1))*sct*35 (z4ax(4)-z4ax(3))*sca*35]);
set(gca,'ColorOrder',[0 0 0;0.6 0.6 0.6;0.2 0.2 0.2])
hold all
plot(time2,AB_true_t_2sided_comp,'-',time2,AB_int_t_2sided,'--',time2,AB_int_t_2sided_FD,'-.','linewidth',2);
axis(z4ax)
box on
set(z4,'fontsize',8,'XTick',[0.388;0.392],'XTickLabel',[0.388;0.392])
%%%%% Following 8 lines adapted from http://www.mathworks.fr/matlabcentral/newsreader/view_thread/112386
oldLabels = str2num(get(gca,'YTickLabel'));
exp = 10^3;
newLabels = num2str(oldLabels);
set(gca,'YTickLabel',newLabels,'units','normalized');
posAxes = get(gca,'position');
textBox = annotation('textbox','linestyle','none','margin',1,'string',['x 10^{' sprintf('%d',log10(1./exp)) '}']);
posAn = get(textBox,'position');
set(textBox,'fontsize',8,'position',[posAxes(1) posAxes(2)+posAxes(4) posAn(3) posAn(4)],'VerticalAlignment','bottom','HorizontalAlignment','left');
%%%%%

set(main1,'position',get(main2,'position'))
