%% Acoustic modelling and interferometry example - 2D

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% This code is distributed as part of the source-code package 
%                   acoustic_modelling_foldy_v.4.1_seg 
% that accompanies Galetti et al. (2013). The package can be downloaded 
% from the Geophysics source-code archive at 
%                   http://software.seg.org/2013/0003
% Use of this code is subject to acceptance of the terms and conditions
% that can be found at http://software.seg.org/disclaimer.txt 
% Copyright 2013 by The Society of Exploration Geophysicists (SEG)
% Reference:
% Galetti, E., Halliday, D., Curtis, A. (2013) A simple and exact acoustic 
% wavefield modelling code for data processing, imaging and interferometry 
% applications. Geophysics.
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% This script serves as an example of how the acoustic GF modelling code
% can be used to model acoustic wavefields from a boundary of sources to
% two enclosed receivers, and how to apply interferometry to construct
% the GF signal between them. In addition, it compares the results 
% obtained from full interferomery (i.e. using both monopole and dipole
% Green's functions) with those obtained from monopole interferometry
% (i.e. using monopole Green's functions only).
% A circular boundary of sources is used for interferometry. 
% Cross-correlation is performed in the frequency domain (using matrix 
% multiplication one frequency at a time). The results match the true 
% Green's functions obtained by direct modelling between the two
% receivers.
%
% Figure 1: Geometry (top)
%           True modelled trace (bottom)
% Figure 2: Two-sided Green's functions in time domain:
%               black solid trace - true Green's function 
%               purple dashed trace - interferometric Green's function
%               (interferometry with monopoles only)
%               green dot-dashed trace - interferometric Green's function
%               (interferometry with monopoles and dipoles)
% Figure 3: Comparison: 'true' Green's function (black solid trace -
%                       obtained from the causal part of the true Green's 
%                       function in Figure 2 according to the 
%                       interferometry formulae) vs. interferometric
%                       Green's functions (purple dashed trace - monopoles
%                       only; green dot-dashed trace - monopoles and
%                       dipoles)
%                       top:            traces in time domain
%                       bottom left:    frequency spectra 
%                       bottom centre:  real part of frequency spectra 
%                       bottom right:   imaginary part of frequency spectra
%%% FIGURES IN PAPER %%%
% Figure 4,6: Geometry
% Figure 5,7: Comparison: causal part of 'true' Green's function (see above)
%                         vs. interferometric Green's functions (see above),
%                         as full trace and zoomed-in panels.
%
%
% References:
%       Groenenboom, J., Snieder, R. (1995) Attenuation, dispersion, and
%   anisotropy by multiple scattering of transmitted waves through
%   distributions of scatterers. J. Acoust. Soc. Am. 98 (6): 3482-3492.
%       Snieder, R. (1999) Imaging and averaging in complex media, in
%   Diffuse waves in complex media, edited by J.P. Fouque, pp. 405-454,
%   Kluwer, Dordrecht
%       Snieder, R. (2009) A Guided Tour of Mathematical Methods for the
%   Physical Sciences, 2nd ed., Cambridge University Press.
%       van Manen, D.-J., Robertsson, J.O.A., Curtis, A. (2005) Modeling of 
%   Wave Propagation in Inhomogeneous Media. Physical Review Letters, 94: 
%   164301.
%       Wapenaar, K., Fokkema, J. (2006) Green's function representations  
%   for seismic interferometry, Geophysics, 71: SI33-SI46.
%
%
% Required:
% makecircbound.m           - creates a circular boundary of points with a
%                             certain radius or point spacing
% do_xcorr_int.m            - performs cross-correlational interferometry
% acoustic_p_gfs_direct_f.m - models direct wave between two points in an
%                             acoustic medium in the frequency domain only
% model_gfs_p_scat_acoust.m - models scattered waves in an acoustic medium
% centerfreqs.m             - creates a two-sided centred frequency
%                             spectrum from positive frequencies
% rickerwavl_time.m         - creates Ricker wavelet for source
%
%
% Erica Galetti, July 2012
% Contact: erica.galetti@ed.ac.uk


clear all
close all



%% van Manen or Wapenaar?
method = 'WFvi';    
    % 'VMvi'    --- van Manen et al. (2005)
    %                   volume injection source
    % 'WFvir'   --- Wapenaar & Fokkema (2006)
    %                   volume injection rate source
    % 'WFvi'    --- Wapenaar & Fokkema (2006)
    %                   volume injection source
    % 'VMvir'   --- Equivalent to van Manen et al. (2005) 
    %                   with a volume injection rate source          

    
%% Include scatterers?
scattering = 'yes'; % 'yes' or 'no'

% Define scatterer locations
if strcmp(scattering,'yes')
    scats = [-50,-150,0;280,50,0;-250,200,0];
    scats(:,4) = -4; % Imaginary part of the scattering amplitude - has to be within the range [-4,0] in 2D --> see Groenenboom & Snieder (1995) or Snieder (1999)
end


%% Normalise amplitudes?
normamp = 'no'; % 'yes' or 'no'


%% Source wavelet type
wavltype = 'wavly_pos'; % the positive-time wavelet is not zero-phase - for an example with zero-phase wavelet, see the source-receiver interferometry example


%% Type of source
if strcmp(method,'WFvir')
    srctype='volinjratepos';
elseif strcmp(method,'WFvi')
    srctype='volinjpos';
elseif strcmp(method,'VMvir')
    srctype='volinjrateneg';
elseif strcmp(method,'VMvi')
    srctype='volinjneg';
end


%% Define a series of input parameters

nf = 200;
nf = 2^(nextpow2(nf)); % round number of frequencies to the nearest power of 2 (makes fft/ifft run faster)
maxf = 100; % max frequency
fc = 30; % central frequency of Ricker wavelet
dt = 1/(2*maxf); % time step

ro = 1; % medium density
c = 1000; % constant background velocity
nyq = c/maxf; % minimum Nyquist wavelength


%% Type of geometry
dim = '2d';
if strcmp(dim,'1d'); ndim = 1;
elseif strcmp(dim,'2d') || strcmp(dim,'2d_far'); ndim = 2;
elseif strcmp(dim,'3d'); ndim = 3;
end


%% Define source and receiver positions

% Define source boundary
% Points on circle
rad_s = 500; % radius of source boundary (gets adjusted depending on source spacing 4 lines below)
dx_s = nyq/2; % spacing of sources on boundary is half the minimum Nyquist wavelength
[coords,radnew] = makecircbound(rad_s,dx_s,'fixds',0,0);
rad_s = radnew;
nsrx = size(coords,1);
srx(1:nsrx,1) = coords(:,1); % boundary x-coordinate
srx(1:nsrx,2) = coords(:,2); % boundary y-coordinate
srx(1:nsrx,3) = 0; % boundary z-coordinate
% Calculate distance between sources (offsets_s) and ds
offsets_s = zeros(nsrx,1);
ds = zeros(nsrx,1);
offsets_s(1:nsrx) = sqrt(diff(srx([1:nsrx 1],1)).^2 + diff(srx([1:nsrx 1],2)).^2);
ds(1:nsrx) = offsets_s(1:nsrx)/2 + offsets_s([nsrx 1:nsrx-1])/2;
clear offsets_s coords radnew

% Calculate boundary normal (the boundary is a circle centred at the
% origin, so its x and y components are simply the cos and sin of the
% angles between the source-position-vector and the x axis).
boundnorm = zeros(nsrx,ndim);
for dd = 1:ndim; boundnorm(:,dd) = srx(:,dd)/rad_s; end


% Define position of receivers
rcA=[-150,0,0];
rcB=[50,40,0];
rcx = [rcA;rcB];
nrcx = size(rcx,1);



%% Model the real Green's function between A and B
if strcmp(scattering,'yes')
    [time1,gftrue_mon_p_t,~,wavelet_t] = model_gfs_p_scat_acoust(rcx,rcx,maxf,nf,c,dim,srctype,'srcdip','scaty',wavltype,'freqn','timey','tot',scats,fc);
elseif strcmp(scattering,'no')
    [time1,gftrue_mon_p_t,~,wavelet_t] = model_gfs_p_scat_acoust(rcx,rcx,maxf,nf,c,dim,srctype,'srcdip','scatn',wavltype,'freqn','timey','tot',fc);
end

% Define two-sided time vector
time2 = [flipud(-time1(2:end));time1(1:end)]; % negative and positive times

% Wavelet in frequency domain
wavelet_f = fft([wavelet_t;zeros(size(wavelet_t))])./sqrt(2*2*nf);
wavelet_f = wavelet_f(2:2*nf+1,:,:); % take only half of the frequency spectrum

% Frequencies
df = maxf/(2*nf);
freq2 =  (1:2*nf).*df;
freqs=[0;freq2.'];
om = 2*pi.*freq2; % angular frequencies



%% Model the boundary to receiver-network Green's functions using the model_gfs_p_scat_acoust function

% mgfs_t = monopole source, time domain
% dgfs_t = dipole source, time domain

% Allocate space
mgfs_t=zeros(2*nf,nrcx,nsrx);
dgfs_t=zeros(2*nf,nrcx,nsrx);


% Loop over boundary sources

for ss = 1:nsrx
    
    if strcmp(scattering,'yes')
        
        % Model pressure from source ss to all receivers
        [~,gf_mon_p_t,gf_dip_p_t,~] = model_gfs_p_scat_acoust(srx(ss,:),rcx,maxf,nf,c,dim,srctype,'srcdip','scaty',wavltype,'freqn','timey','tot',scats,fc);
        mgfs_t(:,:,ss) = gf_mon_p_t; % monopole wavefield - time domain
        for dd = 1:ndim; dgfs_t(:,:,ss) = dgfs_t(:,:,ss) + gf_dip_p_t(:,:,1,dd).*boundnorm(ss,dd); end % weight dipole components by the outward normal components - this gives the full dipole wavefield        
        
    elseif strcmp(scattering,'no')
        
        % Model pressure from source ss to all receivers
        [~,gf_mon_p_t,gf_dip_p_t,~] = model_gfs_p_scat_acoust(srx(ss,:),rcx,maxf,nf,c,dim,srctype,'srcdip','scatn',wavltype,'freqn','timey','tot',fc);
        mgfs_t(:,:,ss) = gf_mon_p_t; % monopole wavefield - time domain
        for dd = 1:ndim; dgfs_t(:,:,ss) = dgfs_t(:,:,ss) + gf_dip_p_t(:,:,1,dd).*boundnorm(ss,dd); end % weight dipole components by the outward normal components - this gives the full dipole wavefield 
	
    end
    
    if mod(ss,20)==0; fprintf(['Done source ' int2str(ss) ' of ' int2str(nsrx) '.\n']); end
    
    clear gf_mon_p_t gf_dip_p_t
    
end

% Apply Tukey window to taper trace edges (useful in case any scattered
% arrivals occur at the end of the trace, as the traces need to be padded
% with zeros before moving to the frequency domain for cross-correlation)
% and pad with zeros to get traces twice as long.

% Construct Tukey window (this assumes the Signal Processing Toolbox is not
% available and avoids calling the tukeywin function, giving similar results)
ntap = 10; % number of time samples to taper on each side of the trace (make sure it is much less than half the total number of time samples!)
theta_t = linspace(0,1,ntap+1).';
taper = (1+cos(pi.*theta_t))./2;
% Create Tukey window as taper-ones-taper
tuk = [taper(end:-1:1);ones(2*nf-2*(ntap+1),1,1);taper];
clear ntap theta_t taper

gftrue_mon_p_t = [gftrue_mon_p_t.*repmat(tuk,[1 size(gftrue_mon_p_t,2) size(gftrue_mon_p_t,3)]);zeros(2*nf,size(gftrue_mon_p_t,2),size(gftrue_mon_p_t,3))];
mgfs_t = [mgfs_t.*repmat(tuk,[1 size(mgfs_t,2) size(mgfs_t,3)]);zeros(2*nf,size(mgfs_t,2),size(mgfs_t,3))];
dgfs_t = [dgfs_t.*repmat(tuk,[1 size(dgfs_t,2) size(dgfs_t,3)]);zeros(2*nf,size(dgfs_t,2),size(dgfs_t,3))];

% Move to frequency domain
gftrue_mon_p_f = fft(gftrue_mon_p_t,[],1)./sqrt(2*2*nf);
clear gftrue_mon_p_t
gftrue_mon_p_f = gftrue_mon_p_f(2:2*nf+1,:,:); % take only half of the frequency spectrum
mgfs_f = fft(mgfs_t,[],1)./sqrt(2*2*nf);
clear mgfs_t
mgfs_f = mgfs_f(2:2*nf+1,:,:); % take only half of the frequency spectrum
dgfs_f = fft(dgfs_t,[],1)./sqrt(2*2*nf);
clear dgfs_t
dgfs_f = dgfs_f(2:2*nf+1,:,:); % take only half of the frequency spectrum


%% Perform xcorr interferometry from boundary S of sources in frequency domain using matrix multiplication one frequency at a time
% Use function do_xcorr_int to do cross-correlational interferometry

% Evaluate GFs with seismic interferometry
[~,full_int_Ss_f] = do_xcorr_int(freq2,mgfs_f,1:nrcx,1:nrcx,ro,c,ds,'intrec','active','freq','full',method,dgfs_f);
[~,mon_int_Ss_f] = do_xcorr_int(freq2,mgfs_f,1:nrcx,1:nrcx,ro,c,ds,'intrec','active','freq','mono',method);

clear  mgfs_f dgfs_f



%% Select results
full_int_Ss_f = [sqrt(2).*full_int_Ss_f(1:2*nf-1,:,:);full_int_Ss_f(2*nf,:,:)]; % We have taken only half of the frequency spectrum, so we have to adjust the amplitudes
mon_int_Ss_f = [sqrt(2).*mon_int_Ss_f(1:2*nf-1,:,:);mon_int_Ss_f(2*nf,:,:)]; % We have taken only half of the frequency spectrum, so we have to adjust the amplitudes
  


%% The true and interferometric GFs are:


% True GF
gftrue_f = gftrue_mon_p_f.*repmat(conj(wavelet_f),[1 size(gftrue_mon_p_f,2) size(gftrue_mon_p_f,3)]); % Convolve spectrum of true trace with conjugate of source wavelet
gftrue_f = [sqrt(2).*gftrue_f(1:2*nf-1,:,:);gftrue_f(2*nf,:,:)]; % We have taken only half of the frequency spectrum, so we have to adjust the amplitudes
gftrue_f_2sided = centerfreqs(gftrue_f,2*nf); % Get two-sided frequency spectrum for ifft
gftrue_t_2sided = fftshift(sqrt(2*2*nf)*ifft(fftshift(gftrue_f_2sided,1),[],1),1); % Move to time domain
gftrue_t_2sided = real([gftrue_t_2sided(end:-1:2*nf+2,:,:);gftrue_t_2sided(2*nf+1:end,:,:)]);
clear gftrue_f_2sided
gftrue_t = gftrue_t_2sided(2*nf:end,:,:);

% True GF as stack of true GF and its complex conjugate - for comparison
% with interferometry results
if strcmp(method,'WFvir'); gftrue_f_comp = gftrue_f + conj(gftrue_f);
elseif strcmp(method,'WFvi'); gftrue_f_comp = gftrue_f - conj(gftrue_f);
elseif strcmp(method,'VMvi'); gftrue_f_comp = conj(gftrue_f) - gftrue_f;
elseif strcmp(method,'VMvir'); gftrue_f_comp = gftrue_f + conj(gftrue_f);
end
gftrue_f_2sided_comp = centerfreqs(gftrue_f_comp,2*nf); % Get two-sided frequency spectrum for ifft
gftrue_t_2sided_comp = fftshift(sqrt(2*2*nf)*ifft(fftshift(gftrue_f_2sided_comp,1),[],1),1); % Move to time domain
gftrue_t_2sided_comp = real(gftrue_t_2sided_comp(2:end,:,:));
clear gftrue_f_2sided_comp

% Interferometric GFs from boundary S of sources - mono
gfint_mon_f = mon_int_Ss_f;
gfint_mon_f_2sided = centerfreqs(mon_int_Ss_f,2*nf); % Get two-sided frequency spectrum for ifft
gfint_mon_t_2sided = fftshift(sqrt(2*2*nf)*ifft(fftshift(gfint_mon_f_2sided,1),[],1),1); % Move to time domain
gfint_mon_t_2sided = real(gfint_mon_t_2sided(2:end,:,:));
clear gfint_mon_f_2sided mon_int_Ss_f

% Interferometric GFs from boundary S of sources - full
gfint_full_f = full_int_Ss_f;
gfint_full_f_2sided = centerfreqs(full_int_Ss_f,2*nf); % Get two-sided frequency spectrum for ifft
gfint_full_t_2sided = fftshift(sqrt(2*2*nf)*ifft(fftshift(gfint_full_f_2sided,1),[],1),1); % Move to time domain
gfint_full_t_2sided = real(gfint_full_t_2sided(2:end,:,:));
clear gfint_full_f_2sided full_int_Ss_f

% Normalise amplitudes
if strcmp(normamp,'yes')
    gftrue_t_2sided = gftrue_t_2sided./repmat(max(abs(gftrue_t_2sided)),[2*2*nf-1 1 1]);
    gftrue_t = gftrue_t./repmat(max(abs(gftrue_t)),[2*nf 1 1]);
    gftrue_f_comp = gftrue_f_comp./repmat(max(abs(gftrue_f_comp)),[2*nf 1 1]);
    gftrue_t_2sided_comp = gftrue_t_2sided_comp./repmat(max(abs(gftrue_t_2sided_comp)),[2*2*nf-1 1 1]);
    gfint_mon_f = gfint_mon_f./repmat(max(abs(gfint_mon_f)),[2*nf 1 1]);
    gfint_mon_t_2sided = gfint_mon_t_2sided./repmat(max(abs(gfint_mon_t_2sided)),[2*2*nf-1 1 1]);
    gfint_full_f = gfint_full_f./repmat(max(abs(gfint_full_f)),[2*nf 1 1]);
    gfint_full_t_2sided = gfint_full_t_2sided./repmat(max(abs(gfint_full_t_2sided)),[2*2*nf-1 1 1]);
end



%% Green's functions AB (from A to B)

% True GF
AB_true_t = gftrue_t(:,2,1);
AB_true_t_2sided = gftrue_t_2sided(:,2,1);

% True GF for comparison
AB_true_f_comp = [0;gftrue_f_comp(:,2,1)];
AB_true_t_2sided_comp = gftrue_t_2sided_comp(:,2,1);
clear gftrue*

% Interferometric GFs from boundary S of sources - mono
AB_int_mon_f = [0;gfint_mon_f(:,2,1)];
AB_int_mon_t_2sided = gfint_mon_t_2sided(:,2,1);
clear gfint_mon*

% Interferometric GFs from boundary S of sources - mono
AB_int_full_f = [0;gfint_full_f(:,2,1)];
AB_int_full_t_2sided = gfint_full_t_2sided(:,2,1);
clear gfint_full*



%% Plots

scrnsz = get(0,'screensize');
amaxt=max(max([abs(AB_true_t_2sided),abs(AB_int_mon_t_2sided),abs(AB_int_full_t_2sided)]));
amaxf=max(max([abs(AB_true_f_comp),abs(AB_int_mon_f),abs(AB_int_full_f)]));

% Plot geometry and true trace
fig1 = figure;
fig1w = 800;
fig1h = 600;
pos = get(fig1,'Position');
outpos = get(fig1,'OuterPosition');
bord = outpos - pos; clear pos outpos
topb = bord(4)-abs(bord(2));
leftb = abs(bord(1));
set(fig1,'Position',[leftb scrnsz(4)-(topb+fig1h) fig1w fig1h])
% Plot geometry
sub1=subplot(211);
hold on
src=plot(srx(1:6:end,1),srx(1:6:end,2),'r*','markersize',8);
rec=plot(rcx(:,1),rcx(:,2),'bv','markersize',8,'markerfacecolor','b');
if strcmp(scattering,'yes')
    scat=plot(scats(:,1),scats(:,2),'go','markersize',8,'markerfacecolor','g');
end
axis equal
axis([min(srx(:,1)-50) max(srx(:,1)+50) min(srx(:,2)-50) max(srx(:,2)+50)])
box on
set(gca,'fontsize',14)
xlabel ('x (m)','fontsize',14)
ylabel ('y (m)','fontsize',14)
title ('Geometry','fontsize',14)
text(rcx(1,1),rcx(1,2)-60,'A','fontsize',14,'horizontalalignment','center')
text(rcx(2,1),rcx(2,2)-60,'B','fontsize',14,'horizontalalignment','center')
if strcmp(scattering,'no')
    legend([src rec],'Source','Receiver','location','bestoutside')
elseif strcmp(scattering,'yes')
    legend([src rec scat],'Source','Receiver','Scatterer','location','bestoutside')
end
% Plot true trace
sub2=subplot(212);
hold on
plot(time1,AB_true_t,'k','linewidth',2)
axis([0 max(time1) -max(abs(AB_true_t)) max(abs(AB_true_t))])
box on
set(gca,'fontsize',14)
xlabel ('Time (s)','fontsize',14)
if strcmp(normamp,'yes'); ylabel ('Normalised Amplitude','fontsize',14)
elseif strcmp(normamp,'no'); ylabel ('Amplitude','fontsize',14);end
title ('True trace obtained for a source at A and a receiver at B','fontsize',14)

set(sub1,'Position',[0.275 0.45 0.45 0.45])
set(sub2,'Position',[0.1 0.075 0.8 0.2])


% Plot results: true GF and interferometric GF 
% Time domain, acausal and causal parts
fig2=figure;
fig2w=1200;
fig2h=300;
pos = get(fig1,'Position');
set(fig2,'Position',[pos(1)+leftb scrnsz(4)-(2*topb+fig2h) fig2w fig2h])
% Complete traces
hold on
line([0 0],[-amaxt amaxt],'Linestyle','--','color','k')
true=plot(time2,AB_true_t_2sided,'k','linewidth',2);
int_mon=plot(time2,AB_int_mon_t_2sided,'r--','linewidth',2);
int_full=plot(time2,AB_int_full_t_2sided,'g-.','linewidth',2);
axis([min(time2) max(time2) -amaxt amaxt])
box on
set(gca,'fontsize',14)
xlabel ('Time (s)','fontsize',14)
if strcmp(normamp,'yes'); ylabel ('Normalised Amplitude','fontsize',14)
elseif strcmp(normamp,'no'); ylabel ('Amplitude','fontsize',14);end
legend([true int_mon int_full],'True trace','Interferometric trace (monopole)','Interferometric trace (full)')
title ('True trace vs. interferometric traces','fontsize',14)
text(min(time2)/2,-amaxt/2,'Acausal','fontsize',14,'horizontalalignment','center')
text(max(time2)/2,-amaxt/2,'Causal','fontsize',14,'horizontalalignment','center')


% Plot results: true GF (with acausal part calculated according to the
% interferometric formulae) and interferometric GF
% Time and frequency domain (absolute, real and imaginary)
fig3=figure;
fig3w=1200;
fig3h=600;
pos = get(fig2,'Position');
set(fig3,'Position',[pos(1)+leftb scrnsz(4)-(3*topb+fig3h) fig3w fig3h])
% Time domain
subplot(211)
hold on
true=plot(time2,AB_true_t_2sided_comp,'k','linewidth',2);
int_mon=plot(time2,AB_int_mon_t_2sided,'r--','linewidth',2);
int_full=plot(time2,AB_int_full_t_2sided,'g-.','linewidth',2);
axis([min(time2) max(time2) -amaxt amaxt])
box on
set(gca,'fontsize',14)
xlabel ('Time (s)','fontsize',14)
if strcmp(normamp,'yes'); ylabel ('Normalised Amplitude','fontsize',14)
elseif strcmp(normamp,'no'); ylabel ('Amplitude','fontsize',14);end
legend([true int_mon int_full],'''True'' trace','Interferometric trace (monopole)','Interferometric trace (full)')
title ('Time domain','fontsize',14)
% Frequency domain - spectrum
subplot(234)
hold on
plot(freqs,abs(AB_true_f_comp),'k','linewidth',2);
plot(freqs,abs(AB_int_mon_f),'r--','linewidth',2);
plot(freqs,abs(AB_int_full_f),'g-.','linewidth',2);
axis([0 max(freqs) 0 amaxf])
box on
set(gca,'fontsize',14)
xlabel ('Frequency (Hz)','fontsize',14)
if strcmp(normamp,'yes'); ylabel ('Normalised Amplitude','fontsize',14)
elseif strcmp(normamp,'no'); ylabel ('Amplitude','fontsize',14);end
title ('Frequency domain - absolute','fontsize',14)
% Frequency domain - real
subplot(235)
hold on
plot(freqs,real(AB_true_f_comp),'k','linewidth',2);
plot(freqs,real(AB_int_mon_f),'r--','linewidth',2);
plot(freqs,real(AB_int_full_f),'g-.','linewidth',2);
axis([0 max(freqs) -amaxf amaxf])
box on
set(gca,'fontsize',14)
xlabel ('Frequency (Hz)','fontsize',14)
if strcmp(normamp,'yes'); ylabel ('Normalised Amplitude','fontsize',14)
elseif strcmp(normamp,'no'); ylabel ('Amplitude','fontsize',14);end
title ('Frequency domain - real','fontsize',14)
% Frequency domain - imaginary
subplot(236)
hold on
plot(freqs,imag(AB_true_f_comp),'k','linewidth',2);
plot(freqs,imag(AB_int_mon_f),'r--','linewidth',2);
plot(freqs,imag(AB_int_full_f),'g-.','linewidth',2);
axis([0 max(freqs) -amaxf amaxf])
box on
set(gca,'fontsize',14)
xlabel ('Frequency (Hz)','fontsize',14)
if strcmp(normamp,'yes'); ylabel ('Normalised Amplitude','fontsize',14)
elseif strcmp(normamp,'no'); ylabel ('Amplitude','fontsize',14);end
title ('Frequency domain - imaginary','fontsize',14)


% Plot geometry
fig4=figure;
fig4w=500;
fig4h=300;
pos = get(fig3,'Position');
set(fig4,'Position',[pos(1)+leftb scrnsz(4)-(4*topb+fig4h) fig4w fig4h])
% Plot
hold on
src=plot(srx(1:6:end,1),srx(1:6:end,2),'r*','markersize',8);
rec=plot(rcx(:,1),rcx(:,2),'bv','markersize',8,'markerfacecolor','b');
if strcmp(scattering,'yes')
    scat=plot(scats(:,1),scats(:,2),'go','markersize',8,'markerfacecolor','g');
end
axis equal
axis([min(srx(:,1)-50) max(srx(:,1)+50) min(srx(:,2)-50) max(srx(:,2)+50)])
box on
set(gca,'fontsize',14,'XTick',-500:250:500,'YTick',-500:250:500)
xlabel ('x (m)','fontsize',14)
ylabel ('y (m)','fontsize',14)
% title ('Geometry','fontsize',14)
text(rcx(1,1),rcx(1,2)-80,'A','fontsize',14,'horizontalalignment','center')
text(rcx(2,1),rcx(2,2)-80,'B','fontsize',14,'horizontalalignment','center')
% if strcmp(scattering,'no')
%     legend([src rec],'Source','Receiver','location','bestoutside')
% elseif strcmp(scattering,'yes')
%     legend([src rec scat],'Source','Receiver','Scatterer','location','bestoutside')
% end


% Plot results: true GF and interferometric GFs 
% Time domain, causal part only and zoomed-in areas
fig5=figure;
fig5w=1200;
fig5h=600;
pos = get(fig4,'Position');
set(fig5,'Position',[pos(1)+leftb scrnsz(4)-(5*topb+fig5h) fig5w fig5h])
% Complete traces
main=axes('position',[0.1 0.1 0.8 0.8]);
sca=0.8/(2*amaxt);
sct=0.8/(0.5*max(time2));
hold on
true=plot(time2,AB_true_t_2sided_comp,'k','linewidth',2);
int_mon=plot(time2,AB_int_mon_t_2sided,'r--','linewidth',2);
int_full=plot(time2,AB_int_full_t_2sided,'g-.','linewidth',2);
axis([0 0.4*max(time2) -amaxt amaxt])
box on
set(gca,'fontsize',18)
xlabel ('Time (s)','fontsize',18)
if strcmp(normamp,'yes'); ylabel ('Normalised amplitude (pressure)','fontsize',18)
elseif strcmp(normamp,'no'); ylabel ('Amplitude (pressure)','fontsize',18);end
leg=legend([true int_mon int_full],'True trace','Interferometric trace (monopole)','Interferometric trace (full)');
set(leg,'fontsize',16,'position',[0.64 0.76 0.25 0.1])
% Zoom 1
z1ax=[0.202 0.213 0.94*amaxt 0.99*amaxt];
z1=axes('position',[0.3 0.6 (z1ax(2)-z1ax(1))*sct*12 (z1ax(4)-z1ax(3))*sca*12]);
hold on
plot(time2,AB_true_t_2sided_comp,'k','linewidth',2);
plot(time2,AB_int_mon_t_2sided,'r--','linewidth',2);
plot(time2,AB_int_full_t_2sided,'g-.','linewidth',2);
axis(z1ax)
box on
set(z1,'fontsize',14,'XTick',[0.205;0.21;0.215],'XTickLabel',[0.205;0.21;0.215])
% xlabel ('Time (s)','fontsize',10)
% if strcmp(normamp,'yes'); ylabel ('Normalised Amplitude','fontsize',10)
% elseif strcmp(normamp,'no'); ylabel ('Amplitude','fontsize',10);end
% Zoom 2
z2ax=[0.39 0.4 -0.17*amaxt -0.12*amaxt];
z2=axes('position',[0.376 0.19 (z2ax(2)-z2ax(1))*sct*10 (z2ax(4)-z2ax(3))*sca*10]);
hold on
plot(time2,AB_true_t_2sided_comp,'k','linewidth',2);
plot(time2,AB_int_mon_t_2sided,'r--','linewidth',2);
plot(time2,AB_int_full_t_2sided,'g-.','linewidth',2);
axis(z2ax)
box on
set(z2,'fontsize',14,'XTick',[0.390;0.395;0.400],'XTickLabel',[0.390;0.395;0.400])
% xlabel ('Time (s)','fontsize',10)
% if strcmp(normamp,'yes'); ylabel ('Normalised Amplitude','fontsize',10)
% elseif strcmp(normamp,'no'); ylabel ('Amplitude','fontsize',10);end
% Zoom 3
z3ax=[0.5575 0.5675 -0.12*amaxt -0.07*amaxt];
z3=axes('position',[0.490 0.570 (z3ax(2)-z3ax(1))*sct*10 (z3ax(4)-z3ax(3))*sca*10]);
hold on
plot(time2,AB_true_t_2sided_comp,'k','linewidth',2);
plot(time2,AB_int_mon_t_2sided,'r--','linewidth',2);
plot(time2,AB_int_full_t_2sided,'g-.','linewidth',2);
axis(z3ax)
box on
set(z3,'fontsize',14,'XTick',[0.560;0.565;0.570],'XTickLabel',[0.560;0.565;0.570])
% xlabel ('Time (s)','fontsize',10)
% if strcmp(normamp,'yes'); ylabel ('Normalised Amplitude','fontsize',10)
% elseif strcmp(normamp,'no'); ylabel ('Amplitude','fontsize',10);end
% Zoom 4
z4ax=[0.6575 0.6675 -0.1*amaxt -0.05*amaxt];
z4=axes('position',[0.584 0.228 (z4ax(2)-z4ax(1))*sct*10 (z4ax(4)-z4ax(3))*sca*10]);
hold on
plot(time2,AB_true_t_2sided_comp,'k','linewidth',2);
plot(time2,AB_int_mon_t_2sided,'r--','linewidth',2);
plot(time2,AB_int_full_t_2sided,'g-.','linewidth',2);
axis(z4ax)
box on
set(z4,'fontsize',14,'XTick',[0.660;0.665;0.670],'XTickLabel',[0.660;0.665;0.670])
% xlabel ('Time (s)','fontsize',10)
% if strcmp(normamp,'yes'); ylabel ('Normalised Amplitude','fontsize',10)
% elseif strcmp(normamp,'no'); ylabel ('Amplitude','fontsize',10);end



%% %%% Figures in GEOPHYSICS format %%% %%

% Plot geometry
fig6=figure;
fig6w=3.33;
fig6h=3.33;
set(fig6,'units','inches','Pos',[1 1 fig6w fig6h])
% Plot
hold on
src=plot(srx(1:6:end,1),srx(1:6:end,2),'k*','markersize',6);
rec=plot(rcx(:,1),rcx(:,2),'kv','markersize',6,'markerfacecolor','k');
if strcmp(scattering,'yes')
    scat=plot(scats(:,1),scats(:,2),'ko','markersize',6,'markerfacecolor','w');
end
axis equal
axis([min(srx(:,1)-50) max(srx(:,1)+50) min(srx(:,2)-50) max(srx(:,2)+50)])
box on
set(gca,'fontsize',8,'XTick',-500:250:500,'YTick',-500:250:500)
xlabel ('x (m)','fontsize',8)
ylabel ('y (m)','fontsize',8)
text(rcx(1,1),rcx(1,2)-50,'A','fontsize',8,'FontAngle','italic','horizontalalignment','center')
text(rcx(2,1),rcx(2,2)-50,'B','fontsize',8,'FontAngle','italic','horizontalalignment','center')


% Plot results: true GF and interferometric GFs 
% Time domain, causal part only and zoomed-in areas
fig7=figure;
fig7w=4.33;
fig7h=6;
set(fig7,'units','inches','Pos',[5 1 fig7w fig7h])
% Complete traces
main=axes('position',[0.1 0.1 0.8 0.8]);
sca=0.8/(2*amaxt);
sct=0.8/(0.5*max(time2));
hold on
set(gca,'ColorOrder',[0 0 0;0.2 0.2 0.2;0.6 0.6 0.6])
hold all
true=plot(time2,AB_true_t_2sided_comp,'-','linewidth',2);
int_mon=plot(time2,AB_int_mon_t_2sided,'--','linewidth',2);
int_full=plot(time2,AB_int_full_t_2sided,'-.','linewidth',2);
axis([0 0.4*max(time2) -amaxt amaxt])
box on
set(gca,'fontsize',8)
%%%%% Following 8 lines adapted from http://www.mathworks.fr/matlabcentral/newsreader/view_thread/112386
oldLabels = str2num(get(gca,'YTickLabel'));
exp = 10^3;
newLabels = num2str(oldLabels);
set(gca,'YTickLabel',newLabels,'units','normalized');
posAxes = get(gca,'position');
textBox = annotation('textbox','linestyle','none','margin',1,'string',['x 10^{' sprintf('%d',log10(1./exp)) '}']);
posAn = get(textBox,'position');
set(textBox,'fontsize',8,'position',[posAxes(1) posAxes(2)+posAxes(4) posAn(3) posAn(4)],'VerticalAlignment','bottom','HorizontalAlignment','left');
%%%%%
xlabel ('Time (s)','fontsize',8)
if strcmp(normamp,'yes'); ylabel ('Normalised amplitude (pressure)','fontsize',8)
elseif strcmp(normamp,'no'); ylabel ('Amplitude (pressure)','fontsize',8);end
leg=legend([true int_mon int_full],'True trace','Interferometric trace (monopole)','Interferometric trace (full)','location','southoutside');
% Zoom 1
z1ax=[0.209 0.2105 0.979*amaxt 0.984*amaxt];
z1=axes('position',[0.35 0.65 (z1ax(2)-z1ax(1))*sct*140 (z1ax(4)-z1ax(3))*sca*100]);
set(gca,'ColorOrder',[0 0 0;0.2 0.2 0.2;0.6 0.6 0.6])
hold all
plot(time2,AB_true_t_2sided_comp,'-',time2,AB_int_mon_t_2sided,'--',time2,AB_int_full_t_2sided,'-.','linewidth',2);
axis(z1ax)
box on
set(z1,'fontsize',8,'XTick',[0.209;0.2105],'XTickLabel',[0.209;0.2105])
%%%%% Following 8 lines adapted from http://www.mathworks.fr/matlabcentral/newsreader/view_thread/112386
oldLabels = str2num(get(gca,'YTickLabel'));
exp = 10^3;
newLabels = num2str(oldLabels);
set(gca,'YTickLabel',newLabels,'units','normalized');
posAxes = get(gca,'position');
textBox = annotation('textbox','linestyle','none','margin',1,'string',['x 10^{' sprintf('%d',log10(1./exp)) '}']);
posAn = get(textBox,'position');
set(textBox,'fontsize',8,'position',[posAxes(1) posAxes(2)+posAxes(4) posAn(3) posAn(4)],'VerticalAlignment','bottom','HorizontalAlignment','left');
%%%%%
% Zoom 2
z2ax=[0.385 0.405 -0.17*amaxt -0.12*amaxt];
z2=axes('position',[0.38 0.28 (z2ax(2)-z2ax(1))*sct*10 (z2ax(4)-z2ax(3))*sca*10]);
set(gca,'ColorOrder',[0 0 0;0.2 0.2 0.2;0.6 0.6 0.6])
hold all
plot(time2,AB_true_t_2sided_comp,'-',time2,AB_int_mon_t_2sided,'--',time2,AB_int_full_t_2sided,'-.','linewidth',2);
axis(z2ax)
box on
set(z2,'fontsize',8,'XTick',[0.390;0.400],'XTickLabel',[0.390;0.400])
%%%%% Following 8 lines adapted from http://www.mathworks.fr/matlabcentral/newsreader/view_thread/112386
oldLabels = str2num(get(gca,'YTickLabel'));
exp = 10^4;
newLabels = num2str(oldLabels);
set(gca,'YTickLabel',newLabels,'units','normalized');
posAxes = get(gca,'position');
textBox = annotation('textbox','linestyle','none','margin',1,'string',['x 10^{' sprintf('%d',log10(1./exp)) '}']);
posAn = get(textBox,'position');
set(textBox,'fontsize',8,'position',[posAxes(1) posAxes(2)+posAxes(4) posAn(3) posAn(4)],'VerticalAlignment','bottom','HorizontalAlignment','left');
%%%%%
% Zoom 3
z3ax=[0.555 0.575 -0.12*amaxt -0.07*amaxt];
z3=axes('position',[0.6 0.65 (z3ax(2)-z3ax(1))*sct*10 (z3ax(4)-z3ax(3))*sca*10]);
set(gca,'ColorOrder',[0 0 0;0.2 0.2 0.2;0.6 0.6 0.6])
hold all
plot(time2,AB_true_t_2sided_comp,'-',time2,AB_int_mon_t_2sided,'--',time2,AB_int_full_t_2sided,'-.','linewidth',2);
axis(z3ax)
box on
set(z3,'fontsize',8,'XTick',[0.560;0.570],'XTickLabel',[0.560;0.570])
%%%%% Following 8 lines adapted from http://www.mathworks.fr/matlabcentral/newsreader/view_thread/112386
oldLabels = str2num(get(gca,'YTickLabel'));
exp = 10^4;
newLabels = num2str(oldLabels);
set(gca,'YTickLabel',newLabels,'units','normalized');
posAxes = get(gca,'position');
textBox = annotation('textbox','linestyle','none','margin',1,'string',['x 10^{' sprintf('%d',log10(1./exp)) '}']);
posAn = get(textBox,'position');
set(textBox,'fontsize',8,'position',[posAxes(1) posAxes(2)+posAxes(4) posAn(3) posAn(4)],'VerticalAlignment','bottom','HorizontalAlignment','left');
%%%%%
% Zoom 4
z4ax=[0.655 0.675 -0.1*amaxt -0.05*amaxt];
z4=axes('position',[0.62 0.28 (z4ax(2)-z4ax(1))*sct*10 (z4ax(4)-z4ax(3))*sca*10]);
set(gca,'ColorOrder',[0 0 0;0.2 0.2 0.2;0.6 0.6 0.6])
hold all
plot(time2,AB_true_t_2sided_comp,'-',time2,AB_int_mon_t_2sided,'--',time2,AB_int_full_t_2sided,'-.','linewidth',2);
axis(z4ax)
box on
set(z4,'fontsize',8,'XTick',[0.66;0.67],'XTickLabel',[0.66;0.67])
%%%%% Following 8 lines adapted from http://www.mathworks.fr/matlabcentral/newsreader/view_thread/112386
oldLabels = str2num(get(gca,'YTickLabel'));
exp = 10^4;
newLabels = num2str(oldLabels);
set(gca,'YTickLabel',newLabels,'units','normalized');
posAxes = get(gca,'position');
textBox = annotation('textbox','linestyle','none','margin',1,'string',['x 10^{' sprintf('%d',log10(1./exp)) '}']);
posAn = get(textBox,'position');
set(textBox,'fontsize',8,'position',[posAxes(1) posAxes(2)+posAxes(4) posAn(3) posAn(4)],'VerticalAlignment','bottom','HorizontalAlignment','left');
%%%%%
