%% Wavefield modelling examples

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% This code is distributed as part of the source-code package 
%                   acoustic_modelling_foldy_v.4.1_seg 
% that accompanies Galetti et al. (2013). The package can be downloaded 
% from the Geophysics source-code archive at 
%                   http://software.seg.org/2013/0003
% Use of this code is subject to acceptance of the terms and conditions
% that can be found at http://software.seg.org/disclaimer.txt 
% Copyright 2013 by The Society of Exploration Geophysicists (SEG)
% Reference:
% Galetti, E., Halliday, D., Curtis, A. (2013) A simple and exact acoustic 
% wavefield modelling code for data processing, imaging and interferometry 
% applications. Geophysics.
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% This script models the wavefield from source srx to receiver rcx in 1D,
% 2D and 3D with and without applying a wavelet. Both monopole and dipole
% Green's functions are evaluated.
%
% Figure 1: Green's functions in 1D:
%               black solid trace - monopole Green's function 
%               magenta dashed trace - dipole (derivative) Green's function
%                   left panel: no wavelet applied
%                   right panel: Ricker wavelet of central frequency fc
%                                applied
% Figure 2: Green's functions in 2D:
%               black solid trace - monopole Green's function 
%               magenta dashed trace - dipole (derivative) Green's function
%                   left panel: no wavelet applied
%                   right panel: Ricker wavelet of central frequency fc
%                                applied
% Figure 3: Green's functions in 3D:
%               black solid trace - monopole Green's function 
%               magenta dashed trace - dipole (derivative) Green's function
%                   left panel: no wavelet applied
%                   right panel: Ricker wavelet of central frequency fc
%                                applied
% Figure 4: Source wavelet
%                   left panel: Ricker wavelet in time domain
%                   right panel: Ricker wavelet in frequency domain
%
%
% References:
%       Groenenboom, J., Snieder, R. (1995) Attenuation, dispersion, and
%   anisotropy by multiple scattering of transmitted waves through
%   distributions of scatterers. J. Acoust. Soc. Am. 98 (6): 3482-3492.
%       Snieder, R. (2009) A Guided Tour of Mathematical Methods for the
%   Physical Sciences, 2nd ed., Cambridge University Press.
%
%
% Required:
% acoustic_p_gfs_direct_f.m - models direct wave between two points in an
%                             acoustic medium in the frequency domain only
% model_gfs_p_scat_acoust.m - models scattered waves in an acoustic medium
% centerfreqs.m             - creates a two-sided centred frequency
%                             spectrum from positive frequencies
% rickerwavl_time.m         - creates Ricker wavelet for source
%
%
% Erica Galetti, July 2012
% Contact: erica.galetti@ed.ac.uk



clear all
close all


%% Define variables
srx = [0,0,0];						% Source position
rcx = [200,0,0];					% Receiver position
dir = [1,0,0];                      % Direction of derivation (for derivative Green's function)
maxf = 100;                         % Maximum frequency to model
nf = 256;							% Number of frequencies to model
c = 1000;							% Medium velocity
srctype = 'volinjpos';              % Source type
wavltype = 'wavly_centr';           % Type of wavelet
fc = 10;							% Central frequency of Ricker wavelet
nyq = c/maxf;                       % Minimum Nyquist wavelength

% Define secondary array of sources used to calculate the derivative
% Green's function numerically
ds = nyq/20;
srxd(:,1) = [-2*ds;-ds;ds;2*ds];
srxd(:,2:3) = 0;



%% Model wavefields

% Wavelet, time and frequency vectors
[freq,~,~,time,~,~,wavelet_f,wavelet_t] = model_gfs_p_scat_acoust(srx,rcx,maxf,nf,c,'1d',srctype,'srcdip','scatn',wavltype,'freqy_1s','timey','tot',fc);

% 1D
dim = '1d';
ndim = 1;
[~,mgf_1d,dgf] = model_gfs_p_scat_acoust(srx,rcx,maxf,nf,c,dim,srctype,'srcdip','scatn','wavln','freqn','timey','tot');
dgf_1d = zeros(size(mgf_1d));
for dd = 1:ndim; dgf_1d = dgf_1d + dgf(:,:,:,dd).*dir(1,dd); end % weight dipole components by the derivative direction components - this gives the full dipole wavefield 
clear dgf
[~,mgf_1d_w,dgf,~] = model_gfs_p_scat_acoust(srx,rcx,maxf,nf,c,dim,srctype,'srcdip','scatn',wavltype,'freqn','timey','tot',fc);
dgf_1d_w = zeros(size(mgf_1d_w));
for dd = 1:ndim; dgf_1d_w = dgf_1d_w + dgf(:,:,:,dd).*dir(1,dd); end % weight dipole components by the derivative direction components - this gives the full dipole wavefield 
clear dgf
[~,mgf,~,~] = model_gfs_p_scat_acoust(srxd,rcx,maxf,nf,c,dim,srctype,'srcdip','scatn',wavltype,'freqn','timey','tot',fc);
% Use the Richardson extrapolation to evaluate the derivative numerically 
dgf1 = (mgf(:,1,3)-mgf(:,1,2))/(2*ds);
dgf2 = (mgf(:,1,4)-mgf(:,1,1))/(4*ds);
dgf_1d_w_n = 1/3.*(4.*dgf1-dgf2);
clear mgf dgf1 dgf2

% 2D
dim = '2d';
ndim = 2;
[~,mgf_2d,dgf] = model_gfs_p_scat_acoust(srx,rcx,maxf,nf,c,dim,srctype,'srcdip','scatn','wavln','freqn','timey','tot');
dgf_2d = zeros(size(mgf_2d));
for dd = 1:ndim; dgf_2d = dgf_2d + dgf(:,:,:,dd).*dir(1,dd); end % weight dipole components by the derivative direction components - this gives the full dipole wavefield 
clear dgf
[~,mgf_2d_w,dgf,~] = model_gfs_p_scat_acoust(srx,rcx,maxf,nf,c,dim,srctype,'srcdip','scatn',wavltype,'freqn','timey','tot',fc);
dgf_2d_w = zeros(size(mgf_2d_w));
for dd = 1:ndim; dgf_2d_w = dgf_2d_w + dgf(:,:,:,dd).*dir(1,dd); end % weight dipole components by the derivative direction components - this gives the full dipole wavefield 
clear dgf
[~,mgf,~,~] = model_gfs_p_scat_acoust(srxd,rcx,maxf,nf,c,dim,srctype,'srcdip','scatn',wavltype,'freqn','timey','tot',fc);
% Use the Richardson extrapolation to evaluate the derivative numerically 
dgf1 = (mgf(:,1,3)-mgf(:,1,2))/(2*ds);
dgf2 = (mgf(:,1,4)-mgf(:,1,1))/(4*ds);
dgf_2d_w_n = 1/3.*(4.*dgf1-dgf2);
clear mgf dgf1 dgf2

% 3D
dim = '3d';
ndim = 3;
[~,mgf_3d,dgf] = model_gfs_p_scat_acoust(srx,rcx,maxf,nf,c,dim,srctype,'srcdip','scatn','wavln','freqn','timey','tot');
dgf_3d = zeros(size(mgf_3d));
for dd = 1:ndim; dgf_3d = dgf_3d + dgf(:,:,:,dd).*dir(1,dd); end % weight dipole components by the derivative direction components - this gives the full dipole wavefield 
clear dgf
[~,mgf_3d_w,dgf,~] = model_gfs_p_scat_acoust(srx,rcx,maxf,nf,c,dim,srctype,'srcdip','scatn',wavltype,'freqn','timey','tot',fc);
dgf_3d_w = zeros(size(mgf_3d_w));
for dd = 1:ndim; dgf_3d_w = dgf_3d_w + dgf(:,:,:,dd).*dir(1,dd); end % weight dipole components by the derivative direction components - this gives the full dipole wavefield 
clear dgf
[~,mgf,~,~] = model_gfs_p_scat_acoust(srxd,rcx,maxf,nf,c,dim,srctype,'srcdip','scatn',wavltype,'freqn','timey','tot',fc);
% Use the Richardson extrapolation to evaluate the derivative numerically 
dgf1 = (mgf(:,1,3)-mgf(:,1,2))/(2*ds);
dgf2 = (mgf(:,1,4)-mgf(:,1,1))/(4*ds);
dgf_3d_w_n = 1/3.*(4.*dgf1-dgf2);
clear mgf dgf1 dgf2



%% Normalise all amplitudes

wavelet_t = wavelet_t./max(abs(wavelet_t));
wavelet_f = wavelet_f./max(abs(wavelet_f));
mgf_1d = mgf_1d./max(abs(mgf_1d));
dgf_1d = dgf_1d./max(abs(dgf_1d));
mgf_1d_w = mgf_1d_w./max(abs(mgf_1d_w));
dgf_1d_w = dgf_1d_w./max(abs(dgf_1d_w));
dgf_1d_w_n = dgf_1d_w_n./max(abs(dgf_1d_w_n));
mgf_2d = mgf_2d./max(abs(mgf_2d));
dgf_2d = dgf_2d./max(abs(dgf_2d));
mgf_2d_w = mgf_2d_w./max(abs(mgf_2d_w));
dgf_2d_w = dgf_2d_w./max(abs(dgf_2d_w));
dgf_2d_w_n = dgf_2d_w_n./max(abs(dgf_2d_w_n));
mgf_3d = mgf_3d./max(abs(mgf_3d));
dgf_3d = dgf_3d./max(abs(dgf_3d));
mgf_3d_w = mgf_3d_w./max(abs(mgf_3d_w));
dgf_3d_w = dgf_3d_w./max(abs(dgf_3d_w));
dgf_3d_w_n = dgf_3d_w_n./max(abs(dgf_3d_w_n));



%% Plot

scrnsz = get(0,'screensize');
range = 1:140;
rangew = 1:120;

% 1D
fig1a = figure;
fig1w = 2*get(0,'ScreenPixelsPerInch');
fig1h = 1.6*get(0,'ScreenPixelsPerInch');
pos = get(fig1a,'Position');
outpos = get(fig1a,'OuterPosition');
bord = outpos - pos; clear pos outpos
topb = bord(4)-abs(bord(2));
leftb = abs(bord(1));
set(fig1a,'Position',[leftb scrnsz(4)-(topb+fig1h) fig1w fig1h]);
% Without wavelet
main=axes('position',[0.24 0.2 0.75 0.75]);
hold on
plot(time(range),mgf_1d(range),'k','linewidth',2)
axis([0 time(range(end)) -1 1])
box on
set(main,'fontsize',8,'YTick',[-1;-0.5;0;0.5;1],'YTickLabel',[-1;-0.5;0;0.5;1])
xlabel ('Time (s)','fontsize',8)
ylabel ('Normalised amplitude','fontsize',8)
% With wavelet
fig1b = figure;
outpos = get(fig1a,'OuterPosition');
set(fig1b,'OuterPosition',[outpos(1)+outpos(3) outpos(2) outpos(3) outpos(4)]);
main=axes('position',[0.24 0.2 0.75 0.75]);
hold on
set(main,'ColorOrder',[0.6 0.6 0.6;0.2 0.2 0.2;0 0 0])
hold all
dg=plot(time(range),dgf_1d_w(range),'-','linewidth',2);
dg_n=plot(time(range),dgf_1d_w_n(range),'--','linewidth',2);
g=plot(time(range),mgf_1d_w(range),'-','linewidth',2);
axis([0 time(range(end)) -1 1])
box on
set(main,'fontsize',8,'YTick',[-1;-0.5;0;0.5;1],'YTickLabel',[-1;-0.5;0;0.5;1])
xlabel ('Time (s)','fontsize',8)
ylabel ('Normalised amplitude','fontsize',8)
leg=legend ([g dg dg_n],strtrim('G'),strtrim('\partialG (an.)'),strtrim('\partialG (num.)'),'location','northeast');
% The legend needs to be resized - define new size values
lw=0.28;    % line width in legend
legw=0.42;  % legend width
legh=0.25;  % legend height
legm=0.05;  % legend margin
%%%%% Following 23 lines adapted from http://www.mathworks.co.uk/matlabcentral/newsreader/view_thread/151519
% Create new legend axes
newleg=axes('position',get(leg,'position'));
% Copy the content of the legend to the new axes
cld = get(leg,'children');
for k=1:length(cld)
	set(cld(k),'parent',newleg)
end
% Set the desired properties of the axes and legend contents
objh=get(newleg,'children');
g_t=get(objh(1),'position'); dg_t=get(objh(4),'position'); dg_n_t=get(objh(7),'position');  % get text handles
% g_l=get(objh(2),'xdata'); dg_l=get(objh(5),'xdata'); dg_n_l=get(objh(8),'xdata');           % get line handles
set(objh(2),'xdata',[0.05 0.05+lw]); set(objh(1),'position',[0.05+lw+0.05 g_t(2)  g_t(3)]);         % line and text of G
set(objh(5),'xdata',[0.05 0.05+lw]); set(objh(4),'position',[0.05+lw+0.05 dg_t(2)  dg_t(3)]);       % line and text of dG (analytical)
set(objh(8),'xdata',[0.05 0.05+lw]); set(objh(7),'position',[0.05+lw+0.05 dg_n_t(2)  dg_n_t(3)]);   % line and text of dG (numerical)
set(newleg,'xtick',[],'ytick',[],'box','on','xlim',[0 1])
mainp=get(main,'position'); % get position of main plot
newlegti=get(newleg,'tightinset');  % get tight inset of new legend axes
set(newleg,'looseinset',[legm legm legm-newlegti(3) legm]); % set loose inset of new legend axes
% Set position of new legend axes by changing the 'outerposition' property
newlegop=[mainp(1)+mainp(3)-(legw+newlegti(1)+newlegti(3)) mainp(2) legw+newlegti(1)+newlegti(3) legh+newlegti(2)+newlegti(4)];
set(newleg,'outerposition',newlegop,'fontsize',8);
% Delete the original legend
delete(leg)
%%%%%

% 2D
fig2a = figure;
fig2w = fig1w;
fig2h = fig1h;
pos = get(fig1a,'Position');
set(fig2a,'Position',[pos(1)+leftb scrnsz(4)-(2*topb+fig2h) fig2w fig2h])
% Without wavelet
main=axes('position',[0.24 0.2 0.75 0.75]);
hold on
plot(time(range),mgf_2d(range),'k','linewidth',2)
axis([0 time(range(end)) -1 1])
box on
set(main,'fontsize',8,'YTick',[-1;-0.5;0;0.5;1],'YTickLabel',[-1;-0.5;0;0.5;1])
xlabel ('Time (s)','fontsize',8)
ylabel ('Normalised amplitude','fontsize',8)
% With wavelet
fig2b = figure;
outpos = get(fig2a,'OuterPosition');
set(fig2b,'OuterPosition',[outpos(1)+outpos(3) outpos(2) outpos(3) outpos(4)])
main=axes('position',[0.24 0.2 0.75 0.75]);
hold on
set(main,'ColorOrder',[0.6 0.6 0.6;0.2 0.2 0.2;0 0 0])
hold all
dg=plot(time(range),dgf_2d_w(range),'-','linewidth',2);
dg_n=plot(time(range),dgf_2d_w_n(range),'--','linewidth',2);
g=plot(time(range),mgf_2d_w(range),'-','linewidth',2);
axis([0 time(range(end)) -1 1])
box on
set(main,'fontsize',8,'YTick',[-1;-0.5;0;0.5;1],'YTickLabel',[-1;-0.5;0;0.5;1])
xlabel ('Time (s)','fontsize',8)
ylabel ('Normalised amplitude','fontsize',8)
leg=legend ([g dg dg_n],strtrim('G'),strtrim('\partialG (an.)'),strtrim('\partialG (num.)'),'location','northeast');
% The legend needs to be resized - define new size values
lw=0.28;    % line width in legend
legw=0.42;  % legend width
legh=0.25;  % legend height
legm=0.05;  % legend margin
%%%%% Following 23 lines adapted from http://www.mathworks.co.uk/matlabcentral/newsreader/view_thread/151519
% Create new legend axes
newleg=axes('position',get(leg,'position'));
% Copy the content of the legend to the new axes
cld = get(leg,'children');
for k=1:length(cld)
	set(cld(k),'parent',newleg)
end
% Set the desired properties of the axes and legend contents
objh=get(newleg,'children');
g_t=get(objh(1),'position'); dg_t=get(objh(4),'position'); dg_n_t=get(objh(7),'position');  % get text handles
% g_l=get(objh(2),'xdata'); dg_l=get(objh(5),'xdata'); dg_n_l=get(objh(8),'xdata');           % get line handles
set(objh(2),'xdata',[0.05 0.05+lw]); set(objh(1),'position',[0.05+lw+0.05 g_t(2)  g_t(3)]);         % line and text of G
set(objh(5),'xdata',[0.05 0.05+lw]); set(objh(4),'position',[0.05+lw+0.05 dg_t(2)  dg_t(3)]);       % line and text of dG (analytical)
set(objh(8),'xdata',[0.05 0.05+lw]); set(objh(7),'position',[0.05+lw+0.05 dg_n_t(2)  dg_n_t(3)]);   % line and text of dG (numerical)
set(newleg,'xtick',[],'ytick',[],'box','on','xlim',[0 1])
mainp=get(main,'position'); % get position of main plot
newlegti=get(newleg,'tightinset');  % get tight inset of new legend axes
set(newleg,'looseinset',[legm legm legm-newlegti(3) legm]); % set loose inset of new legend axes
% Set position of new legend axes by changing the 'outerposition' property
newlegop=[mainp(1)+mainp(3)-(legw+newlegti(1)+newlegti(3)) mainp(2) legw+newlegti(1)+newlegti(3) legh+newlegti(2)+newlegti(4)];
set(newleg,'outerposition',newlegop,'fontsize',8);
% Delete the original legend
delete(leg)
%%%%%

% 3D
fig3a = figure;
fig3w = fig1w;
fig3h = fig1h;
pos = get(fig2a,'Position');
set(fig3a,'Position',[pos(1)+leftb scrnsz(4)-(3*topb+fig3h) fig3w fig3h])
% Without wavelet
main=axes('position',[0.24 0.2 0.75 0.75]);
hold on
plot(time(range),mgf_3d(range),'k','linewidth',2)
axis([0 time(range(end)) -1 1]) 
box on
set(main,'fontsize',8,'YTick',[-1;-0.5;0;0.5;1],'YTickLabel',[-1;-0.5;0;0.5;1])
xlabel ('Time (s)','fontsize',8)
ylabel ('Normalised amplitude','fontsize',8)
% With wavelet
fig3b = figure;
outpos = get(fig3a,'OuterPosition');
set(fig3b,'OuterPosition',[outpos(1)+outpos(3) outpos(2) outpos(3) outpos(4)])
main=axes('position',[0.24 0.2 0.75 0.75]);
hold on
set(main,'ColorOrder',[0.6 0.6 0.6;0.2 0.2 0.2;0 0 0])
hold all
dg=plot(time(range),dgf_3d_w(range),'-','linewidth',2);
dg_n=plot(time(range),dgf_3d_w_n(range),'--','linewidth',2);
g=plot(time(range),mgf_3d_w(range),'-','linewidth',2);
axis([0 time(range(end)) -1 1])
box on
set(main,'fontsize',8,'YTick',[-1;-0.5;0;0.5;1],'YTickLabel',[-1;-0.5;0;0.5;1])
xlabel ('Time (s)','fontsize',8)
ylabel ('Normalised amplitude','fontsize',8)
leg=legend ([g dg dg_n],strtrim('G'),strtrim('\partialG (an.)'),strtrim('\partialG (num.)'),'location','northeast');
% The legend needs to be resized - define new size values
lw=0.28;    % line width in legend
legw=0.42;  % legend width
legh=0.25;  % legend height
legm=0.05;  % legend margin
%%%%% Following 23 lines adapted from http://www.mathworks.co.uk/matlabcentral/newsreader/view_thread/151519
% Create new legend axes
newleg=axes('position',get(leg,'position'));
% Copy the content of the legend to the new axes
cld = get(leg,'children');
for k=1:length(cld)
	set(cld(k),'parent',newleg)
end
% Set the desired properties of the axes and legend contents
objh=get(newleg,'children');
g_t=get(objh(1),'position'); dg_t=get(objh(4),'position'); dg_n_t=get(objh(7),'position');  % get text handles
% g_l=get(objh(2),'xdata'); dg_l=get(objh(5),'xdata'); dg_n_l=get(objh(8),'xdata');           % get line handles
set(objh(2),'xdata',[0.05 0.05+lw]); set(objh(1),'position',[0.05+lw+0.05 g_t(2)  g_t(3)]);         % line and text of G
set(objh(5),'xdata',[0.05 0.05+lw]); set(objh(4),'position',[0.05+lw+0.05 dg_t(2)  dg_t(3)]);       % line and text of dG (analytical)
set(objh(8),'xdata',[0.05 0.05+lw]); set(objh(7),'position',[0.05+lw+0.05 dg_n_t(2)  dg_n_t(3)]);   % line and text of dG (numerical)
set(newleg,'xtick',[],'ytick',[],'box','on','xlim',[0 1])
mainp=get(main,'position'); % get position of main plot
newlegti=get(newleg,'tightinset');  % get tight inset of new legend axes
set(newleg,'looseinset',[legm legm legm-newlegti(3) legm]); % set loose inset of new legend axes
% Set position of new legend axes by changing the 'outerposition' property
newlegop=[mainp(1)+mainp(3)-(legw+newlegti(1)+newlegti(3)) mainp(2) legw+newlegti(1)+newlegti(3) legh+newlegti(2)+newlegti(4)];
set(newleg,'outerposition',newlegop,'fontsize',8);
% Delete the original legend
delete(leg)
%%%%%

% Wavelet
fig4a = figure;
fig4w = fig1w;
fig4h = fig1h;
pos = get(fig3a,'Position');
set(fig4a,'Position',[pos(1)+leftb scrnsz(4)-(4*topb+fig4h) fig4w fig4h])
% Wavelet in time domain
main=axes('position',[0.24 0.2 0.75 0.75],'looseinset',[0 0 0 0]);
hold on
plot([flipud(-time(rangew(2:end)));time(rangew)],fftshift([wavelet_t(rangew);wavelet_t((2*nf-(rangew(end)-1)+1):end)]),'k','linewidth',2)
axis([-time(rangew(end)) time(rangew(end)) -1 1])
box on
set(main,'fontsize',8,'YTick',[-1;-0.5;0;0.5;1],'YTickLabel',[-1;-0.5;0;0.5;1])
xlabel ('Time (s)','fontsize',8)
ylabel ('Normalised amplitude','fontsize',8)
% Wavelet in frequency domain
fig4b = figure;
outpos = get(fig4a,'OuterPosition');
set(fig4b,'OuterPosition',[outpos(1)+outpos(3) outpos(2) outpos(3) outpos(4)])
main=axes('position',[0.24 0.2 0.75 0.75],'looseinset',[0 0 0 0]);
hold on
plot(freq,abs(wavelet_f),'k','linewidth',2)
axis([0 80 0 1])
box on
set(main,'fontsize',8,'XTick',[0;25;50;75],'XTickLabel',[0;25;50;75])
set(main,'fontsize',8,'YTick',[-1;-0.5;0;0.5;1],'YTickLabel',[-1;-0.5;0;0.5;1])
xlabel ('Frequency (Hz)','fontsize',8)
ylabel ('Normalised amplitude','fontsize',8)



% %% Save figures using function savefig
% % The savefig function can be downloaded from http://www.mathworks.co.uk/matlabcentral/fileexchange/10889-savefig
% savefig('fig_2a_model_f1a',fig1a,'eps','-r300','-crop')
% savefig('fig_2b_model_f1b',fig1b,'eps','-r300','-crop')
% savefig('fig_2c_model_f2a',fig2a,'eps','-r300','-crop')
% savefig('fig_2d_model_f2b',fig2b,'eps','-r300','-crop')
% savefig('fig_2e_model_f3a',fig3a,'eps','-r300','-crop')
% savefig('fig_2f_model_f3b',fig3b,'eps','-r300','-crop')
% savefig('fig_2g_model_f4a',fig4a,'eps','-r300','-crop')
% savefig('fig_2h_model_f4b',fig4b,'eps','-r300','-crop')
% 
% %% Convert to tiff using ImageMagick
% % ImageMagick can be downloaded from http://www.imagemagick.org
% system('convert -density 300 -alpha off fig_2a_model_f1a.eps fig_2a_model_f1a.tiff');
% system('convert -density 300 -alpha off fig_2b_model_f1b.eps fig_2b_model_f1b.tiff');
% system('convert -density 300 -alpha off fig_2c_model_f2a.eps fig_2c_model_f2a.tiff');
% system('convert -density 300 -alpha off fig_2d_model_f2b.eps fig_2d_model_f2b.tiff');
% system('convert -density 300 -alpha off fig_2e_model_f3a.eps fig_2e_model_f3a.tiff');
% system('convert -density 300 -alpha off fig_2f_model_f3b.eps fig_2f_model_f3b.tiff');
% system('convert -density 300 -alpha off fig_2g_model_f4a.eps fig_2g_model_f4a.tiff');
% system('convert -density 300 -alpha off fig_2h_model_f4b.eps fig_2h_model_f4b.tiff');
